@@ -1,5 +1,5 @@
 /***************************************************************************************************
- * Copyright (c) 2023 - 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2023 - 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,217 +30,183 @@
  **************************************************************************************************/
 #pragma once
 
-#include <tuple>
-#include <utility>
-
 #include <cute/config.hpp>
 #include <cute/util/type_traits.hpp>
-
 #include <cute/numeric/integral_constant.hpp>  // cute::true_type, cute::false_type
+#include <cute/numeric/integer_sequence.hpp>
+
+#include <cute/container/cuda_types.hpp>
+#include <cute/container/type_list.hpp>
 //#include <cute/container/array.hpp>            // Advanced optimizations
 
-#if 0
-//
-// Use of agency::tuple is functional, but is over-engineered for our purposes...
-//   This tends to result in slow compilation times and unintentionally propagated cvref types
-//
-
-#include <agency/tuple.hpp>
-
-namespace cute
-{
-
-using agency::tuple;
-
-using agency::make_tuple;
-using agency::tuple_cat;
-
-} // end namespace cute
-#endif
-
-// cute::tuple is like std::tuple, with two differences.
+// cute::tuple is like std::tuple, with differences:
 //
 // 1. It works on both host and device.
 // 2. Its template arguments must be semiregular types.
+// 3. It is always a standard-layout type if all of its template arguments are standard-layout types.
+// 4. It is always an empty type if all of its template arguments are empty types.
 //
 // Semiregular types are default constructible and copyable.
 // They include "value types" like int or float,
 // but do _not_ include references like int& or float&.
 // (See std::tie for an example of a tuple of references.)
 //
-// This is simplified over the implementation in std:: and agency:: by ignoring much of
-//    the conversion SFINAE, special overloading, and avoiding cvref template types.
-//    Furthermore, the empty base optimization (EBO) is MORE aggressive by avoiding
-//    construction calls, and ignoring any need for unique element addresses.
+// Standard-layout types preserve ABI across host-device boundaries. They are safe to use as device kernel parameters.
+// The standard-layout requirement prevents a more common EBO-based implemented of cute::tuple.
 //
-// Over the agency::tuple implementation, this appears to accelerate compilation times by over 3x.
+// The cute::tuple is also simplified over the implementations in std::, cuda::std::, and thrust:: by ignoring much of
+// the conversion SFINAE, special overloading, and avoiding cvref template types.
+//
+// Over standard-conforming tuple implementations, this appears to accelerate compilation times by over 3x.
 
 namespace cute
 {
 
-namespace detail
-{
-
-// EBO stands for "empty base optimization."
-// We use this technique to ensure that cute::tuple
-// doesn't need to waste space storing any template arguments
-// of cute::tuple that have no data (like integral_constant).
-// Otherwise, cute::tuple would need to spend at least 1 byte
-// for each of its template arguments.
-//
-// EBO always "holds" a single value of type T.
-// N is like an array index that TupleBase uses
-// to access the desired tuple element.
-template <std::size_t N, class T, bool IsEmpty = std::is_empty<T>::value>
-struct EBO;
-
-// Specialization for types T that have no data;
-// the "static tuple leaf."  Valid T here include
-// integral_constant<U, Value>, Int<Value>,
-// and any other semiregular type
-// for which std::is_empty_v<T> is true.
-template <std::size_t N, class T>
-struct EBO<N, T, true>
-{
-  CUTE_HOST_DEVICE constexpr
-  EBO() {}
-
-  CUTE_HOST_DEVICE constexpr
-  EBO(T const&) {}
-};
-
-template <std::size_t N, class T>
-CUTE_HOST_DEVICE constexpr T getv(EBO<N, T, true> const&)
-{ return {}; }
-
-// Specialization for types T that are not empty;
-// the "dynamic tuple leaf."  Valid T here include int,
-// any other integral or floating-point type,
-// or any semiregular type for which std::is_empty_v<T> is false.
-template <std::size_t N, class T>
-struct EBO<N, T, false>
-{
-  CUTE_HOST_DEVICE constexpr
-  EBO() : t_{} {}
-
-  template <class U>
-  CUTE_HOST_DEVICE constexpr
-  EBO(U const& u) : t_{u} {}
-
-  T t_;
-};
-
-template <std::size_t N, class T>
-CUTE_HOST_DEVICE constexpr T const& getv(EBO<N, T, false> const& x)
-{ return x.t_; }
-
-template <std::size_t N, class T>
-CUTE_HOST_DEVICE constexpr T& getv(EBO<N, T, false>& x)
-{ return x.t_; }
-
-template <std::size_t N, class T>
-CUTE_HOST_DEVICE constexpr T&& getv(EBO<N, T, false>&& x)
-{ return static_cast<T&&>(x.t_); }
-
-template <class IdxSeq, class... T>
-struct TupleBase;
-
-// Base class of cute::tuple.
-// It inherits from EBO<i, t> for each (i, t) in (I..., T...).
-// The actual storage (for nonempty t) lives in the base classes.
-// index_sequence is a way to wrap up a sequence of zero or more
-// compile-time integer values in a single type.
-// We only ever use index_sequence<0, 1, ..., sizeof...(T)> in practice,
-// as the type alias TupleBase below indicates.
-template <std::size_t... I, class... T>
-struct TupleBase<std::index_sequence<I...>, T...>
-    : EBO<I,T>...
-{
-  CUTE_HOST_DEVICE constexpr
-  TupleBase() {}
-
-  template <class... U>
-  CUTE_HOST_DEVICE constexpr explicit
-  TupleBase(U const&... u)
-      : EBO<I,T>(u)... {}
-
-  template <class... U>
-  CUTE_HOST_DEVICE constexpr
-  TupleBase(TupleBase<std::index_sequence<I...>, U...> const& u)
-      : EBO<I,T>(getv(static_cast<EBO<I,U> const&>(u)))... {}
-};
-
-} // end namespace detail
-
-// make_index_sequence<K> returns index_sequence<0, 1, ..., K-1>.
 template <class... T>
-using TupleBase = detail::TupleBase<std::make_index_sequence<sizeof...(T)>, T...>;
+struct tuple;
+
+namespace eso
+{
+
+// ESO stands for "empty structure optimization."
+// We use this technique to ensure that cute::tuple doesn't waste space
+// storing template arguments that have no data (like integral_constant).
+// Empty types in the template argument list are not even constructed,
+// and do not have unique element addresses. Calling `get`
+// constructs and returns an instance of an empty type on demand.
+
+template <bool IsFirstEmpty, bool IsRestEmpty, class... T>
+struct ESO;
+
+template <class First, class... Rest>
+static constexpr bool is_first_empty_v = cute::is_empty<First>::value;
+template <class First, class... Rest>
+static constexpr bool is_rest_empty_v  = (cute::is_empty<Rest>::value && ...);
 
-// This is the actual cute::tuple class.
-// The storage (if any) lives in TupleBase's EBO base classes.
 template <class... T>
-struct tuple : TupleBase<T...>
+using ESO_t = ESO<is_first_empty_v<T...>, is_rest_empty_v<T...>, T...>;
+
+// Empty First and Empty Rest...
+template <class First, class... Rest>
+struct ESO<true, true, First, Rest...> {
+  CUTE_HOST_DEVICE constexpr
+  ESO() {}
+
+  CUTE_HOST_DEVICE constexpr
+  ESO(First const&, Rest const&...) {}
+};
+
+// NonEmpty First and Empty Rest...
+template <class First, class... Rest>
+struct ESO<false, true, First, Rest...> {
+  CUTE_HOST_DEVICE constexpr
+  ESO() : first_{} {}
+
+  CUTE_HOST_DEVICE constexpr
+  ESO(First const& first, Rest const&...) : first_{first} {}
+
+  First first_;
+};
+
+// Empty First and NonEmpty Rest...
+template <class First, class... Rest>
+struct ESO<true, false, First, Rest...> {
+  CUTE_HOST_DEVICE constexpr
+  ESO() : rest_{} {}
+
+  CUTE_HOST_DEVICE constexpr
+  ESO(First const&, Rest const&... rest) : rest_{rest...} {}
+
+  ESO_t<Rest...> rest_;
+};
+
+// NonEmpty T and NonEmpty Rest...
+template <class First, class... Rest>
+struct ESO<false, false, First, Rest...> {
+  CUTE_HOST_DEVICE constexpr
+  ESO() : first_{}, rest_{} {}
+
+  CUTE_HOST_DEVICE constexpr
+  ESO(First const& first, Rest const&... rest) : first_{first}, rest_{rest...} {}
+
+  First first_;
+  ESO_t<Rest...> rest_;
+};
+
+// Get Nth value from ESO
+template <class R, size_t N, class S>
+CUTE_HOST_DEVICE constexpr
+R
+getr(S&& s) noexcept
+{
+  if constexpr (N == 0) {
+    return static_cast<S&&>(s).first_;
+  } else {
+    return getr<R,N-1>(static_cast<S&&>(s).rest_);
+  }
+  CUTE_GCC_UNREACHABLE;
+}
+
+// Compilers disagree on decltype(auto), so these implementations avoid it at cost
+template <size_t N, bool F, bool R, class... T>
+CUTE_HOST_DEVICE constexpr
+cute::conditional_t<cute::is_empty<cute::tuple_element_t<N, cute::tuple<T...>>>::value,
+                    cute::tuple_element_t<N, cute::tuple<T...>>,
+                    cute::tuple_element_t<N, cute::tuple<T...>> const&>
+getv_cr(ESO<F, R, T...> const& s) noexcept
+{
+  if constexpr (cute::is_empty<cute::tuple_element_t<N, cute::tuple<T...>>>::value) {
+    return {};
+  } else {
+    return getr<cute::tuple_element_t<N, cute::tuple<T...>> const&, N>(s);
+  }
+  CUTE_GCC_UNREACHABLE;
+}
+
+template <size_t N, bool F, bool R, class... T>
+CUTE_HOST_DEVICE constexpr
+cute::conditional_t<cute::is_empty<cute::tuple_element_t<N, cute::tuple<T...>>>::value,
+                    cute::tuple_element_t<N, cute::tuple<T...>>,
+                    cute::tuple_element_t<N, cute::tuple<T...>> &>
+getv_r(ESO<F, R, T...>& s) noexcept
+{
+  if constexpr (cute::is_empty<cute::tuple_element_t<N, cute::tuple<T...>>>::value) {
+    return {};
+  } else {
+    return getr<cute::tuple_element_t<N, cute::tuple<T...>> &, N>(s);
+  }
+  CUTE_GCC_UNREACHABLE;
+}
+
+template <size_t N, bool F, bool R, class... T>
+CUTE_HOST_DEVICE constexpr
+cute::conditional_t<cute::is_empty<cute::tuple_element_t<N, cute::tuple<T...>>>::value,
+                    cute::tuple_element_t<N, cute::tuple<T...>>,
+                    cute::tuple_element_t<N, cute::tuple<T...>> &&>
+getv_rr(ESO<F, R, T...>&& s) noexcept
+{
+  if constexpr (cute::is_empty<cute::tuple_element_t<N, cute::tuple<T...>>>::value) {
+    return {};
+  } else {
+    return getr<cute::tuple_element_t<N, cute::tuple<T...>> &&, N>(static_cast<ESO<F, R, T...>&&>(s));
+  }
+  CUTE_GCC_UNREACHABLE;
+}
+
+} // end namespace eso
+
+template <class... T>
+struct tuple : eso::ESO_t<T...>
 {
   CUTE_HOST_DEVICE constexpr
   tuple() {}
 
-  template <class... U>
   CUTE_HOST_DEVICE constexpr
-  tuple(U const&... u) : TupleBase<T...>(u...) {}
-
-  template <class... U>
-  CUTE_HOST_DEVICE constexpr
-  tuple(tuple<U...> const& u)
-      : TupleBase<T...>(static_cast<TupleBase<U...> const&>(u)) {}
+  tuple(T const&... t) : eso::ESO_t<T...>(t...) {}
 };
 
-//
-// get for cute::tuple (just like std::get for std::tuple)
-//
-
-template <std::size_t I, class... T>
-CUTE_HOST_DEVICE constexpr
-decltype(auto)
-get(tuple<T...> const& t) noexcept
-{
-  static_assert(I < sizeof...(T), "Index out of range");
-  return detail::getv<I>(t);
-}
-
-template <std::size_t I, class... T>
-CUTE_HOST_DEVICE constexpr
-decltype(auto)
-get(tuple<T...>& t) noexcept
-{
-  static_assert(I < sizeof...(T), "Index out of range");
-  return detail::getv<I>(t);
-}
-
-template <std::size_t I, class... T>
-CUTE_HOST_DEVICE constexpr
-decltype(auto)
-get(tuple<T...>&& t) noexcept
-{
-  static_assert(I < sizeof...(T), "Index out of range");
-  return detail::getv<I>(static_cast<tuple<T...>&&>(t));
-}
-
-//
-// Custom is_tuple trait simply checks the existence of std::tuple_size
-//      and assumes std::get<I>(.), std::tuple_element<I,.>
-//
-namespace detail {
-
-template <class T>
-std::integral_constant<bool, std::tuple_size<T>::value >= 0> has_tuple_size(int);
-
-template <class T>
-std::false_type has_tuple_size(...);
-
-} // end namespace detail
-
-template <class T>
-struct is_tuple : decltype(detail::has_tuple_size<T>(0)) {};
+template <>
+struct tuple<> {};
 
 //
 // make_tuple (value-based implementation)
@@ -254,6 +220,62 @@ make_tuple(T const&... t)
   return {t...};
 }
 
+// Returns the element in the ith position of the tuple
+template <size_t I, class... T>
+CUTE_HOST_DEVICE constexpr
+decltype(auto)
+get(tuple<T...> const& t) noexcept
+{
+  static_assert(I < sizeof...(T), "Index out of range");
+  return eso::getv_cr<I>(t);
+}
+
+template <size_t I, class... T>
+CUTE_HOST_DEVICE constexpr
+decltype(auto)
+get(tuple<T...>& t) noexcept
+{
+  static_assert(I < sizeof...(T), "Index out of range");
+  return eso::getv_r<I>(t);
+}
+
+template <size_t I, class... T>
+CUTE_HOST_DEVICE constexpr
+decltype(auto)
+get(tuple<T...>&& t) noexcept
+{
+  static_assert(I < sizeof...(T), "Index out of range");
+  return eso::getv_rr<I>(static_cast<eso::ESO_t<T...>&&>(t));
+}
+
+// Returns the first position of type X (as a static integer) in the tuple
+// type's argument list.
+template <class X, class... T>
+CUTE_HOST_DEVICE constexpr
+auto
+find(tuple<T...> const&) noexcept
+{
+  return cute::C<find_true_v<cute::is_same_v<X,T>...>>{};
+}
+
+//
+// Custom is_tuple trait simply checks the existence of tuple_size
+//      and assumes get<I>(.), tuple_element<I,.>
+//
+namespace detail {
+
+template <class T>
+auto has_tuple_size( T*) -> bool_constant<(0 <= tuple_size<T>::value)>;
+auto has_tuple_size(...) -> false_type;
+
+} // end namespace detail
+
+template <class T>
+struct is_tuple : decltype(detail::has_tuple_size((T*)0)) {};
+
+template <class T>
+static constexpr bool is_tuple_v = cute::is_tuple<T>::value;
+
 //
 // tuple_cat concatenates multiple cute::tuple into a single cute::tuple,
 // just like std::tuple_cat for std::tuple.
@@ -265,11 +287,11 @@ make_tuple(T const&... t)
 namespace detail {
 
 template <class T0, class T1,
-          std::size_t... I0, std::size_t... I1>
+          size_t... I0, size_t... I1>
 CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1,
-          std::index_sequence<I0...>, std::index_sequence<I1...>)
+          index_sequence<I0...>, index_sequence<I1...>)
 {
   return cute::make_tuple(get<I0>(t0)..., get<I1>(t1)...);
 }
@@ -298,8 +320,8 @@ auto
 tuple_cat(T0 const& t0, T1 const& t1)
 {
   return detail::tuple_cat(t0, t1,
-                           std::make_index_sequence<std::tuple_size<T0>::value>{},
-                           std::make_index_sequence<std::tuple_size<T1>::value>{});
+                           make_index_sequence<tuple_size<T0>::value>{},
+                           make_index_sequence<tuple_size<T1>::value>{});
 }
 
 template <class T0, class T1, class T2, class... Ts>
@@ -317,45 +339,53 @@ tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, Ts const&... ts)
 namespace detail {
 
 template <class T0, class T1,
-          std::size_t... I0, std::size_t... I1>
+          size_t... I0, size_t... I1>
 CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1,
-          std::index_sequence<I0...>, std::index_sequence<I1...>)
+          index_sequence<I0...>, index_sequence<I1...>)
 {
   return cute::make_tuple(get<I0>(t0)..., get<I1>(t1)...);
 }
 
 template <class T0, class T1, class T2,
-          std::size_t... I0, std::size_t... I1, std::size_t... I2>
+          size_t... I0, size_t... I1, size_t... I2>
 CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2,
-          std::index_sequence<I0...>, std::index_sequence<I1...>, std::index_sequence<I2...>)
+          index_sequence<I0...>, index_sequence<I1...>, index_sequence<I2...>)
 {
   return cute::make_tuple(get<I0>(t0)..., get<I1>(t1)..., get<I2>(t2)...);
 }
 
 template <class T0, class T1, class T2, class T3,
-          std::size_t... I0, std::size_t... I1, std::size_t... I2, std::size_t... I3>
+          size_t... I0, size_t... I1, size_t... I2, size_t... I3>
 CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, T3 const& t3,
-          std::index_sequence<I0...>, std::index_sequence<I1...>, std::index_sequence<I2...>, std::index_sequence<I3...>)
+          index_sequence<I0...>, index_sequence<I1...>, index_sequence<I2...>, index_sequence<I3...>)
 {
   return cute::make_tuple(get<I0>(t0)..., get<I1>(t1)..., get<I2>(t2)..., get<I3>(t3)...);
 }
 
 template <class T0, class T1, class T2, class T3, class T4,
-          std::size_t... I0, std::size_t... I1, std::size_t... I2, std::size_t... I3, std::size_t... I4>
+          size_t... I0, size_t... I1, size_t... I2, size_t... I3, size_t... I4>
 CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, T3 const& t3, T4 const& t4,
-          std::index_sequence<I0...>, std::index_sequence<I1...>, std::index_sequence<I2...>, std::index_sequence<I3...>, std::index_sequence<I4...>)
+          index_sequence<I0...>, index_sequence<I1...>, index_sequence<I2...>, index_sequence<I3...>, index_sequence<I4...>)
 {
   return cute::make_tuple(get<I0>(t0)..., get<I1>(t1)..., get<I2>(t2)..., get<I3>(t3)..., get<I4>(t4)...);
 }
 
+template <class T0, class T1>
+struct tuple_cat_static;
+
+template <class... T0s, class... T1s>
+struct tuple_cat_static<tuple<T0s...>, tuple<T1s...>> {
+  using type = tuple<T0s..., T1s...>;
+};
+
 } // end namespace detail
 
 CUTE_HOST_DEVICE constexpr
@@ -379,9 +409,16 @@ CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1)
 {
-  return detail::tuple_cat(t0, t1,
-                           std::make_index_sequence<std::tuple_size<T0>::value>{},
-                           std::make_index_sequence<std::tuple_size<T1>::value>{});
+  if constexpr (is_static<T0>::value && is_static<T1>::value &&
+		is_tuple<T0>::value && is_tuple<T1>::value) {
+    return typename detail::tuple_cat_static<T0, T1>::type{};
+  } else {
+    return detail::tuple_cat(t0, t1,
+                           make_index_sequence<tuple_size<T0>::value>{},
+                           make_index_sequence<tuple_size<T1>::value>{});
+  }
+
+  CUTE_GCC_UNREACHABLE;
 }
 
 template <class T0, class T1, class T2>
@@ -390,9 +427,9 @@ auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2)
 {
   return detail::tuple_cat(t0, t1, t2,
-                           std::make_index_sequence<std::tuple_size<T0>::value>{},
-                           std::make_index_sequence<std::tuple_size<T1>::value>{},
-                           std::make_index_sequence<std::tuple_size<T2>::value>{});
+                           make_index_sequence<tuple_size<T0>::value>{},
+                           make_index_sequence<tuple_size<T1>::value>{},
+                           make_index_sequence<tuple_size<T2>::value>{});
 }
 
 template <class T0, class T1, class T2, class T3>
@@ -401,10 +438,10 @@ auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, T3 const& t3)
 {
   return detail::tuple_cat(t0, t1, t2, t3,
-                           std::make_index_sequence<std::tuple_size<T0>::value>{},
-                           std::make_index_sequence<std::tuple_size<T1>::value>{},
-                           std::make_index_sequence<std::tuple_size<T2>::value>{},
-                           std::make_index_sequence<std::tuple_size<T3>::value>{});
+                           make_index_sequence<tuple_size<T0>::value>{},
+                           make_index_sequence<tuple_size<T1>::value>{},
+                           make_index_sequence<tuple_size<T2>::value>{},
+                           make_index_sequence<tuple_size<T3>::value>{});
 }
 
 template <class T0, class T1, class T2, class T3, class T4>
@@ -413,11 +450,11 @@ auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, T3 const& t3, T4 const& t4)
 {
   return detail::tuple_cat(t0, t1, t2, t3, t4,
-                           std::make_index_sequence<std::tuple_size<T0>::value>{},
-                           std::make_index_sequence<std::tuple_size<T1>::value>{},
-                           std::make_index_sequence<std::tuple_size<T2>::value>{},
-                           std::make_index_sequence<std::tuple_size<T3>::value>{},
-                           std::make_index_sequence<std::tuple_size<T4>::value>{});
+                           make_index_sequence<tuple_size<T0>::value>{},
+                           make_index_sequence<tuple_size<T1>::value>{},
+                           make_index_sequence<tuple_size<T2>::value>{},
+                           make_index_sequence<tuple_size<T3>::value>{},
+                           make_index_sequence<tuple_size<T4>::value>{});
 }
 
 template <class T0, class T1, class T2, class T3, class T4, class T5, class... Ts>
@@ -425,7 +462,7 @@ CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, T3 const& t3, T4 const& t4, T5 const& t5, Ts const&... ts)
 {
-  return cute::tuple_cat(cute::tuple_cat(t0,t1,t2,t3,t4), t5, ts...);
+  return cute::tuple_cat(cute::tuple_cat(t0,t1,t2,t3,t4), cute::tuple_cat(t5, ts...));
 }
 #endif
 
@@ -434,24 +471,24 @@ tuple_cat(T0 const& t0, T1 const& t1, T2 const& t2, T3 const& t3, T4 const& t4,
 
 namespace detail {
 
-template <std::size_t... Ns>
+template <size_t... Ns>
 struct tuple_cat_helper
 {
-  static constexpr cute::array<std::size_t,sizeof...(Ns)> ns = {Ns...};
+  static constexpr cute::array<size_t,sizeof...(Ns)> ns = {Ns...};
 
-  static constexpr std::size_t total_size() {
-    std::size_t sum = 0;
-    for (std::size_t n : ns) sum += n;
+  static constexpr size_t total_size() {
+    size_t sum = 0;
+    for (size_t n : ns) sum += n;
     return sum;
   }
-  static constexpr std::size_t total_size_ = total_size();
+  static constexpr size_t total_size_ = total_size();
 
   static constexpr auto values() {
-    cute::array<std::size_t[2],total_size_> outer_inner = {};
+    cute::array<size_t[2],total_size_> outer_inner = {};
 
-    std::size_t idx = 0;
-    for (std::size_t i = 0; i < ns.size(); ++i) {
-      for (std::size_t j = 0; j < ns[i]; ++j, ++idx) {
+    size_t idx = 0;
+    for (size_t i = 0; i < ns.size(); ++i) {
+      for (size_t j = 0; j < ns[i]; ++j, ++idx) {
         outer_inner[idx][0] = i;
         outer_inner[idx][1] = j;
       }
@@ -460,23 +497,23 @@ struct tuple_cat_helper
   }
   static constexpr auto outer_inner_ = values();
 
-  using total_sequence = std::make_index_sequence<total_size_>;
+  using total_sequence = make_index_sequence<total_size_>;
 };
 
-template <class Helper, class Tuple, std::size_t... I>
+template <class Helper, class Tuple, size_t... I>
 CUTE_HOST_DEVICE constexpr
 auto
-tuple_cat(Tuple const& t, std::index_sequence<I...>)
+tuple_cat(Tuple const& t, index_sequence<I...>)
 {
   return cute::make_tuple(get<Helper::outer_inner_[I][1]>(get<Helper::outer_inner_[I][0]>(t))...);
 }
 
 template <class T0, class T1,
-          std::size_t... I0, std::size_t... I1>
+          size_t... I0, size_t... I1>
 CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(T0 const& t0, T1 const& t1,
-          std::index_sequence<I0...>, std::index_sequence<I1...>)
+          index_sequence<I0...>, index_sequence<I1...>)
 {
   return cute::make_tuple(get<I0>(t0)..., get<I1>(t1)...);
 }
@@ -505,8 +542,8 @@ auto
 tuple_cat(T0 const& t0, T1 const& t1)
 {
   return detail::tuple_cat(t0, t1,
-                           std::make_index_sequence<std::tuple_size<T0>::value>{},
-                           std::make_index_sequence<std::tuple_size<T1>::value>{});
+                           make_index_sequence<tuple_size<T0>::value>{},
+                           make_index_sequence<tuple_size<T1>::value>{});
 }
 
 template <class... Tuples>
@@ -514,8 +551,8 @@ CUTE_HOST_DEVICE constexpr
 auto
 tuple_cat(Tuples const&... ts)
 {
-  using Helper = detail::tuple_cat_helper<std::tuple_size<Tuples>::value...>;
-  return detail::tuple_cat<Helper>(make_tuple(ts...), typename Helper::total_sequence{});
+  using Helper = detail::tuple_cat_helper<tuple_size<Tuples>::value...>;
+  return detail::tuple_cat<Helper>(cute::make_tuple(ts...), typename Helper::total_sequence{});
 }
 #endif
 
@@ -525,20 +562,12 @@ tuple_cat(Tuples const&... ts)
 
 namespace detail {
 
-template <std::size_t I, class TupleA, class TupleB>
+template <class TupleA, class TupleB, size_t... I>
 CUTE_HOST_DEVICE constexpr
 auto
-equal_impl(TupleA const& a, TupleB const& b)
+equal_impl(TupleA const& a, TupleB const& b, index_sequence<I...>)
 {
-  if constexpr (I == std::tuple_size<TupleA>::value) {
-    return cute::true_type{};   // Terminal: TupleA is exhausted
-  } else if constexpr (I == std::tuple_size<TupleB>::value) {
-    return cute::false_type{};  // Terminal: TupleA is not exhausted, TupleB is exhausted
-  } else {
-    return (get<I>(a) == get<I>(b)) && equal_impl<I+1>(a,b);
-  }
-
-  CUTE_GCC_UNREACHABLE;
+  return (cute::true_type{} && ... && (get<I>(a) == get<I>(b)));
 }
 
 } // end namespace detail
@@ -549,7 +578,13 @@ CUTE_HOST_DEVICE constexpr
 auto
 operator==(TupleT const& t, TupleU const& u)
 {
-  return detail::equal_impl<0>(t, u);
+  if constexpr (tuple_size<TupleT>::value == tuple_size<TupleU>::value) {
+    return detail::equal_impl(t, u, make_index_sequence<tuple_size<TupleT>::value>{});
+  } else {
+    return cute::false_type{};
+  }
+
+  CUTE_GCC_UNREACHABLE;
 }
 
 template <class TupleT, class TupleU,
@@ -596,42 +631,36 @@ operator!=(TupleT const& t, TupleU const& u)
 // That said, see int_tuple for more explicitly named common comparison ops.
 //
 
-//
-// Shortcuts
-//
-
-//using std::get;
-using std::tuple_size;
-using std::tuple_element;
-using std::tuple_element_t;
-
 //
 // Display utilities
 //
 
 namespace detail {
 
-template <class Tuple, std::size_t... Is>
-CUTE_HOST_DEVICE void print_tuple(Tuple const& t,
-                                  std::index_sequence<Is...>, char s = '(', char e = ')')
+template <class Tuple, size_t... Is>
+CUTE_HOST_DEVICE void print_tuple(Tuple const& t, index_sequence<Is...>, char s = '(', char e = ')')
 {
-  using eat = int[];
   using cute::print;
-  (void) eat {(print(s), 0),
-              (print(Is == 0 ? "" : ","), print(get<Is>(t)), 0)...,
-              (print(e), 0)};
+  if (sizeof...(Is) == 0) {
+    print(s);
+  } else {
+    ((void(print(Is == 0 ? s : ',')), void(print(get<Is>(t)))), ...);
+  }
+  print(e);
 }
 
+#if !defined(__CUDACC_RTC__)
 template <class Tuple, std::size_t... Is>
-CUTE_HOST std::ostream& print_tuple_os(std::ostream& os, Tuple const& t,
-                                       std::index_sequence<Is...>, char s = '(', char e = ')')
+CUTE_HOST std::ostream& print_tuple_os(std::ostream& os, Tuple const& t, index_sequence<Is...>, char s = '(', char e = ')')
 {
-  using eat = int[];
-  (void) eat {(void(os << s), 0),
-              (void(os << (Is == 0 ? "" : ",") << get<Is>(t)), 0)...,
-              (void(os << e), 0)};
-  return os;
+  if (sizeof...(Is) == 0) {
+    os << s;
+  } else {
+    (void(os << (Is == 0 ? s : ',') << get<Is>(t)), ...);
+  }
+  return os << e;
 }
+#endif // !defined(__CUDACC_RTC__)
 
 } // end namespace detail
 
@@ -639,33 +668,56 @@ template <class Tuple,
           __CUTE_REQUIRES(is_tuple<Tuple>::value)>
 CUTE_HOST_DEVICE void print(Tuple const& t)
 {
-  return detail::print_tuple(t, std::make_index_sequence<std::tuple_size<Tuple>::value>{});
+  return detail::print_tuple(t, make_index_sequence<tuple_size<Tuple>::value>{});
 }
 
+#if !defined(__CUDACC_RTC__)
 template <class Tuple,
           __CUTE_REQUIRES(is_tuple<Tuple>::value)>
 CUTE_HOST std::ostream& operator<<(std::ostream& os, Tuple const& t)
 {
-  return detail::print_tuple_os(os, t, std::make_index_sequence<std::tuple_size<Tuple>::value>{});
+  return detail::print_tuple_os(os, t, make_index_sequence<tuple_size<Tuple>::value>{});
 }
+#endif // !defined(__CUDACC_RTC__)
 
 } // end namespace cute
 
-//
-// std:: compatability
-//
-
-namespace std
+namespace CUTE_STL_NAMESPACE
 {
 
 template <class... T>
 struct tuple_size<cute::tuple<T...>>
-    : std::integral_constant<std::size_t, sizeof...(T)>
+    : CUTE_STL_NAMESPACE::integral_constant<size_t, sizeof...(T)>
 {};
 
-template <std::size_t I, class... T>
+template <size_t I, class... T>
 struct tuple_element<I, cute::tuple<T...>>
-    : std::tuple_element<I, std::tuple<T...>>
+    : CUTE_STL_NAMESPACE::tuple_element<I, CUTE_STL_NAMESPACE::tuple<T...>>
 {};
 
-} // end std
+} // end namespace CUTE_STL_NAMESPACE
+
+#ifdef CUTE_STL_NAMESPACE_IS_CUDA_STD
+namespace std
+{
+
+#if defined(__CUDACC_RTC__)
+template <class... _Tp>
+struct tuple_size;
+
+template <size_t _Ip, class... _Tp>
+struct tuple_element;
+#endif
+
+template <class... T>
+struct tuple_size<cute::tuple<T...>>
+    : CUTE_STL_NAMESPACE::integral_constant<size_t, sizeof...(T)>
+{};
+
+template <size_t I, class... T>
+struct tuple_element<I, cute::tuple<T...>>
+    : CUTE_STL_NAMESPACE::tuple_element<I, CUTE_STL_NAMESPACE::tuple<T...>>
+{};
+
+} // end namespace std
+#endif // CUTE_STL_NAMESPACE_IS_CUDA_STD
