@@ -1,5 +1,5 @@
 /***************************************************************************************************
- * Copyright (c) 2017 - 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2017 - 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,10 @@
 namespace cutlass {
 namespace arch {
 
-#if defined(__NVCC__) || (defined(__clang__) && defined(__CUDA__))
+constexpr int sm100_smem_capacity_bytes = 232448;  
+constexpr int sm120_smem_capacity_bytes = 101376;
+
+#if defined(__NVCC__) || defined(__CUDACC_RTC__) || (defined(__clang__) && defined(__CUDA__))
 
 /// Computes laneId within a warp
 CUTLASS_DEVICE
@@ -86,11 +89,26 @@ struct Sm80 {
 struct Sm86 {
   static int const kMinComputeCapability = 86;
 };
-
+struct Sm89 {
+  static int const kMinComputeCapability = 89;
+};
 struct Sm90 {
   static int const kMinComputeCapability = 90; 
 };
 
+
+struct Sm100 {
+  static int const kMinComputeCapability = 100; 
+};
+
+struct Sm101 {
+  static int const kMinComputeCapability = 101; 
+};
+
+struct Sm120 {
+  static int const kMinComputeCapability = 120;
+};
+
 /// Triggers a breakpoint on the device
 CUTLASS_DEVICE
 void device_breakpoint() {
