@@ -40,18 +40,21 @@
 #include "cutlass/half.h"
 #include "cutlass/numeric_types.h"
 
-namespace cutlass {
+namespace cutlass
+{
 
 // This converter is meant to be used with data interleaved in a 32-bit register where the even elements are in the low
 // bits and the odd elemeents are in the high bits of the register. In addition, it assumes elements were originally
 // signed and had a bias of 2**(b-1) added (where b is the number of bits in the type) to make all numbers unsigned.
 // This converter will uninterleave the data and subtract the bias while converting to the result type.
-template<typename T, typename S, int N>
-struct FastInterleavedAndBiasedNumericArrayConverter {
+template <typename T, typename S, int N>
+struct FastInterleavedAndBiasedNumericArrayConverter
+{
 };
 
-template<>
-struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, 4> {
+template <>
+struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, 4>
+{
     using result_type = Array<half_t, 4>;
     using source_type = Array<uint8_t, 4>;
 
@@ -60,11 +63,11 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, 4> {
     {
         result_type result;
 
-        uint32_t*      h   = reinterpret_cast<uint32_t*>(&result);
+        uint32_t* h = reinterpret_cast<uint32_t*>(&result);
         uint32_t const i8s = reinterpret_cast<uint32_t const&>(source);
 
-        static constexpr uint32_t mask_for_elt_01     = 0x5250;
-        static constexpr uint32_t mask_for_elt_23     = 0x5351;
+        static constexpr uint32_t mask_for_elt_01 = 0x5250;
+        static constexpr uint32_t mask_for_elt_23 = 0x5351;
         static constexpr uint32_t start_byte_for_fp16 = 0x64646464;
         asm volatile("prmt.b32 %0,%1,%2,%3;\n" : "=r"(h[0]) : "r"(i8s), "n"(start_byte_for_fp16), "n"(mask_for_elt_01));
         asm volatile("prmt.b32 %0,%1,%2,%3;\n" : "=r"(h[1]) : "r"(i8s), "n"(start_byte_for_fp16), "n"(mask_for_elt_23));
@@ -84,8 +87,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, 4> {
     }
 };
 
-template<int N>
-struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, N> {
+template <int N>
+struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, N>
+{
     static constexpr int VEC_WIDTH = 4;
     static_assert(!(N % VEC_WIDTH), "N must be multiple of 4.");
 
@@ -104,11 +108,12 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, N> {
         using vec_result = Array<scalar_result_type, VEC_WIDTH>;
         using vec_source = Array<scalar_source_type, VEC_WIDTH>;
 
-        vec_result*       result_ptr = reinterpret_cast<vec_result*>(&result);
+        vec_result* result_ptr = reinterpret_cast<vec_result*>(&result);
         vec_source const* source_ptr = reinterpret_cast<vec_source const*>(&source);
 
         CUTLASS_PRAGMA_UNROLL
-        for (int i = 0; i < N / VEC_WIDTH; ++i) {
+        for (int i = 0; i < N / VEC_WIDTH; ++i)
+        {
             result_ptr[i] = convert_vector_(source_ptr[i]);
         }
 
@@ -122,8 +127,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint8_t, N> {
     }
 };
 
-template<>
-struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, 4> {
+template <>
+struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, 4>
+{
     using result_type = Array<bfloat16_t, 4>;
     using source_type = Array<uint8_t, 4>;
 
@@ -133,35 +139,37 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, 4> {
         result_type result;
 #if (defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 800))
 
-        uint32_t*      bf16_result_ptr = reinterpret_cast<uint32_t*>(&result);
-        uint32_t const i8s             = reinterpret_cast<uint32_t const&>(source);
+        uint32_t* bf16_result_ptr = reinterpret_cast<uint32_t*>(&result);
+        uint32_t const i8s = reinterpret_cast<uint32_t const&>(source);
 
         static constexpr uint32_t fp32_base = 0x4B000000;
-        float                     fp32_intermediates[4];
+        float fp32_intermediates[4];
 
         // Construct FP32s, bfloat does not have enough mantissa for IADD trick
         uint32_t* fp32_intermediates_casted = reinterpret_cast<uint32_t*>(fp32_intermediates);
-        fp32_intermediates_casted[0]        = __byte_perm(i8s, fp32_base, 0x7650);
-        fp32_intermediates_casted[1]        = __byte_perm(i8s, fp32_base, 0x7652);
-        fp32_intermediates_casted[2]        = __byte_perm(i8s, fp32_base, 0x7651);
-        fp32_intermediates_casted[3]        = __byte_perm(i8s, fp32_base, 0x7653);
+        fp32_intermediates_casted[0] = __byte_perm(i8s, fp32_base, 0x7650);
+        fp32_intermediates_casted[1] = __byte_perm(i8s, fp32_base, 0x7652);
+        fp32_intermediates_casted[2] = __byte_perm(i8s, fp32_base, 0x7651);
+        fp32_intermediates_casted[3] = __byte_perm(i8s, fp32_base, 0x7653);
 
         // Subtract out fp32_base + 128 to make the unsigned integer signed.
         CUTLASS_PRAGMA_UNROLL
-        for (int ii = 0; ii < 4; ++ii) {
+        for (int ii = 0; ii < 4; ++ii)
+        {
             fp32_intermediates[ii] -= 8388736.f;
         }
 
         // Truncate the fp32 representation and pack up as bfloat16s.
         CUTLASS_PRAGMA_UNROLL
-        for (int ii = 0; ii < 2; ++ii) {
-            bf16_result_ptr[ii] =
-                __byte_perm(fp32_intermediates_casted[2 * ii + 0], fp32_intermediates_casted[2 * ii + 1], 0x7632);
+        for (int ii = 0; ii < 2; ++ii)
+        {
+            bf16_result_ptr[ii]
+                = __byte_perm(fp32_intermediates_casted[2 * ii + 0], fp32_intermediates_casted[2 * ii + 1], 0x7632);
         }
 #else
         // Disable this on architectures older than Ampere since they lack hardware for bf16 mma. If one wishes to use
         // HMMA on older hardware, they should Convert directly to FP16 using FP16 converters.
-        result.clear();  // Suppress compiler warning
+        result.clear(); // Suppress compiler warning
         arch::device_breakpoint();
 #endif
         return result;
@@ -174,8 +182,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, 4> {
     }
 };
 
-template<int N>
-struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, N> {
+template <int N>
+struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, N>
+{
     static constexpr int VEC_WIDTH = 4;
     static_assert(!(N % VEC_WIDTH), "N must be multiple of 4.");
 
@@ -194,11 +203,12 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, N> {
         using vec_result = Array<scalar_result_type, VEC_WIDTH>;
         using vec_source = Array<scalar_source_type, VEC_WIDTH>;
 
-        vec_result*       result_ptr = reinterpret_cast<vec_result*>(&result);
+        vec_result* result_ptr = reinterpret_cast<vec_result*>(&result);
         vec_source const* source_ptr = reinterpret_cast<vec_source const*>(&source);
 
         CUTLASS_PRAGMA_UNROLL
-        for (int i = 0; i < N / VEC_WIDTH; ++i) {
+        for (int i = 0; i < N / VEC_WIDTH; ++i)
+        {
             result_ptr[i] = convert_vector_(source_ptr[i]);
         }
 
@@ -212,8 +222,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint8_t, N> {
     }
 };
 
-template<>
-struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, 8> {
+template <>
+struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, 8>
+{
     using result_type = Array<half_t, 8>;
     using source_type = Array<uint4b_t, 8>;
 
@@ -222,13 +233,13 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, 8> {
     {
         result_type result;
 
-        uint32_t*      h   = reinterpret_cast<uint32_t*>(&result);
+        uint32_t* h = reinterpret_cast<uint32_t*>(&result);
         uint32_t const i4s = reinterpret_cast<uint32_t const&>(source);
 
         // First, we extract the i4s and construct an intermediate fp16 number.
-        static constexpr uint32_t immLut                = (0xf0 & 0xcc) | 0xaa;
-        static constexpr uint32_t BOTTOM_MASK           = 0x000f000f;
-        static constexpr uint32_t TOP_MASK              = 0x00f000f0;
+        static constexpr uint32_t immLut = (0xf0 & 0xcc) | 0xaa;
+        static constexpr uint32_t BOTTOM_MASK = 0x000f000f;
+        static constexpr uint32_t TOP_MASK = 0x00f000f0;
         static constexpr uint32_t I4s_TO_F16s_MAGIC_NUM = 0x64006400;
 
         // Note that the entire sequence only requires 1 shift instruction. This is thanks to the register packing
@@ -286,8 +297,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, 8> {
     }
 };
 
-template<int N>
-struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, N> {
+template <int N>
+struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, N>
+{
     static constexpr int VEC_WIDTH = 8;
     static_assert(!(N % VEC_WIDTH), "N must be multiple of 8.");
 
@@ -306,11 +318,12 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, N> {
         using vec_result = Array<scalar_result_type, VEC_WIDTH>;
         using vec_source = Array<scalar_source_type, VEC_WIDTH>;
 
-        vec_result*       result_ptr = reinterpret_cast<vec_result*>(&result);
+        vec_result* result_ptr = reinterpret_cast<vec_result*>(&result);
         vec_source const* source_ptr = reinterpret_cast<vec_source const*>(&source);
 
         CUTLASS_PRAGMA_UNROLL
-        for (int i = 0; i < N / VEC_WIDTH; ++i) {
+        for (int i = 0; i < N / VEC_WIDTH; ++i)
+        {
             result_ptr[i] = convert_vector_(source_ptr[i]);
         }
 
@@ -324,8 +337,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<half_t, uint4b_t, N> {
     }
 };
 
-template<>
-struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8> {
+template <>
+struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8>
+{
     using result_type = Array<bfloat16_t, 8>;
     using source_type = Array<uint4b_t, 8>;
 
@@ -335,12 +349,12 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8> {
         result_type result;
 #if (defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 800))
 
-        uint32_t*      h          = reinterpret_cast<uint32_t*>(&result);
+        uint32_t* h = reinterpret_cast<uint32_t*>(&result);
         uint32_t const source_i4s = reinterpret_cast<uint32_t const&>(source);
 
         // First, we extract the i4s and construct an intermediate fp16 number.
-        static constexpr uint32_t immLut                 = (0xf0 & 0xcc) | 0xaa;
-        static constexpr uint32_t MASK                   = 0x000f000f;
+        static constexpr uint32_t immLut = (0xf0 & 0xcc) | 0xaa;
+        static constexpr uint32_t MASK = 0x000f000f;
         static constexpr uint32_t I4s_TO_BF16s_MAGIC_NUM = 0x43004300;
 
         // We don't have enough mantissa to remove as much shift overhead as FP16, so we must loop.
@@ -350,7 +364,8 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8> {
                      : "=r"(h[0])
                      : "r"(i4s), "n"(MASK), "n"(I4s_TO_BF16s_MAGIC_NUM), "n"(immLut));
         CUTLASS_PRAGMA_UNROLL
-        for (int ii = 1; ii < result_type::kElements / 2; ++ii) {
+        for (int ii = 1; ii < result_type::kElements / 2; ++ii)
+        {
             i4s >>= sizeof_bits<typename source_type::Element>::value;
             // (i4s & 0x000f000f) | 0x43004300
             asm volatile("lop3.b32 %0, %1, %2, %3, %4;\n"
@@ -360,11 +375,12 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8> {
 
         // This is the BF16 {-136, -136} represented as an integer.
         static constexpr uint32_t BF16_BIAS = 0xC308C308;
-        static constexpr uint32_t BF16_ONE  = 0x3F803F80;
+        static constexpr uint32_t BF16_ONE = 0x3F803F80;
 
         // Finally, we construct the output numbers.
         CUTLASS_PRAGMA_UNROLL
-        for (int ii = 0; ii < result_type::kElements / 2; ++ii) {
+        for (int ii = 0; ii < result_type::kElements / 2; ++ii)
+        {
             // Since this section is for Ampere+, we use bf16 fma to do the bias subtraction
             asm("fma.rn.bf16x2 %0, %1, %2, %3;\n" : "=r"(h[ii]) : "r"(h[ii]), "r"(BF16_ONE), "r"(BF16_BIAS));
         }
@@ -372,7 +388,7 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8> {
         // Disable this on architectures older than Ampere since they lack hardware for bf16 mma. If one wishes to use
         // HMMA on older hardware, they should Convert directly to FP16 using FP16 converters.
         arch::device_breakpoint();
-        result.clear();  // Suppress compiler warning.
+        result.clear(); // Suppress compiler warning.
 #endif
         return result;
     }
@@ -384,8 +400,9 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, 8> {
     }
 };
 
-template<int N>
-struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, N> {
+template <int N>
+struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, N>
+{
     static constexpr int VEC_WIDTH = 8;
     static_assert(!(N % VEC_WIDTH), "N must be multiple of 8.");
 
@@ -404,11 +421,12 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, N> {
         using vec_result = Array<scalar_result_type, VEC_WIDTH>;
         using vec_source = Array<scalar_source_type, VEC_WIDTH>;
 
-        vec_result*       result_ptr = reinterpret_cast<vec_result*>(&result);
+        vec_result* result_ptr = reinterpret_cast<vec_result*>(&result);
         vec_source const* source_ptr = reinterpret_cast<vec_source const*>(&source);
 
         CUTLASS_PRAGMA_UNROLL
-        for (int i = 0; i < N / VEC_WIDTH; ++i) {
+        for (int i = 0; i < N / VEC_WIDTH; ++i)
+        {
             result_ptr[i] = convert_vector_(source_ptr[i]);
         }
 
@@ -424,6 +442,6 @@ struct FastInterleavedAndBiasedNumericArrayConverter<bfloat16_t, uint4b_t, N> {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 
-}  // namespace cutlass
+} // namespace cutlass
 
-/////////////////////////////////////////////////////////////////////////////////////////////////
\ No newline at end of file
+/////////////////////////////////////////////////////////////////////////////////////////////////
