@@ -18,9 +18,11 @@
 
 #include "cuda_utils.h"
 
-namespace fastertransformer {
+namespace fastertransformer
+{
 
-enum class ActivationType {
+enum class ActivationType
+{
     Gelu,
     Relu,
     Silu,
@@ -34,7 +36,21 @@ enum class ActivationType {
 inline bool isGatedActivation(ActivationType activaiton_type)
 {
     return activaiton_type == ActivationType::GeGLU || activaiton_type == ActivationType::ReGLU
-           || activaiton_type == ActivationType::SiGLU;
+        || activaiton_type == ActivationType::SiGLU;
 }
 
-}  // namespace fastertransformer
+inline ActivationType get_activation(const std::string& activation_name)
+{
+    if (activation_name == "identity")
+        return ActivationType::Identity;
+    if (activation_name == "relu")
+        return ActivationType::Relu;
+    if (activation_name == "silu")
+        return ActivationType::Silu;
+    if (activation_name == "gelu")
+        return ActivationType::Gelu;
+    // todo: more
+    return ActivationType::InvalidType;
+}
+
+} // namespace fastertransformer
