@@ -23,7 +23,9 @@ from library import *
 class GemmOperation:
   #
   def __init__(self, gemm_kind, arch, tile_description, A, B, C, element_epilogue, \
-      epilogue_functor = EpilogueFunctor.LinearCombination, swizzling_functor = SwizzlingFunctor.Identity8):
+      epilogue_functor = EpilogueFunctor.LinearCombination, swizzling_functor = SwizzlingFunctor.Identity8, D = None,
+      kernel_schedule = KernelScheduleType.ScheduleAuto, epilogue_schedule = EpilogueScheduleType.ScheduleAuto,
+      tile_scheduler = TileSchedulerType.Default):
 
     self.prefix = "3x" if gemm_kind == GemmKind.Universal3x else ""
     self.operation_kind = OperationKind.Gemm
@@ -33,14 +35,24 @@ class GemmOperation:
     self.A = A
     self.B = B
     self.C = C
+    self.D = D
+    if self.D == None:
+      self.D = self.C
+
+    if gemm_kind != GemmKind.Universal3x:
+      assert(kernel_schedule == KernelScheduleType.ScheduleAuto)
+      assert(epilogue_schedule == EpilogueScheduleType.ScheduleAuto)
+    self.kernel_schedule = kernel_schedule
+    self.epilogue_schedule = epilogue_schedule
     self.element_epilogue = element_epilogue
     self.epilogue_functor = epilogue_functor
     self.swizzling_functor = swizzling_functor
+    self.tile_scheduler = tile_scheduler
 
   #
   def is_complex(self):
     complex_operators = [
-      MathOperation.multiply_add_complex, 
+      MathOperation.multiply_add_complex,
       MathOperation.multiply_add_complex_gaussian,
       MathOperation.multiply_add_complex_fast_f32
     ]
@@ -69,13 +81,14 @@ class GemmOperation:
   #
   def core_name(self):
     ''' The basic operation kind is prefixed with a letter indicating the accumulation type. '''
-    
+
     inst_shape = ''
     inst_operation = ''
     intermediate_type = ''
 
     math_operations_map = {
       MathOperation.xor_popc: 'xor',
+      MathOperation.and_popc: 'and'
     }
 
     if self.tile_description.math_instruction.opcode_class == OpcodeClass.TensorOp or \
@@ -122,11 +135,12 @@ class GemmOperation:
 
   def extended_name_3x(self):
     '''Generates a string representing the MMA atom. Assumes accumulator type is C type.'''
-    extended_name = "{core_name}_{element_a}_{element_b}_{element_acc}_{element_c}".format(
+    extended_name = "{core_name}_{element_a}_{element_b}_{element_acc}_{element_c}_{element_d}".format(
       element_a = DataTypeNames[self.A.element],
       element_b = DataTypeNames[self.B.element],
       element_acc = DataTypeNames[self.tile_description.math_instruction.element_accumulator],
       element_c = DataTypeNames[self.C.element],
+      element_d = DataTypeNames[self.D.element],
       core_name = self.core_name())
     return extended_name
 
@@ -134,7 +148,7 @@ class GemmOperation:
   def layout_name(self):
     if self.is_complex() or self.is_planar_complex():
       return "%s%s" % (
-        ShortComplexLayoutNames[(self.A.layout, self.A.complex_transform)], 
+        ShortComplexLayoutNames[(self.A.layout, self.A.complex_transform)],
         ShortComplexLayoutNames[(self.B.layout, self.B.complex_transform)]
       )
     return "%s%s" % (ShortLayoutTypeNames[self.A.layout], ShortLayoutTypeNames[self.B.layout])
@@ -143,7 +157,7 @@ class GemmOperation:
   def layout_name_3x(self):
     if self.is_complex() or self.is_planar_complex():
       return "{}{}{}".format(
-        ShortComplexLayoutNames[(self.A.layout, self.A.complex_transform)], 
+        ShortComplexLayoutNames[(self.A.layout, self.A.complex_transform)],
         ShortComplexLayoutNames[(self.B.layout, self.B.complex_transform)],
         ShortComplexLayoutNames[(self.C.layout, self.C.complex_transform)])
     else:
@@ -152,26 +166,37 @@ class GemmOperation:
         ShortLayoutTypeNames[self.B.layout],
         ShortLayoutTypeNames[self.C.layout])
 
+  # Generates a short string representing underlying kernel schedule type
+  def kernel_schedule_name_3x(self):
+    return KernelScheduleSuffixes[self.kernel_schedule]
+
+  # Generates a short string representing underlying epilogue schedule type
+  def epilogue_schedule_name_3x(self):
+    return EpilogueScheduleSuffixes[self.epilogue_schedule]
+
   # Generates the full kernel function name
   def procedural_name(self):
     ''' The full procedural name indicates architecture, extended name, tile size, and layout. '''
     opcode_class_name = OpcodeClassNames[self.tile_description.math_instruction.opcode_class]
     if self.arch >= 90:
-      kernel_name_template = "cutlass{p}_sm{ar}_{op}_{ex}_{tbm}x{tbn}x{tbk}_{cm}x{cn}x{ck}_{l}_{s}_align{al}"
+      kernel_name_template = "cutlass{p}_sm{ar}_{op}_{ex}_{tbm}x{tbn}x{tbk}_{cm}x{cn}x{ck}_{l}_{s}_align{al}{t}{k}{e}"
       return kernel_name_template.format(
           p = self.prefix,
           ar = self.arch,
           op = opcode_class_name,
           ex = self.extended_name_3x(),
-          tbm = self.tile_description.threadblock_shape[0],
-          tbn = self.tile_description.threadblock_shape[1],
-          tbk = self.tile_description.threadblock_shape[2],
+          tbm = self.tile_description.tile_shape[0],
+          tbn = self.tile_description.tile_shape[1],
+          tbk = self.tile_description.tile_shape[2],
           cm = self.tile_description.cluster_shape[0],
           cn = self.tile_description.cluster_shape[1],
           ck = self.tile_description.cluster_shape[2],
           l = self.tile_description.stages,
           s = self.layout_name_3x(),
-          al = str(max(self.A.alignment, self.B.alignment)))
+          al = str(max(self.A.alignment, self.B.alignment)),
+          t = TileSchedulerSuffixes[self.tile_scheduler],
+          k = self.kernel_schedule_name_3x(),
+          e = self.epilogue_schedule_name_3x())
     else:
       threadblock = self.tile_description.procedural_name()
       return "cutlass{p}_{op}_{ex}_{tb}_{l}_align{a}".format(
@@ -187,6 +212,11 @@ class GemmOperation:
     ''' The full procedural name indicates architecture, extended name, tile size, and layout. '''
     return self.procedural_name()
 
+  def __hash__(self):
+    return hash(self.configuration_name())
+
+  def __eq__(self, other):
+    return self.configuration_name() == other.configuration_name()
 
 ###################################################################################################
 #
@@ -299,7 +329,7 @@ ${compile_guard_end}
     epilogue_vector_length = int(min(operation.C.alignment * DataTypeSize[operation.C.element], 128) / DataTypeSize[operation.C.element])
 
     residual = ''
-    
+
     values = {
       'operation_name': operation.procedural_name(),
       'element_a': DataTypeTag[operation.A.element],
@@ -389,7 +419,7 @@ ${compile_guard_end}
     epilogue_vector_length = int(min(operation.C.alignment * DataTypeSize[operation.C.element], 128) / DataTypeSize[operation.C.element])
 
     residual = ''
-    
+
     values = {
       'operation_name': operation.procedural_name(),
       'element_a': DataTypeTag[operation.A.element],
@@ -456,7 +486,7 @@ class EmitGemmUniversalInstance:
 """
     self.gemm_template = """
 // Gemm operator ${operation_name}
-using ${operation_name}_base = 
+using ${operation_name}_base =
   typename cutlass::gemm::kernel::DefaultGemmUniversal<
     ${element_b}, ${layout_b}, ${transform_b}, ${align_b},    // transposed B operand
     ${element_a}, ${layout_a}, ${transform_a}, ${align_a},    // transposed A operand
@@ -474,12 +504,12 @@ using ${operation_name}_base =
 >::GemmKernel;
 
 // Define named type
-struct ${operation_name}${operation_suffix} : 
+struct ${operation_name}${operation_suffix} :
   public ${operation_name}_base { };
 """
     self.gemm_template_interleaved = """
 // Gemm operator ${operation_name}
-using ${operation_name}_base = 
+using ${operation_name}_base =
   typename cutlass::gemm::kernel::DefaultGemmUniversal<
     ${element_a}, ${layout_a}, ${transform_a}, ${align_a},
     ${element_b}, ${layout_b}, ${transform_b}, ${align_b},
@@ -497,7 +527,7 @@ using ${operation_name}_base =
 >::GemmKernel;
 
 // Define named type
-struct ${operation_name}${operation_suffix} : 
+struct ${operation_name}${operation_suffix} :
   public ${operation_name}_base { };
 """
 
@@ -604,8 +634,7 @@ class EmitGemmUniversal3xInstance:
       "cutlass/numeric_types.h",
       "cutlass/gemm/kernel/gemm_universal.hpp",
       "cutlass/gemm/collective/collective_builder.hpp",
-      "cutlass/epilogue/collective/default_epilogue.hpp",
-      "cutlass/epilogue/thread/linear_combination.h",
+      "cutlass/epilogue/collective/collective_builder.hpp",
     ]
     self.builtin_epilogue_functor_template = """
     ${epilogue_functor}<
@@ -617,31 +646,36 @@ class EmitGemmUniversal3xInstance:
 """
     self.gemm_template = """
 
+using ${operation_name}_epilogue =
+  typename cutlass::epilogue::collective::CollectiveBuilder<
+    ${arch}, ${opcode_class},
+    cute::Shape<cute::_${tile_shape_m}, cute::_${tile_shape_n}, cute::_${tile_shape_k}>,
+    cute::Shape<cute::_${cluster_m},cute::_${cluster_n},cute::_${cluster_k}>,
+    cutlass::epilogue::collective::EpilogueTileAuto,
+    ${element_accumulator}, ${element_epilogue},
+    ${element_c}, ${layout_c}, ${align_c},
+    ${element_d}, ${layout_d}, ${align_d},
+    ${epilogue_schedule}
+  >::CollectiveOp;
+
 using ${operation_name}_mainloop =
   typename cutlass::gemm::collective::CollectiveBuilder<
     ${arch}, ${opcode_class},
     ${element_a}, ${layout_a}, ${align_a},
     ${element_b}, ${layout_b}, ${align_b},
     ${element_accumulator},
-    cute::Shape<cute::_${threadblock_shape_m}, cute::_${threadblock_shape_n}, cute::_${threadblock_shape_k}>,
+    cute::Shape<cute::_${tile_shape_m}, cute::_${tile_shape_n}, cute::_${tile_shape_k}>,
     cute::Shape<cute::_${cluster_m},cute::_${cluster_n},cute::_${cluster_k}>,
-    cutlass::gemm::collective::StageCountAuto,
-    cutlass::gemm::collective::KernelScheduleAuto
+    ${stages},
+  ${kernel_schedule}
   >::CollectiveOp;
 
-using ${operation_name}_epilogue =
-  cutlass::epilogue::collective::DefaultEpilogue<
-    cutlass::gemm::TagToStrideC_t<${layout_c}>,
-    cutlass::gemm::TagToStrideC_t<${layout_c}>,
-    cutlass::epilogue::thread::LinearCombination<
-      ${element_c}, ${epilogue_vector_length}, ${element_accumulator}, ${element_epilogue}>
-  >;
-
 // Gemm operator ${operation_name}
 using ${operation_name}_base = cutlass::gemm::kernel::GemmUniversal<
     cute::Shape<int,int,int,int>,
     ${operation_name}_mainloop,
-    ${operation_name}_epilogue>;
+    ${operation_name}_epilogue,
+    ${tile_scheduler}>;
 
 // Define named type
 struct ${operation_name} :
@@ -661,17 +695,17 @@ ${compile_guard_end}
   #
   def emit(self, operation):
 
-    threadblock_shape = operation.tile_description.threadblock_shape
+    tile_shape = operation.tile_description.tile_shape
     warp_count = operation.tile_description.warp_count
     # stage count set to zero indicates builder automatic stage selection
     if operation.tile_description.stages > 0:
       stage_count_string = f"cutlass::gemm::collective::StageCount<{str(operation.tile_description.stages)}>"
     else:
-      stage_count_string = "cutlass::gemm::collective::StageCountAuto"
-    warp_shape = [threadblock_shape[idx] // warp_count[idx] for idx in range(3)]
+      stage_count_string = f"cutlass::gemm::collective::StageCountAutoCarveout<sizeof(typename {str(operation.procedural_name())}_epilogue::SharedStorage)>"
+    warp_shape = [tile_shape[idx] // warp_count[idx] for idx in range(3)]
 
-    instance_layout_A, instance_layout_B, instance_layout_C = \
-      (operation.A.layout, operation.B.layout, operation.C.layout)
+    instance_layout_A, instance_layout_B, instance_layout_C , instance_layout_D = \
+      (operation.A.layout, operation.B.layout, operation.C.layout, operation.D.layout)
 
     # 3.0 profiler integration only supports trivial epilogues for now
     epilogue_vector_length = 1
@@ -697,12 +731,14 @@ ${compile_guard_end}
       'layout_b': LayoutTag[instance_layout_B],
       'element_c': DataTypeTag[operation.C.element],
       'layout_c': LayoutTag[instance_layout_C],
+      'element_d': DataTypeTag[operation.D.element],
+      'layout_d': LayoutTag[instance_layout_D],
       'element_accumulator': DataTypeTag[operation.accumulator_type()],
       'opcode_class': OpcodeClassTag[operation.tile_description.math_instruction.opcode_class],
       'arch': "cutlass::arch::Sm%d" % operation.arch,
-      'threadblock_shape_m': str(operation.tile_description.threadblock_shape[0]),
-      'threadblock_shape_n': str(operation.tile_description.threadblock_shape[1]),
-      'threadblock_shape_k': str(operation.tile_description.threadblock_shape[2]),
+      'tile_shape_m': str(operation.tile_description.tile_shape[0]),
+      'tile_shape_n': str(operation.tile_description.tile_shape[1]),
+      'tile_shape_k': str(operation.tile_description.tile_shape[2]),
       'cluster_m': str(operation.tile_description.cluster_shape[0]),
       'cluster_n': str(operation.tile_description.cluster_shape[1]),
       'cluster_k': str(operation.tile_description.cluster_shape[2]),
@@ -712,15 +748,20 @@ ${compile_guard_end}
       'instruction_shape_m': str(operation.tile_description.math_instruction.instruction_shape[0]),
       'instruction_shape_n': str(operation.tile_description.math_instruction.instruction_shape[1]),
       'instruction_shape_k': str(operation.tile_description.math_instruction.instruction_shape[2]),
+      'kernel_schedule' : str(KernelScheduleTag[operation.kernel_schedule]),
+      'epilogue_schedule' : str(EpilogueScheduleTag[operation.epilogue_schedule]),
       'epilogue_functor': epilogue_functor,
       'stages': stage_count_string,
       'align_a': str(operation.A.alignment),
       'align_b': str(operation.B.alignment),
+      'align_c': str(operation.C.alignment),
+      'align_d': str(operation.C.alignment),
       'transform_a': ComplexTransformTag[operation.A.complex_transform],
       'transform_b': ComplexTransformTag[operation.B.complex_transform],
       'math_operation': MathOperationTag[operation.tile_description.math_instruction.math_operation],
       'epilogue_vector_length': str(epilogue_vector_length),
       'element_epilogue': str(DataTypeTag[operation.element_epilogue]),
+      'tile_scheduler': str(TileSchedulerTag[operation.tile_scheduler])
     }
 
     return SubstituteTemplate(self.gemm_template, values)
@@ -757,7 +798,7 @@ class EmitGemmPlanarComplexInstance:
     ${math_operator}
   >::GemmKernel;
 
-  struct ${operation_name} : 
+  struct ${operation_name} :
     public Operation_${operation_name} { };
 """
 
@@ -1134,7 +1175,7 @@ void initialize_${configuration_name}(Manifest &manifest) {
       'compile_guard_start': SubstituteTemplate(self.wmma_guard_start, {'sm_number': str(operation.arch)}) \
         if operation.tile_description.math_instruction.opcode_class == OpcodeClass.WmmaTensorOp else "",
       'compile_guard_end': "#endif" \
-        if operation.tile_description.math_instruction.opcode_class == OpcodeClass.WmmaTensorOp else "" 
+        if operation.tile_description.math_instruction.opcode_class == OpcodeClass.WmmaTensorOp else ""
       }))
 
   def __exit__(self, exception_type, exception_value, traceback):
@@ -1154,9 +1195,9 @@ void initialize_${configuration_name}(Manifest &manifest) {
     self.configuration_file.write(SubstituteTemplate(self.initialize_function_template, {
       'configuration_name': self.configuration_name
       }))
-   
+
     for instance_wrapper in self.instance_wrappers:
-      self.configuration_file.write(instance_wrapper) 
+      self.configuration_file.write(instance_wrapper)
 
     self.configuration_file.write(self.epilogue_template)
     self.configuration_file.close()
