@@ -32,10 +32,18 @@
  * Android logcat.
  */
 #define TVM_LOG_CUSTOMIZE 1
+#define TVM_FFI_USE_LIBBACKTRACE 0
 
-#include "../src/runtime/c_runtime_api.cc"
-#include "../src/runtime/container.cc"
+#include "../ffi/src/ffi/container.cc"
+#include "../ffi/src/ffi/dtype.cc"
+#include "../ffi/src/ffi/error.cc"
+#include "../ffi/src/ffi/function.cc"
+#include "../ffi/src/ffi/ndarray.cc"
+#include "../ffi/src/ffi/object.cc"
+#include "../ffi/src/ffi/testing.cc"
+#include "../ffi/src/ffi/traceback.cc"
 #include "../src/runtime/cpu_device_api.cc"
+#include "../src/runtime/device_api.cc"
 #include "../src/runtime/dso_library.cc"
 #include "../src/runtime/file_utils.cc"
 #include "../src/runtime/library_module.cc"
@@ -44,7 +52,6 @@
 #include "../src/runtime/minrpc/minrpc_logger.cc"
 #include "../src/runtime/module.cc"
 #include "../src/runtime/ndarray.cc"
-#include "../src/runtime/object.cc"
 #include "../src/runtime/profiling.cc"
 #include "../src/runtime/registry.cc"
 #include "../src/runtime/rpc/rpc_channel.cc"
