@@ -22,11 +22,6 @@ set -u
 set -x
 set -o pipefail
 
-export DEBIAN_FRONTEND=noninteractive
-export TZ=Etc/UTC
-ln -snf /usr/share/zoneinfo/$TZ /etc/localtime
-echo $TZ > /etc/timezone
-
 # install libraries for building c++ core on ubuntu
 apt-get update && apt-install-and-clear -y --no-install-recommends \
     apt-transport-https \
