@@ -25,6 +25,7 @@
 #define TVM_ARITH_ANALYZER_H_
 
 #include <tvm/arith/int_set.h>
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/ir/expr.h>
 #include <tvm/support/with.h>
 
@@ -86,9 +87,11 @@ class ConstIntBoundNode : public Object {
   int64_t min_value;
   int64_t max_value;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("min_value", &min_value);
-    v->Visit("max_value", &max_value);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ConstIntBoundNode>()
+        .def_ro("min_value", &ConstIntBoundNode::min_value)
+        .def_ro("max_value", &ConstIntBoundNode::max_value);
   }
 
   bool SEqualReduce(const ConstIntBoundNode* other, SEqualReducer equal) const {
@@ -154,6 +157,7 @@ class ConstIntBoundAnalyzer {
    * \param allow_override whether we allow override of existing information.
    */
   TVM_DLL void Update(const Var& var, const ConstIntBound& info, bool allow_override = false);
+
   /*!
    * \brief Bind variable to a range.
    *
@@ -163,6 +167,13 @@ class ConstIntBoundAnalyzer {
    */
   TVM_DLL void Bind(const Var& var, const Range& range, bool allow_override = false);
 
+  /*!
+   * \brief Check if a variable is bound to a range.
+   * \param var The variable.
+   * \return Whether the variable is bound to a range.
+   */
+  TVM_DLL bool IsBound(const Var& var) const;
+
  private:
   friend class Analyzer;
   friend class ConstraintContext;
@@ -200,9 +211,11 @@ class ModularSetNode : public Object {
   /*! \brief The base */
   int64_t base;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("coeff", &coeff);
-    v->Visit("base", &base);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ModularSetNode>()
+        .def_ro("coeff", &ModularSetNode::coeff)
+        .def_ro("base", &ModularSetNode::base);
   }
 
   bool SEqualReduce(const ModularSetNode* other, SEqualReducer equal) const {
