@@ -26,6 +26,7 @@
 #ifndef TVM_IR_DIAGNOSTIC_H_
 #define TVM_IR_DIAGNOSTIC_H_
 
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/ir/module.h>
 
 #include <sstream>
@@ -33,8 +34,6 @@
 
 namespace tvm {
 
-using tvm::runtime::TypedPackedFunc;
-
 /*! \brief The diagnostic level, controls the printing of the message. */
 enum class DiagnosticLevel : int {
   kBug = 10,
@@ -67,11 +66,12 @@ class DiagnosticNode : public Object {
   /*! \brief The diagnostic message. */
   String message;
 
-  // override attr visitor
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("level", &level);
-    v->Visit("span", &span);
-    v->Visit("message", &message);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DiagnosticNode>()
+        .def_ro("level", &DiagnosticNode::level)
+        .def_ro("span", &DiagnosticNode::span)
+        .def_ro("message", &DiagnosticNode::message);
   }
 
   bool SEqualReduce(const DiagnosticNode* other, SEqualReducer equal) const {
@@ -165,10 +165,12 @@ class DiagnosticContext;
  */
 class DiagnosticRendererNode : public Object {
  public:
-  TypedPackedFunc<void(DiagnosticContext ctx)> renderer;
+  ffi::TypedFunction<void(DiagnosticContext ctx)> renderer;
 
-  // override attr visitor
-  void VisitAttrs(AttrVisitor* v) {}
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DiagnosticRendererNode>().def_ro("renderer", &DiagnosticRendererNode::renderer);
+  }
 
   static constexpr const char* _type_key = "DiagnosticRenderer";
   TVM_DECLARE_FINAL_OBJECT_INFO(DiagnosticRendererNode, Object);
@@ -176,9 +178,9 @@ class DiagnosticRendererNode : public Object {
 
 class DiagnosticRenderer : public ObjectRef {
  public:
-  TVM_DLL DiagnosticRenderer(TypedPackedFunc<void(DiagnosticContext ctx)> render);
+  TVM_DLL DiagnosticRenderer(ffi::TypedFunction<void(DiagnosticContext ctx)> render);
   TVM_DLL DiagnosticRenderer()
-      : DiagnosticRenderer(TypedPackedFunc<void(DiagnosticContext ctx)>()) {}
+      : DiagnosticRenderer(ffi::TypedFunction<void(DiagnosticContext ctx)>()) {}
 
   void Render(const DiagnosticContext& ctx);
 
@@ -201,9 +203,11 @@ class DiagnosticContextNode : public Object {
   /*! \brief The renderer set for the context. */
   DiagnosticRenderer renderer;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("module", &module);
-    v->Visit("diagnostics", &diagnostics);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DiagnosticContextNode>()
+        .def_ro("module", &DiagnosticContextNode::module)
+        .def_ro("diagnostics", &DiagnosticContextNode::diagnostics);
   }
 
   bool SEqualReduce(const DiagnosticContextNode* other, SEqualReducer equal) const {
