@@ -27,6 +27,7 @@
 #include <string>
 #include <unordered_map>
 
+#include "tvm/ffi/reflection/reflection.h"
 #include "tvm/ir/expr.h"
 #include "tvm/ir/module.h"
 #include "tvm/ir/name_supply.h"
@@ -75,12 +76,15 @@ class GlobalVarSupplyNode : public Object {
    */
   void ReserveGlobalVar(const GlobalVar& var, bool allow_conflict = false);
 
-  void VisitAttrs(AttrVisitor* v) {}
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<GlobalVarSupplyNode>();
+  }
 
   /*! \brief The NameSupply used to generate unique name hints to GlobalVars. */
   NameSupply name_supply_;
 
-  static constexpr const char* _type_key = "GlobalVarSupply";
+  static constexpr const char* _type_key = "ir.GlobalVarSupply";
   static constexpr const bool _type_has_method_sequal_reduce = false;
   static constexpr const bool _type_has_method_shash_reduce = false;
   TVM_DECLARE_FINAL_OBJECT_INFO(GlobalVarSupplyNode, Object);
