@@ -19,10 +19,11 @@
 #ifndef TVM_META_SCHEDULE_EXTRACTED_TASK_H_
 #define TVM_META_SCHEDULE_EXTRACTED_TASK_H_
 
+#include <tvm/ffi/container/array.h>
+#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/string.h>
 #include <tvm/ir/module.h>
 #include <tvm/node/reflection.h>
-#include <tvm/runtime/container/array.h>
-#include <tvm/runtime/container/string.h>
 #include <tvm/runtime/object.h>
 #include <tvm/target/target.h>
 
@@ -52,15 +53,18 @@ class ExtractedTaskNode : public runtime::Object {
   /*! \brief Weight of the task */
   int weight;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("task_name", &task_name);
-    v->Visit("mod", &mod);
-    v->Visit("target", &target);
-    v->Visit("dispatched", &dispatched);
-    v->Visit("weight", &weight);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ExtractedTaskNode>()
+        .def_ro("task_name", &ExtractedTaskNode::task_name)
+        .def_ro("mod", &ExtractedTaskNode::mod)
+        .def_ro("target", &ExtractedTaskNode::target)
+        .def_ro("dispatched", &ExtractedTaskNode::dispatched)
+        .def_ro("weight", &ExtractedTaskNode::weight);
   }
 
   static constexpr const char* _type_key = "meta_schedule.ExtractedTask";
+
   TVM_DECLARE_FINAL_OBJECT_INFO(ExtractedTaskNode, runtime::Object);
 };
 
