@@ -20,16 +20,17 @@
 #ifndef TVM_META_SCHEDULE_MEASURE_CALLBACK_H_
 #define TVM_META_SCHEDULE_MEASURE_CALLBACK_H_
 
+#include <tvm/ffi/container/array.h>
+#include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/string.h>
 #include <tvm/meta_schedule/builder.h>
 #include <tvm/meta_schedule/measure_candidate.h>
 #include <tvm/meta_schedule/runner.h>
 #include <tvm/meta_schedule/search_strategy.h>
 #include <tvm/meta_schedule/tune_context.h>
 #include <tvm/node/reflection.h>
-#include <tvm/runtime/container/array.h>
-#include <tvm/runtime/container/string.h>
 #include <tvm/runtime/object.h>
-#include <tvm/runtime/packed_func.h>
 
 namespace tvm {
 namespace meta_schedule {
@@ -42,7 +43,9 @@ class MeasureCallbackNode : public runtime::Object {
   /*! \brief Virtual destructor. */
   virtual ~MeasureCallbackNode() = default;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {}
+  static void RegisterReflection() {
+    // No fields to register
+  }
 
   /*!
    * \brief Apply a measure callback rule with given arguments.
@@ -74,26 +77,25 @@ class PyMeasureCallbackNode : public MeasureCallbackNode {
    * \param results The runner results by running the built measure candidates.
    * \return Whether the measure callback was successfully applied.
    */
-  using FApply =
-      runtime::TypedPackedFunc<void(const TaskScheduler& task_scheduler,                //
-                                    int task_id,                                        //
-                                    const Array<MeasureCandidate>& measure_candidates,  //
-                                    const Array<BuilderResult>& builds,                 //
-                                    const Array<RunnerResult>& results)>;
+  using FApply = ffi::TypedFunction<void(const TaskScheduler& task_scheduler,                //
+                                         int task_id,                                        //
+                                         const Array<MeasureCandidate>& measure_candidates,  //
+                                         const Array<BuilderResult>& builds,                 //
+                                         const Array<RunnerResult>& results)>;
   /*!
    * \brief Get the measure callback function as string with name.
    * \return The string of the measure callback function.
    */
-  using FAsString = runtime::TypedPackedFunc<String()>;
+  using FAsString = ffi::TypedFunction<String()>;
 
   /*! \brief The packed function to the `Apply` function. */
   FApply f_apply;
   /*! \brief The packed function to the `AsString` function. */
   FAsString f_as_string;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    // `f_apply` is not visited
-    // `f_as_string` is not visited
+  static void RegisterReflection() {
+    // `f_apply` is not registered
+    // `f_as_string` is not registered
   }
 
   void Apply(const TaskScheduler& task_scheduler,                //
