@@ -19,14 +19,15 @@
 #ifndef TVM_META_SCHEDULE_RUNNER_H_
 #define TVM_META_SCHEDULE_RUNNER_H_
 
+#include <tvm/ffi/container/array.h>
+#include <tvm/ffi/function.h>
+#include <tvm/ffi/optional.h>
+#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/string.h>
 #include <tvm/ir/expr.h>
 #include <tvm/meta_schedule/arg_info.h>
 #include <tvm/node/reflection.h>
-#include <tvm/runtime/container/array.h>
-#include <tvm/runtime/container/optional.h>
-#include <tvm/runtime/container/string.h>
 #include <tvm/runtime/object.h>
-#include <tvm/runtime/packed_func.h>
 
 namespace tvm {
 namespace meta_schedule {
@@ -41,13 +42,16 @@ class RunnerInputNode : public runtime::Object {
   /*! \brief The argument information. */
   Array<ArgInfo> args_info;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("artifact_path", &artifact_path);
-    v->Visit("device_type", &device_type);
-    v->Visit("args_info", &args_info);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<RunnerInputNode>()
+        .def_ro("artifact_path", &RunnerInputNode::artifact_path)
+        .def_ro("device_type", &RunnerInputNode::device_type)
+        .def_ro("args_info", &RunnerInputNode::args_info);
   }
 
   static constexpr const char* _type_key = "meta_schedule.RunnerInput";
+
   TVM_DECLARE_FINAL_OBJECT_INFO(RunnerInputNode, runtime::Object);
 };
 
@@ -75,12 +79,15 @@ class RunnerResultNode : public runtime::Object {
   /*! \brief The error message, if any. */
   Optional<String> error_msg;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("run_secs", &run_secs);
-    v->Visit("error_msg", &error_msg);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<RunnerResultNode>()
+        .def_ro("run_secs", &RunnerResultNode::run_secs)
+        .def_ro("error_msg", &RunnerResultNode::error_msg);
   }
 
   static constexpr const char* _type_key = "meta_schedule.RunnerResult";
+
   TVM_DECLARE_FINAL_OBJECT_INFO(RunnerResultNode, runtime::Object);
 };
 
@@ -110,21 +117,21 @@ class RunnerFutureNode : public runtime::Object {
    * \brief The function type to check whether the runner has finished.
    * \return Whether the runner's output is ready.
    */
-  using FDone = runtime::TypedPackedFunc<bool()>;
+  using FDone = ffi::TypedFunction<bool()>;
   /*!
    * \brief The function type to fetch runner output if it is ready.
    * \return The runner's output.
    */
-  using FResult = runtime::TypedPackedFunc<RunnerResult()>;
+  using FResult = ffi::TypedFunction<RunnerResult()>;
 
   /*! \brief The packed function to check whether the runner has finished. */
   FDone f_done;
   /*! \brief The packed function to fetch runner output if it is ready. */
   FResult f_result;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    // `f_done` is not visited
-    // `f_result` is not visited
+  static void RegisterReflection() {
+    // `f_done` is not registered
+    // `f_result` is not registered
   }
 
   /*!
@@ -176,7 +183,7 @@ class RunnerNode : public runtime::Object {
    * \return The runner futures.
    * \sa RunnerFuture
    */
-  using FRun = runtime::TypedPackedFunc<Array<RunnerFuture>(Array<RunnerInput>)>;
+  using FRun = ffi::TypedFunction<Array<RunnerFuture>(Array<RunnerInput>)>;
 
   /*! \brief Default destructor */
   virtual ~RunnerNode() = default;
@@ -215,8 +222,8 @@ class PyRunnerNode : public RunnerNode {
   /*! \brief The packed function to run the built artifacts and get runner futures. */
   FRun f_run;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    // `f_run` is not visited
+  static void RegisterReflection() {
+    // `f_run` is not registered
   }
 
   Array<RunnerFuture> Run(Array<RunnerInput> runner_inputs) final {
