@@ -19,6 +19,12 @@
 #ifndef TVM_META_SCHEDULE_TUNE_CONTEXT_H_
 #define TVM_META_SCHEDULE_TUNE_CONTEXT_H_
 
+#include <tvm/ffi/container/array.h>
+#include <tvm/ffi/container/map.h>
+#include <tvm/ffi/function.h>
+#include <tvm/ffi/optional.h>
+#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/string.h>
 #include <tvm/ir/expr.h>
 #include <tvm/ir/module.h>
 #include <tvm/meta_schedule/builder.h>
@@ -26,12 +32,7 @@
 #include <tvm/meta_schedule/search_strategy.h>
 #include <tvm/meta_schedule/space_generator.h>
 #include <tvm/node/reflection.h>
-#include <tvm/runtime/container/array.h>
-#include <tvm/runtime/container/map.h>
-#include <tvm/runtime/container/optional.h>
-#include <tvm/runtime/container/string.h>
 #include <tvm/runtime/object.h>
-#include <tvm/runtime/packed_func.h>
 #include <tvm/support/random_engine.h>
 #include <tvm/target/target.h>
 
@@ -62,18 +63,21 @@ class TuneContextNode : public runtime::Object {
   /*! \brief The random state. */
   TRandState rand_state;
   /*! \brief The tuning task's logging function. t*/
-  PackedFunc logger;
+  ffi::Function logger;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("mod", &mod);
-    v->Visit("target", &target);
-    v->Visit("space_generator", &space_generator);
-    v->Visit("search_strategy", &search_strategy);
-    v->Visit("task_name", &task_name);
-    v->Visit("num_threads", &num_threads);
-    v->Visit("rand_state", &rand_state);
-    // `logger` is not visited
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<TuneContextNode>()
+        .def_ro("mod", &TuneContextNode::mod)
+        .def_ro("target", &TuneContextNode::target)
+        .def_ro("space_generator", &TuneContextNode::space_generator)
+        .def_ro("search_strategy", &TuneContextNode::search_strategy)
+        .def_ro("task_name", &TuneContextNode::task_name)
+        .def_ro("num_threads", &TuneContextNode::num_threads)
+        .def_ro("rand_state", &TuneContextNode::rand_state);
+    // `logger` is not registered
   }
+
   /*!
    * \brief Initialize members that needs initialization with tune context.
    */
@@ -109,7 +113,7 @@ class TuneContext : public runtime::ObjectRef {
   TVM_DLL explicit TuneContext(Optional<IRModule> mod, Optional<Target> target,
                                Optional<SpaceGenerator> space_generator,
                                Optional<SearchStrategy> search_strategy, Optional<String> task_name,
-                               int num_threads, TRandState rand_state, PackedFunc logger);
+                               int num_threads, TRandState rand_state, ffi::Function logger);
   TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(TuneContext, ObjectRef, TuneContextNode);
 };
 
