@@ -43,6 +43,8 @@ class ReprPrinter {
 
   /*! \brief The node to be printed. */
   TVM_DLL void Print(const ObjectRef& node);
+  /*! \brief The node to be printed. */
+  TVM_DLL void Print(const ffi::Any& node);
   /*! \brief Print indent to the stream */
   TVM_DLL void PrintIndent();
   // Allow registration to be printer.
@@ -50,32 +52,6 @@ class ReprPrinter {
   TVM_DLL static FType& vtable();
 };
 
-/*! \brief Legacy behavior of ReprPrinter. */
-class ReprLegacyPrinter {
- public:
-  /*! \brief The indentation level. */
-  int indent{0};
-
-  explicit ReprLegacyPrinter(std::ostream& stream)  // NOLINT(*)
-      : stream(stream) {}
-
-  /*! \brief The node to be printed. */
-  TVM_DLL void Print(const ObjectRef& node);
-  /*! \brief Print indent to the stream */
-  TVM_DLL void PrintIndent();
-  /*! \brief Could the LegacyPrinter dispatch the node */
-  TVM_DLL static bool CanDispatch(const ObjectRef& node);
-  /*! \brief Return the ostream it maintains */
-  TVM_DLL std::ostream& Stream() const;
-  // Allow registration to be printer.
-  using FType = NodeFunctor<void(const ObjectRef&, ReprLegacyPrinter*)>;
-  TVM_DLL static FType& vtable();
-
- private:
-  /*! \brief The output stream */
-  std::ostream& stream;
-};
-
 /*!
  * \brief Dump the node to stderr, used for debug purposes.
  * \param node The input node
@@ -91,7 +67,7 @@ TVM_DLL void Dump(const runtime::Object* node);
 }  // namespace tvm
 
 namespace tvm {
-namespace runtime {
+namespace ffi {
 // default print function for all objects
 // provide in the runtime namespace as this is where objectref originally comes from.
 inline std::ostream& operator<<(std::ostream& os, const ObjectRef& n) {  // NOLINT(*)
@@ -99,12 +75,18 @@ inline std::ostream& operator<<(std::ostream& os, const ObjectRef& n) {  // NOLI
   return os;
 }
 
-inline std::string AsLegacyRepr(const ObjectRef& n) {
-  std::ostringstream os;
-  ReprLegacyPrinter(os).Print(n);
-  return os.str();
+// default print function for any
+inline std::ostream& operator<<(std::ostream& os, const Any& n) {  // NOLINT(*)
+  ReprPrinter(os).Print(n);
+  return os;
 }
-}  // namespace runtime
-using runtime::AsLegacyRepr;
+
+template <typename... V>
+inline std::ostream& operator<<(std::ostream& os, const Variant<V...>& n) {  // NOLINT(*)
+  ReprPrinter(os).Print(Any(n));
+  return os;
+}
+
+}  // namespace ffi
 }  // namespace tvm
 #endif  // TVM_NODE_REPR_PRINTER_H_
