@@ -19,13 +19,14 @@
 #ifndef TVM_RELAX_EXPR_H_
 #define TVM_RELAX_EXPR_H_
 
+#include <tvm/ffi/container/array.h>
+#include <tvm/ffi/container/map.h>
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/ir/expr.h>
 #include <tvm/ir/function.h>
 #include <tvm/ir/source_map.h>
 #include <tvm/node/node.h>
 #include <tvm/relax/type.h>
-#include <tvm/runtime/container/array.h>
-#include <tvm/runtime/container/map.h>
 #include <tvm/runtime/object.h>
 #include <tvm/tir/expr.h>
 #include <tvm/tir/op.h>
@@ -54,7 +55,10 @@ class IdNode : public Object {
    */
   String name_hint;
 
-  void VisitAttrs(tvm::AttrVisitor* v) { v->Visit("name_hint", &name_hint); }
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<IdNode>().def_ro("name_hint", &IdNode::name_hint);
+  }
 
   bool SEqualReduce(const IdNode* other, SEqualReducer equal) const {
     return equal.FreeVarEqualImpl(this, other);
@@ -116,7 +120,7 @@ class StructInfoNode : public Object {
    */
   mutable Span span;
 
-  static constexpr const char* _type_key = "StructInfo";
+  static constexpr const char* _type_key = "ir.StructInfo";
   static constexpr const bool _type_has_method_sequal_reduce = true;
   static constexpr const bool _type_has_method_shash_reduce = true;
   static constexpr const uint32_t _type_child_slots = 7;
@@ -160,14 +164,13 @@ class CallNode : public ExprNode {
    */
   Array<StructInfo> sinfo_args;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("op", &op);
-    v->Visit("args", &args);
-    v->Visit("attrs", &attrs);
-    v->Visit("sinfo_args", &sinfo_args);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<CallNode>()
+        .def_ro("op", &CallNode::op)
+        .def_ro("args", &CallNode::args)
+        .def_ro("attrs", &CallNode::attrs)
+        .def_ro("sinfo_args", &CallNode::sinfo_args);
   }
 
   bool SEqualReduce(const CallNode* other, SEqualReducer equal) const {
@@ -222,11 +225,9 @@ class TupleNode : public ExprNode {
   /*! \brief the fields of the tuple */
   tvm::Array<Expr> fields;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("fields", &fields);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<TupleNode>().def_ro("fields", &TupleNode::fields);
   }
 
   bool SEqualReduce(const TupleNode* other, SEqualReducer equal) const {
@@ -287,12 +288,11 @@ class TupleGetItemNode : public ExprNode {
   /*! \brief which value to get */
   int index;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("tuple_value", &tuple);
-    v->Visit("index", &index);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<TupleGetItemNode>()
+        .def_ro("tuple_value", &TupleGetItemNode::tuple)
+        .def_ro("index", &TupleGetItemNode::index);
   }
 
   bool SEqualReduce(const TupleGetItemNode* other, SEqualReducer equal) const {
@@ -359,11 +359,9 @@ class ShapeExprNode : public LeafExprNode {
   /*! The values of the shape expression. */
   Array<PrimExpr> values;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("values", &values);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ShapeExprNode>().def_ro("values", &ShapeExprNode::values);
   }
 
   bool SEqualReduce(const ShapeExprNode* other, SEqualReducer equal) const {
@@ -396,11 +394,9 @@ class VarNode : public LeafExprNode {
   /*! \return The name hint of the variable */
   const String& name_hint() const { return vid->name_hint; }
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("vid", &vid);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<VarNode>().def_ro("vid", &VarNode::vid);
   }
 
   bool SEqualReduce(const VarNode* other, SEqualReducer equal) const {
@@ -437,11 +433,9 @@ class Var : public LeafExpr {
  */
 class DataflowVarNode : public VarNode {
  public:
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("vid", &vid);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DataflowVarNode>();
   }
 
   bool SEqualReduce(const DataflowVarNode* other, SEqualReducer equal) const {
@@ -489,11 +483,9 @@ class ConstantNode : public LeafExprNode {
   /*! \return Whether it is scalar(ndim-0 tensor) */
   bool is_scalar() const { return data->ndim == 0; }
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("data", &data);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ConstantNode>().def_ro("data", &ConstantNode::data);
   }
 
   bool SEqualReduce(const ConstantNode* other, SEqualReducer equal) const {
@@ -520,7 +512,7 @@ class Constant : public LeafExpr {
    * \param span The source span of the expression.
    */
   TVM_DLL explicit Constant(runtime::NDArray data,
-                            Optional<StructInfo> struct_info_annotation = NullOpt,
+                            Optional<StructInfo> struct_info_annotation = std::nullopt,
                             Span span = Span());
 
   TVM_DEFINE_OBJECT_REF_METHODS(Constant, LeafExpr, ConstantNode);
@@ -537,11 +529,9 @@ class PrimValueNode : public LeafExprNode {
   /*! \brief The prim expr representing the value */
   PrimExpr value;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("value", &value);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<PrimValueNode>().def_ro("value", &PrimValueNode::value);
   }
 
   bool SEqualReduce(const PrimValueNode* other, SEqualReducer equal) const {
@@ -588,11 +578,9 @@ class StringImmNode : public LeafExprNode {
   /*! \brief The data value. */
   String value;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("value", &value);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<StringImmNode>().def_ro("value", &StringImmNode::value);
   }
 
   bool SEqualReduce(const StringImmNode* other, SEqualReducer equal) const {
@@ -631,11 +619,9 @@ class DataTypeImmNode : public LeafExprNode {
   /*! \brief The data value. */
   DataType value;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("value", &value);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DataTypeImmNode>().def_ro("value", &DataTypeImmNode::value);
   }
 
   bool SEqualReduce(const DataTypeImmNode* other, SEqualReducer equal) const {
@@ -673,6 +659,13 @@ class BindingNode : public Object {
   Var var;
   mutable Span span;
 
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BindingNode>()
+        .def_ro("var", &BindingNode::var)
+        .def_ro("span", &BindingNode::span);
+  }
+
   static constexpr const char* _type_key = "relax.expr.Binding";
   static constexpr const bool _type_has_method_sequal_reduce = true;
   static constexpr const bool _type_has_method_shash_reduce = true;
@@ -705,11 +698,12 @@ class MatchCastNode : public BindingNode {
   /*! \brief The struct info pattern to match to. */
   StructInfo struct_info;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("var", &var);
-    v->Visit("value", &value);
-    v->Visit("struct_info", &struct_info);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<MatchCastNode>()
+        .def_ro("var", &MatchCastNode::var)
+        .def_ro("value", &MatchCastNode::value)
+        .def_ro("struct_info", &MatchCastNode::struct_info);
   }
 
   bool SEqualReduce(const MatchCastNode* other, SEqualReducer equal) const;
@@ -738,10 +732,11 @@ class VarBindingNode : public BindingNode {
   /*! \brief The binding value. */
   Expr value;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("var", &var);
-    v->Visit("value", &value);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<VarBindingNode>()
+        .def_ro("var", &VarBindingNode::var)
+        .def_ro("value", &VarBindingNode::value);
   }
 
   bool SEqualReduce(const VarBindingNode* other, SEqualReducer equal) const;
@@ -765,9 +760,9 @@ class BindingBlockNode : public Object {
   mutable Span span;
   Array<Binding> bindings;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("span", &span);
-    v->Visit("bindings", &bindings);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BindingBlockNode>().def_ro("bindings", &BindingBlockNode::bindings);
   }
 
   bool SEqualReduce(const BindingBlockNode* other, SEqualReducer equal) const {
@@ -792,6 +787,11 @@ class BindingBlock : public ObjectRef {
 
 class DataflowBlockNode : public BindingBlockNode {
  public:
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DataflowBlockNode>();
+  }
+
   bool SEqualReduce(const DataflowBlockNode* other, SEqualReducer equal) const {
     return equal(bindings, other->bindings);
   }
@@ -820,12 +820,11 @@ class SeqExprNode : public ExprNode {
   Array<BindingBlock> blocks;
   Expr body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("blocks", &blocks);
-    v->Visit("body", &body);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<SeqExprNode>()
+        .def_ro("blocks", &SeqExprNode::blocks)
+        .def_ro("body", &SeqExprNode::body);
   }
 
   bool SEqualReduce(const SeqExprNode* other, SEqualReducer equal) const {
@@ -885,13 +884,12 @@ class IfNode : public ExprNode {
   /*! \brief The expression evaluated when condition is false */
   SeqExpr false_branch;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("cond", &cond);
-    v->Visit("true_branch", &true_branch);
-    v->Visit("false_branch", &false_branch);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<IfNode>()
+        .def_ro("cond", &IfNode::cond)
+        .def_ro("true_branch", &IfNode::true_branch)
+        .def_ro("false_branch", &IfNode::false_branch);
   }
 
   bool SEqualReduce(const IfNode* other, SEqualReducer equal) const {
@@ -959,15 +957,13 @@ class FunctionNode : public BaseFuncNode {
   /*! \brief Whether the function is annotated as pure or not. */
   bool is_pure;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("params", &params);
-    v->Visit("body", &body);
-    v->Visit("is_pure", &is_pure);
-    v->Visit("ret_struct_info", &ret_struct_info);
-    v->Visit("attrs", &attrs);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<FunctionNode>()
+        .def_ro("params", &FunctionNode::params)
+        .def_ro("body", &FunctionNode::body)
+        .def_ro("ret_struct_info", &FunctionNode::ret_struct_info)
+        .def_ro("is_pure", &FunctionNode::is_pure);
   }
 
   bool SEqualReduce(const FunctionNode* other, SEqualReducer equal) const {
@@ -1006,7 +1002,7 @@ class Function : public BaseFunc {
    *     SeqExpr.
    *
    * \param ret_struct_info The StructInfo returned by the function.
-   *     If NullOpt, will be inferred from the StructInfo of the
+   *     If std::nullopt, will be inferred from the StructInfo of the
    *     function's body.
    *
    * \param is_pure The purity of the function.
@@ -1068,11 +1064,9 @@ class ExternFuncNode : public BaseFuncNode {
   /*! \brief The name of global symbol. */
   String global_symbol;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("global_symbol", &global_symbol);
-    v->Visit("struct_info_", &struct_info_);
-    v->Visit("_checked_type_", &checked_type_);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ExternFuncNode>().def_ro("global_symbol", &ExternFuncNode::global_symbol);
   }
 
   bool SEqualReduce(const ExternFuncNode* other, SEqualReducer equal) const {
