@@ -24,7 +24,9 @@
 #ifndef TVM_RUNTIME_DATA_TYPE_H_
 #define TVM_RUNTIME_DATA_TYPE_H_
 
-#include <tvm/runtime/c_runtime_api.h>
+#include <tvm/ffi/container/shape.h>
+#include <tvm/ffi/dtype.h>
+#include <tvm/runtime/base.h>
 #include <tvm/runtime/logging.h>
 
 #include <cstring>
@@ -34,6 +36,8 @@
 namespace tvm {
 namespace runtime {
 
+using tvm_index_t = ffi::Shape::index_type;
+
 /*!
  * \brief Runtime primitive data type.
  *
@@ -54,11 +58,19 @@ class DataType {
     kInt = kDLInt,
     kUInt = kDLUInt,
     kFloat = kDLFloat,
-    kHandle = TVMArgTypeCode::kTVMOpaqueHandle,
+    kHandle = kDLOpaqueHandle,
     kBFloat = kDLBfloat,
-    kFloat8_e4m3fn = 6U,
-    kFloat8_e5m2 = 7U,
-    kFloat4_e2m1fn = 8U,
+    kFloat8_e3m4 = kDLFloat8_e3m4,
+    kFloat8_e4m3 = kDLFloat8_e4m3,
+    kFloat8_e4m3b11fnuz = kDLFloat8_e4m3b11fnuz,
+    kFloat8_e4m3fn = kDLFloat8_e4m3fn,
+    kFloat8_e4m3fnuz = kDLFloat8_e4m3fnuz,
+    kFloat8_e5m2 = kDLFloat8_e5m2,
+    kFloat8_e5m2fnuz = kDLFloat8_e5m2fnuz,
+    kFloat8_e8m0fnu = kDLFloat8_e8m0fnu,
+    kFloat6_e2m3fn = kDLFloat6_e2m3fn,
+    kFloat6_e3m2fn = kDLFloat6_e3m2fn,
+    kFloat4_e2m1fn = kDLFloat4_e2m1fn,
     kCustomBegin = 129
   };
   /*! \brief default constructor */
@@ -85,9 +97,14 @@ class DataType {
     if (code == kBFloat) {
       ICHECK_EQ(bits, 16);
     }
-    if (code == kFloat8_e4m3fn || code == kFloat8_e5m2) {
+    if (code == kFloat8_e3m4 || code == kFloat8_e4m3 || code == kFloat8_e4m3b11fnuz ||
+        code == kFloat8_e4m3fn || code == kFloat8_e4m3fnuz || code == kFloat8_e5m2 ||
+        code == kFloat8_e5m2fnuz || code == kFloat8_e8m0fnu) {
       ICHECK_EQ(bits, 8);
     }
+    if (code == kFloat6_e2m3fn || code == kFloat6_e3m2fn) {
+      ICHECK_EQ(bits, 6);
+    }
     if (code == kFloat4_e2m1fn) {
       ICHECK_EQ(bits, 4);
     }
@@ -126,17 +143,45 @@ class DataType {
   bool is_float() const { return code() == DataType::kFloat; }
   /*! \return whether type is a bfloat type. */
   bool is_bfloat() const { return code() == DataType::kBFloat; }
-  /*! \return whether type is a float8 type. */
+  /*! \return whether type is any 8-bit custom Float8 variant. */
   bool is_float8() const {
-    return (code() == DataType::kFloat || code() == DataType::kFloat8_e4m3fn ||
-            code() == DataType::kFloat8_e5m2) &&
-           bits() == 8;
+    return bits() == 8 &&
+           (code() == DataType::kFloat8_e3m4 || code() == DataType::kFloat8_e4m3 ||
+            code() == DataType::kFloat8_e4m3b11fnuz || code() == DataType::kFloat8_e4m3fn ||
+            code() == DataType::kFloat8_e4m3fnuz || code() == DataType::kFloat8_e5m2 ||
+            code() == DataType::kFloat8_e5m2fnuz || code() == DataType::kFloat8_e8m0fnu);
   }
-  /*! \return whether type is a float4 type. */
-  bool is_float4() const { return code() == DataType::kFloat4_e2m1fn && bits() == 4; }
-  bool is_float8_e4m3fn() const { return (code() == DataType::kFloat8_e4m3fn && bits() == 8); }
-  bool is_float8_e5m2() const { return (code() == DataType::kFloat8_e5m2 && bits() == 8); }
-  bool is_float4_e2m1fn() const { return (code() == DataType::kFloat4_e2m1fn && bits() == 4); }
+  /*! \return whether type is any 6-bit custom Float6 variant. */
+  bool is_float6() const {
+    return bits() == 6 &&
+           (code() == DataType::kFloat6_e2m3fn || code() == DataType::kFloat6_e3m2fn);
+  }
+  /*! \return whether type is the 4-bit custom Float4_e2m1fn variant. */
+  bool is_float4() const { return bits() == 4 && code() == DataType::kFloat4_e2m1fn; }
+  /*! \return whether type is Float8E3M4. */
+  bool is_float8_e3m4() const { return bits() == 8 && code() == DataType::kFloat8_e3m4; }
+  /*! \return whether type is Float8E4M3. */
+  bool is_float8_e4m3() const { return bits() == 8 && code() == DataType::kFloat8_e4m3; }
+  /*! \return whether type is Float8E4M3B11FNUZ. */
+  bool is_float8_e4m3b11fnuz() const {
+    return bits() == 8 && code() == DataType::kFloat8_e4m3b11fnuz;
+  }
+  /*! \return whether type is Float8E4M3FN. */
+  bool is_float8_e4m3fn() const { return bits() == 8 && code() == DataType::kFloat8_e4m3fn; }
+  /*! \return whether type is Float8E4M3FNUZ. */
+  bool is_float8_e4m3fnuz() const { return bits() == 8 && code() == DataType::kFloat8_e4m3fnuz; }
+  /*! \return whether type is Float8E5M2. */
+  bool is_float8_e5m2() const { return bits() == 8 && code() == DataType::kFloat8_e5m2; }
+  /*! \return whether type is Float8E5M2FNUZ. */
+  bool is_float8_e5m2fnuz() const { return bits() == 8 && code() == DataType::kFloat8_e5m2fnuz; }
+  /*! \return whether type is Float8E8M0FNU. */
+  bool is_float8_e8m0fnu() const { return bits() == 8 && code() == DataType::kFloat8_e8m0fnu; }
+  /*! \return whether type is Float6E2M3FN. */
+  bool is_float6_e2m3fn() const { return bits() == 6 && code() == DataType::kFloat6_e2m3fn; }
+  /*! \return whether type is Float6E3M2FN. */
+  bool is_float6_e3m2fn() const { return bits() == 6 && code() == DataType::kFloat6_e3m2fn; }
+  /*! \return whether type is Float4E2M1FN. */
+  bool is_float4_e2m1fn() const { return bits() == 4 && code() == DataType::kFloat4_e2m1fn; }
   /*! \return whether type is a float16 type. */
   bool is_float16() const { return is_float() && bits() == 16; }
   /*! \return whether type is a bfloat16 type. */
@@ -250,23 +295,83 @@ class DataType {
    */
   static DataType BFloat(int bits, int lanes = 1) { return DataType(kDLBfloat, bits, lanes); }
   /*!
-   * \brief Construct NV float8 e4m3 datatype.
+   * \brief Construct float8 e3m4 datatype.
    * \param lanes The number of lanes
    * \return The constructed data type.
    */
-  static DataType NVFloat8E4M3(int lanes = 1) { return DataType(kFloat8_e4m3fn, 8, lanes); }
+  static DataType Float8E3M4(int lanes = 1) { return DataType(kFloat8_e3m4, 8, lanes); }
+
   /*!
-   * \brief Construct NV float8 e5m2 datatype.
+   * \brief Construct float8 e4m3 datatype.
    * \param lanes The number of lanes
    * \return The constructed data type.
    */
-  static DataType NVFloat8E5M2(int lanes = 1) { return DataType(kFloat8_e5m2, 8, lanes); }
+  static DataType Float8E4M3(int lanes = 1) { return DataType(kFloat8_e4m3, 8, lanes); }
+
   /*!
-   * \brief Construct NV float4_e2m1fn datatype.
+   * \brief Construct float8 e4m3b11fnuz datatype.
    * \param lanes The number of lanes
    * \return The constructed data type.
    */
-  static DataType NVFloat4E2M1FN(int lanes = 1) { return DataType(kFloat4_e2m1fn, 4, lanes); }
+  static DataType Float8E4M3B11FNUZ(int lanes = 1) {
+    return DataType(kFloat8_e4m3b11fnuz, 8, lanes);
+  }
+
+  /*!
+   * \brief Construct float8 e4m3fn datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float8E4M3FN(int lanes = 1) { return DataType(kFloat8_e4m3fn, 8, lanes); }
+
+  /*!
+   * \brief Construct float8 e4m3fnuz datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float8E4M3FNUZ(int lanes = 1) { return DataType(kFloat8_e4m3fnuz, 8, lanes); }
+
+  /*!
+   * \brief Construct float8 e5m2 datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float8E5M2(int lanes = 1) { return DataType(kFloat8_e5m2, 8, lanes); }
+
+  /*!
+   * \brief Construct float8 e5m2fnuz datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float8E5M2FNUZ(int lanes = 1) { return DataType(kFloat8_e5m2fnuz, 8, lanes); }
+
+  /*!
+   * \brief Construct float8 e8m0fnu datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float8E8M0FNU(int lanes = 1) { return DataType(kFloat8_e8m0fnu, 8, lanes); }
+
+  /*!
+   * \brief Construct float6 e2m3fn datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float6E2M3FN(int lanes = 1) { return DataType(kFloat6_e2m3fn, 6, lanes); }
+
+  /*!
+   * \brief Construct float6 e3m2fn datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float6E3M2FN(int lanes = 1) { return DataType(kFloat6_e3m2fn, 6, lanes); }
+
+  /*!
+   * \brief Construct float4 e2m1fn datatype.
+   * \param lanes The number of lanes
+   * \return The constructed data type.
+   */
+  static DataType Float4E2M1FN(int lanes = 1) { return DataType(kFloat4_e2m1fn, 4, lanes); }
   /*!
    * \brief Construct a bool type.
    * \param lanes The number of lanes.
@@ -313,7 +418,8 @@ inline int GetVectorBytes(DataType dtype) {
   int data_bits = dtype.bits() * dtype.lanes();
   // allow bool to exist
   if (dtype == DataType::Bool() || dtype == DataType::Int(4) || dtype == DataType::UInt(4) ||
-      dtype == DataType::Int(1) || dtype == DataType::NVFloat4E2M1FN()) {
+      dtype == DataType::Int(1) || dtype == DataType::Float4E2M1FN() ||
+      dtype == DataType::Float6E2M3FN() || dtype == DataType::Float6E3M2FN()) {
     return 1;
   }
   ICHECK_EQ(data_bits % 8, 0U) << "Need to load/store by multiple of bytes";
@@ -339,206 +445,57 @@ inline bool TypeEqual(DLDataType lhs, DLDataType rhs) {
   return lhs.code == rhs.code && lhs.bits == rhs.bits && lhs.lanes == rhs.lanes;
 }
 
-/*!
- * \brief Runtime utility for getting custom type name from code
- * \param type_code Custom type code
- * \return Custom type name
- */
-TVM_DLL std::string GetCustomTypeName(uint8_t type_code);
-
-/*!
- * \brief Runtime utility for checking whether custom type is registered
- * \param type_code Custom type code
- * \return Bool representing whether type is registered
- */
-TVM_DLL bool GetCustomTypeRegistered(uint8_t type_code);
-
-/*!
- * \brief Runtime utility for parsing string of the form "custom[<typename>]"
- * \param s String to parse
- * \param scan pointer to parsing pointer, which is scanning across s
- * \return type code of custom type parsed
- */
-TVM_DLL uint8_t ParseCustomDatatype(const std::string& s, const char** scan);
-
-/*!
- * \brief Convert type code to its name
- * \param type_code The type code .
- * \return The name of type code.
- */
-inline const char* DLDataTypeCode2Str(DLDataTypeCode type_code);
-
-/*!
- * \brief convert a string to TVM type.
- * \param s The string to be converted.
- * \return The corresponding tvm type.
- */
-inline DLDataType String2DLDataType(std::string s);
-
-/*!
- * \brief convert a TVM type to string.
- * \param t The type to be converted.
- * \return The corresponding tvm type in string.
- */
-inline std::string DLDataType2String(DLDataType t);
-
-// implementation details
-inline const char* DLDataTypeCode2Str(DLDataTypeCode type_code) {
-  switch (static_cast<int>(type_code)) {
-    case kDLInt:
-      return "int";
-    case kDLUInt:
-      return "uint";
-    case kDLFloat:
-      return "float";
-    case DataType::kHandle:
-      return "handle";
-    case kDLBfloat:
-      return "bfloat";
-    case DataType::kFloat8_e4m3fn:
-      return "float8_e4m3fn";
-    case DataType::kFloat8_e5m2:
-      return "float8_e5m2";
-    case DataType::kFloat4_e2m1fn:
-      return "float4_e2m1fn";
-    default:
-      LOG(FATAL) << "unknown type_code=" << static_cast<int>(type_code);
-  }
-  throw;
-}
-
-inline std::ostream& operator<<(std::ostream& os, DLDataType t) {  // NOLINT(*)
-  if (t.bits == 1 && t.lanes == 1 && t.code == kDLUInt) {
-    os << "bool";
-    return os;
-  }
-  if (DataType(t).is_void()) {
-    return os << "void";
-  }
-  if (t.code < DataType::kCustomBegin) {
-    os << DLDataTypeCode2Str(static_cast<DLDataTypeCode>(t.code));
-  } else {
-    os << "custom[" << GetCustomTypeName(t.code) << "]";
-  }
-  if (t.code == kTVMOpaqueHandle) return os;
-  int16_t lanes = static_cast<int16_t>(t.lanes);
-  if (t.code != DataType::kFloat8_e4m3fn && t.code != DataType::kFloat8_e5m2 &&
-      t.code != DataType::kFloat4_e2m1fn) {
-    os << static_cast<int>(t.bits);
-  }
-  if (lanes > 1) {
-    os << 'x' << lanes;
-  } else if (lanes < -1) {
-    os << "xvscalex" << -lanes;
-  }
-  return os;
-}
+using ffi::DLDataTypeToString;
+using ffi::StringToDLDataType;
 
 inline std::ostream& operator<<(std::ostream& os, const DataType& dtype) {  // NOLINT(*)
   return os << dtype.operator DLDataType();
 }
-
-inline std::string DLDataType2String(DLDataType t) {
-  if (t.bits == 0) return "";
-  std::ostringstream os;
-  os << t;
-  return os.str();
-}
-
-inline DLDataType String2DLDataType(std::string s) {
-  DLDataType t;
-  // handle void type
-  if (s.length() == 0 || s == "void") {
-    t = DataType::Void();
-    return t;
-  }
-  t.bits = 32;
-  t.lanes = 1;
-  const char* scan;
-  if (s.substr(0, 3) == "int") {
-    t.code = kDLInt;
-    scan = s.c_str() + 3;
-  } else if (s.substr(0, 4) == "uint") {
-    t.code = kDLUInt;
-    scan = s.c_str() + 4;
-  } else if (s.substr(0, 13) == "float4_e2m1fn") {
-    // Avoid being treated as "float"
-    t.code = DataType::kFloat4_e2m1fn;
-    t.bits = 4;
-    scan = s.c_str() + 13;
-    char* endpt = nullptr;
-    if (*scan == 'x') {
-      t.lanes = static_cast<uint16_t>(strtoul(scan + 1, &endpt, 10));
-      scan = endpt;
-    }
-    ICHECK(scan == s.c_str() + s.length()) << "unknown type " << s;
-    return t;
-  } else if (s.substr(0, 13) == "float8_e4m3fn") {
-    // Avoid being treated as "float"
-    t.code = DataType::kFloat8_e4m3fn;
-    t.bits = 8;
-    scan = s.c_str() + 13;
-    char* endpt = nullptr;
-    if (*scan == 'x') {
-      t.lanes = static_cast<uint16_t>(strtoul(scan + 1, &endpt, 10));
-      scan = endpt;
-    }
-    ICHECK(scan == s.c_str() + s.length()) << "unknown type " << s;
-    return t;
-  } else if (s.substr(0, 11) == "float8_e5m2") {
-    // Avoid being treated as "float"
-    t.code = DataType::kFloat8_e5m2;
-    t.bits = 8;
-    scan = s.c_str() + 11;
-    char* endpt = nullptr;
-    if (*scan == 'x') {
-      t.lanes = static_cast<uint16_t>(strtoul(scan + 1, &endpt, 10));
-      scan = endpt;
-    }
-    ICHECK(scan == s.c_str() + s.length()) << "unknown type " << s;
-    return t;
-  } else if (s.substr(0, 5) == "float") {
-    t.code = kDLFloat;
-    scan = s.c_str() + 5;
-  } else if (s.substr(0, 6) == "handle") {
-    t.code = kTVMOpaqueHandle;
-    t.bits = 64;  // handle uses 64 bit by default.
-    scan = s.c_str() + 6;
-  } else if (s == "bool") {
-    t.code = kDLUInt;
-    t.bits = 1;
-    t.lanes = 1;
-    return t;
-  } else if (s.substr(0, 6) == "bfloat") {
-    t.code = DataType::kBFloat;
-    t.bits = 16;
-    scan = s.c_str() + 6;
-  } else if (s.substr(0, 6) == "custom") {
-    t.code = ParseCustomDatatype(s, &scan);
-  } else {
-    scan = s.c_str();
-    LOG(FATAL) << "unknown type " << s;
-  }
-  char* xdelim;  // emulate sscanf("%ux%u", bits, lanes)
-  uint8_t bits = static_cast<uint8_t>(strtoul(scan, &xdelim, 10));
-  if (bits != 0) t.bits = bits;
-  int scalable_multiplier = 1;
-  if (strncmp(xdelim, "xvscale", 7) == 0) {
-    scalable_multiplier = -1;
-    xdelim += 7;
-  }
-  char* endpt = xdelim;
-  if (*xdelim == 'x') {
-    t.lanes = static_cast<uint16_t>(scalable_multiplier * strtoul(xdelim + 1, &endpt, 10));
-  }
-  ICHECK(endpt == s.c_str() + s.length()) << "unknown type " << s;
-  return t;
-}
-
 }  // namespace runtime
 
 using DataType = runtime::DataType;
 
+namespace ffi {
+
+// runtime::DataType
+template <>
+struct TypeTraits<runtime::DataType> : public TypeTraitsBase {
+  static constexpr int32_t field_static_type_index = TypeIndex::kTVMFFIDataType;
+
+  TVM_FFI_INLINE static void CopyToAnyView(const runtime::DataType& src, TVMFFIAny* result) {
+    // clear padding part to ensure the equality check can always check the v_uint64 part
+    result->v_uint64 = 0;
+    result->type_index = TypeIndex::kTVMFFIDataType;
+    result->v_dtype = src;
+  }
+
+  TVM_FFI_INLINE static void MoveToAny(runtime::DataType src, TVMFFIAny* result) {
+    // clear padding part to ensure the equality check can always check the v_uint64 part
+    result->v_uint64 = 0;
+    result->type_index = TypeIndex::kTVMFFIDataType;
+    result->v_dtype = src;
+  }
+
+  TVM_FFI_INLINE static std::optional<runtime::DataType> TryCastFromAnyView(const TVMFFIAny* src) {
+    auto opt_dtype = TypeTraits<DLDataType>::TryCastFromAnyView(src);
+    if (opt_dtype) {
+      return runtime::DataType(opt_dtype.value());
+    }
+    return std::nullopt;
+  }
+
+  TVM_FFI_INLINE static bool CheckAnyStrict(const TVMFFIAny* src) {
+    return TypeTraits<DLDataType>::CheckAnyStrict(src);
+  }
+
+  TVM_FFI_INLINE static runtime::DataType CopyFromAnyViewAfterCheck(const TVMFFIAny* src) {
+    return runtime::DataType(TypeTraits<DLDataType>::CopyFromAnyViewAfterCheck(src));
+  }
+
+  TVM_FFI_INLINE static std::string TypeStr() { return ffi::StaticTypeKey::kTVMFFIDataType; }
+};
+
+}  // namespace ffi
 }  // namespace tvm
 
 namespace std {
