@@ -24,14 +24,59 @@
 #ifndef TVM_RUNTIME_DEVICE_API_H_
 #define TVM_RUNTIME_DEVICE_API_H_
 
-#include <tvm/runtime/c_runtime_api.h>
-#include <tvm/runtime/ndarray.h>
-#include <tvm/runtime/packed_func.h>
+#include <tvm/ffi/any.h>
+#include <tvm/ffi/optional.h>
+#include <tvm/runtime/base.h>
+#include <tvm/runtime/logging.h>
 
 #include <string>
+/*!
+ * \brief The stream that is specific to device
+ * can be NULL, which indicates the default one.
+ */
+typedef void* TVMStreamHandle;
 
 namespace tvm {
+
+// alias DLDevice
+using Device = DLDevice;
+
 namespace runtime {
+
+/*! \brief Extension device types in TVM
+ *
+ * Additional enumerators to supplement those provided by
+ * DLPack's `DLDeviceType` enumeration.
+ *
+ * MAINTAINERS NOTE #1: We need to ensure that the two devices
+ * are identified by the same integer.
+ * Currently this requires manual verification.
+ * Discussed here: https://github.com/dmlc/dlpack/issues/111
+ * As of DLPack v0.7, the highest-valued enumerator in
+ * `DLDeviceType` is kDLHexagon = 16.
+ *
+ * MAINTAINERS NOTE #2: As of DLPack v0.7, the definition for
+ * `DLDeviceType` specifies an underlying storage type of
+ * `int32_t`.  That guarantees a variable of type
+ * `DLDeviceType` is capable of holding any integers provided
+ * by *either* of these enumerations.
+ *
+ * However, the `int32_t` specification only applies when the
+ * header file is compiled as C++, and this header file is also
+ * meant to work as C code.  So the unspecified storage type
+ * could be a latent bug when compiled as C.
+ */
+#ifdef __cplusplus
+typedef enum : int32_t {
+#else
+typedef enum {
+#endif
+  // To help avoid accidental conflicts between `DLDeviceType`
+  // and this enumeration, start numbering the new enumerators
+  // a little higher than (currently) seems necessary.
+  TVMDeviceExtType_End = 36,  // sentinel value
+} TVMDeviceExtType;
+
 /*!
  * \brief the query type into GetAttr
  */
@@ -96,7 +141,7 @@ class TVM_DLL DeviceAPI {
    * \param rv The return value.
    * \sa DeviceAttrKind
    */
-  virtual void GetAttr(Device dev, DeviceAttrKind kind, TVMRetValue* rv) = 0;
+  virtual void GetAttr(Device dev, DeviceAttrKind kind, ffi::Any* rv) = 0;
 
   /*!
    * \brief Get the physical memory size required.
@@ -104,7 +149,8 @@ class TVM_DLL DeviceAPI {
    * \param mem_scope the memory scope if any
    * \return the memory size.
    */
-  virtual size_t GetDataSize(const DLTensor& arr, Optional<String> mem_scope = NullOpt);
+  virtual size_t GetDataSize(const DLTensor& arr,
+                             ffi::Optional<ffi::String> mem_scope = std::nullopt);
 
   /*!
    * \brief Query the device for specified properties.
@@ -112,7 +158,7 @@ class TVM_DLL DeviceAPI {
    * This is used to expand "-from_device=N" in the target string to
    * all properties that can be determined from that device.
    */
-  virtual void GetTargetProperty(Device dev, const std::string& property, TVMRetValue* rv) {}
+  virtual void GetTargetProperty(Device dev, const std::string& property, ffi::Any* rv) {}
 
   /*!
    * \brief Allocate a data space on device.
@@ -135,7 +181,7 @@ class TVM_DLL DeviceAPI {
    * \return The allocated device pointer.
    */
   virtual void* AllocDataSpace(Device dev, int ndim, const int64_t* shape, DLDataType dtype,
-                               Optional<String> mem_scope = NullOpt);
+                               ffi::Optional<ffi::String> mem_scope = std::nullopt);
   /*!
    * \brief Free a data space on device.
    * \param dev The device device to perform operation.
@@ -263,6 +309,47 @@ class TVM_DLL DeviceAPI {
                               DLDataType type_hint, TVMStreamHandle stream);
 };
 
+/*!
+ * \brief The name of DLDeviceType.
+ * \param type The device type.
+ * \return the device name.
+ */
+inline const char* DLDeviceType2Str(int type) {
+  switch (type) {
+    case kDLCPU:
+      return "cpu";
+    case kDLCUDA:
+      return "cuda";
+    case kDLCUDAHost:
+      return "cuda_host";
+    case kDLCUDAManaged:
+      return "cuda_managed";
+    case kDLOpenCL:
+      return "opencl";
+    case kDLVulkan:
+      return "vulkan";
+    case kDLMetal:
+      return "metal";
+    case kDLVPI:
+      return "vpi";
+    case kDLROCM:
+      return "rocm";
+    case kDLROCMHost:
+      return "rocm_host";
+    case kDLExtDev:
+      return "ext_dev";
+    case kDLOneAPI:
+      return "oneapi";
+    case kDLWebGPU:
+      return "webgpu";
+    case kDLHexagon:
+      return "hexagon";
+    default:
+      LOG(FATAL) << "unknown type = " << type;
+  }
+  throw;
+}
+
 /*! \brief The device type bigger than this is RPC device */
 constexpr int kRPCSessMask = 128;
 static_assert(kRPCSessMask >= TVMDeviceExtType_End);
