@@ -19,6 +19,7 @@
 #ifndef TVM_SCRIPT_IR_BUILDER_RELAX_FRAME_H_
 #define TVM_SCRIPT_IR_BUILDER_RELAX_FRAME_H_
 
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/relax/block_builder.h>
 #include <tvm/relax/expr.h>
 #include <tvm/script/ir_builder/base.h>
@@ -33,7 +34,10 @@ namespace relax {
 /*! \brief The base ir_builder frame for the relax dialect. */
 class RelaxFrameNode : public IRBuilderFrameNode {
  public:
-  void VisitAttrs(tvm::AttrVisitor* v) { IRBuilderFrameNode::VisitAttrs(v); }
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<RelaxFrameNode>();
+  }
 
   static constexpr const char* _type_key = "script.ir_builder.relax.RelaxFrame";
   TVM_DECLARE_BASE_OBJECT_INFO(RelaxFrameNode, IRBuilderFrameNode);
@@ -54,13 +58,14 @@ class SeqExprFrameNode : public RelaxFrameNode {
  public:
   /*! \brief The binding blocks inside the frame. */
   Array<tvm::relax::BindingBlock> binding_blocks;
-  /*! \brief The frame output expr. `NullOpt` when undefined. */
+  /*! \brief The frame output expr. `std::nullopt` when undefined. */
   Optional<tvm::relax::Expr> output;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    RelaxFrameNode::VisitAttrs(v);
-    v->Visit("binding_blocks", &binding_blocks);
-    v->Visit("output", &output);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<SeqExprFrameNode>()
+        .def_ro("binding_blocks", &SeqExprFrameNode::binding_blocks)
+        .def_ro("output", &SeqExprFrameNode::output);
   }
 
   static constexpr const char* _type_key = "script.ir_builder.relax.SeqExprFrame";
@@ -102,20 +107,21 @@ class FunctionFrameNode : public SeqExprFrameNode {
   /*! \brief Whether the function is annotated as private */
   Optional<Bool> is_private;
   /*! \brief The function attributes. */
-  Map<String, ObjectRef> attrs;
+  Map<String, Any> attrs;
   /*! \brief The block builder to create Relax function. */
   tvm::relax::BlockBuilder block_builder;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    SeqExprFrameNode::VisitAttrs(v);
-    v->Visit("name", &name);
-    v->Visit("params", &params);
-    v->Visit("ret_struct_info", &ret_struct_info);
-    v->Visit("is_pure", &is_pure);
-    v->Visit("attrs", &attrs);
-    v->Visit("binding_blocks", &binding_blocks);
-    v->Visit("output", &output);
-    // `block_builder` is not visited.
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<FunctionFrameNode>()
+        .def_ro("name", &FunctionFrameNode::name)
+        .def_ro("params", &FunctionFrameNode::params)
+        .def_ro("ret_struct_info", &FunctionFrameNode::ret_struct_info)
+        .def_ro("is_pure", &FunctionFrameNode::is_pure)
+        .def_ro("attrs", &FunctionFrameNode::attrs)
+        .def_ro("binding_blocks", &FunctionFrameNode::binding_blocks)
+        .def_ro("output", &FunctionFrameNode::output);
+    // `block_builder` is not registered as it's not visited.
   }
 
   static constexpr const char* _type_key = "script.ir_builder.relax.FunctionFrame";
@@ -150,12 +156,13 @@ class BlockFrameNode : public RelaxFrameNode {
    */
   Array<tvm::relax::Var> output_vars;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    RelaxFrameNode::VisitAttrs(v);
-    v->Visit("is_dataflow", &is_dataflow);
-    v->Visit("emitted_vars", &emitted_vars);
-    v->Visit("output_vars", &output_vars);
-    // `block_ended` is not visited.
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BlockFrameNode>()
+        .def_ro("is_dataflow", &BlockFrameNode::is_dataflow)
+        .def_ro("emitted_vars", &BlockFrameNode::emitted_vars)
+        .def_ro("output_vars", &BlockFrameNode::output_vars);
+    // `block_ended` is not registered as it's not visited.
   }
 
   static constexpr const char* _type_key = "script.ir_builder.relax.BlockFrame";
@@ -189,13 +196,14 @@ class IfFrameNode : public RelaxFrameNode {
   /*! \brief The binding var name. */
   String var_name;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    RelaxFrameNode::VisitAttrs(v);
-    v->Visit("condition", &condition);
-    v->Visit("then_expr", &then_expr);
-    v->Visit("else_expr", &else_expr);
-    v->Visit("var", &var);
-    v->Visit("var_name", &var_name);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<IfFrameNode>()
+        .def_ro("condition", &IfFrameNode::condition)
+        .def_ro("then_expr", &IfFrameNode::then_expr)
+        .def_ro("else_expr", &IfFrameNode::else_expr)
+        .def_ro("var", &IfFrameNode::var)
+        .def_ro("var_name", &IfFrameNode::var_name);
   }
 
   static constexpr const char* _type_key = "script.ir_builder.relax.IfFrame";
@@ -231,6 +239,11 @@ class IfFrame : public RelaxFrame {
  */
 class ThenFrameNode : public SeqExprFrameNode {
  public:
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ThenFrameNode>();
+  }
+
   static constexpr const char* _type_key = "script.ir_builder.relax.ThenFrame";
   TVM_DECLARE_FINAL_OBJECT_INFO(ThenFrameNode, SeqExprFrameNode);
 
@@ -264,6 +277,11 @@ class ThenFrame : public SeqExprFrame {
  */
 class ElseFrameNode : public SeqExprFrameNode {
  public:
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ElseFrameNode>();
+  }
+
   static constexpr const char* _type_key = "script.ir_builder.relax.ElseFrame";
   TVM_DECLARE_FINAL_OBJECT_INFO(ElseFrameNode, SeqExprFrameNode);
 
