@@ -25,6 +25,7 @@
 #define TVM_TE_TENSOR_H_
 
 #include <tvm/arith/bound.h>
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/tir/expr.h>
 #include <tvm/tir/op.h>
 
@@ -75,23 +76,19 @@ class TensorNode : public DataProducerNode {
   Operation op;
   /*! \brief the output index from source operation */
   int value_index{0};
-  /*! \brief constructor */
-  TensorNode() {}
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("shape", &shape);
-    v->Visit("dtype", &dtype);
-    v->Visit("op", &op);
-    v->Visit("value_index", &value_index);
-  }
+  static void RegisterReflection();
 
   Array<PrimExpr> GetShape() const final { return shape; }
 
   DataType GetDataType() const final { return dtype; }
 
+  TVM_DLL PrimExpr ToPrimExpr() const final;
+
   TVM_DLL String GetNameHint() const final;
 
-  static constexpr const char* _type_key = "Tensor";
+  static constexpr const char* _type_key = "te.Tensor";
+
   TVM_DECLARE_FINAL_OBJECT_INFO(TensorNode, DataProducerNode);
 };
 
