@@ -24,10 +24,11 @@
 #ifndef TVM_TIR_BUFFER_H_
 #define TVM_TIR_BUFFER_H_
 
+#include <tvm/ffi/container/array.h>
+#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/string.h>
 #include <tvm/ir/expr.h>
 #include <tvm/node/script_printer.h>
-#include <tvm/runtime/container/array.h>
-#include <tvm/runtime/container/string.h>
 #include <tvm/tir/var.h>
 
 #include <string>
@@ -111,18 +112,20 @@ class BufferNode : public Object {
   /*! \brief constructor */
   BufferNode() {}
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("data", &data);
-    v->Visit("dtype", &dtype);
-    v->Visit("shape", &shape);
-    v->Visit("strides", &strides);
-    v->Visit("axis_separators", &axis_separators);
-    v->Visit("elem_offset", &elem_offset);
-    v->Visit("name", &name);
-    v->Visit("data_alignment", &data_alignment);
-    v->Visit("offset_factor", &offset_factor);
-    v->Visit("buffer_type", &buffer_type);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BufferNode>()
+        .def_ro("data", &BufferNode::data)
+        .def_ro("dtype", &BufferNode::dtype)
+        .def_ro("shape", &BufferNode::shape)
+        .def_ro("strides", &BufferNode::strides)
+        .def_ro("axis_separators", &BufferNode::axis_separators)
+        .def_ro("elem_offset", &BufferNode::elem_offset)
+        .def_ro("name", &BufferNode::name)
+        .def_ro("data_alignment", &BufferNode::data_alignment)
+        .def_ro("offset_factor", &BufferNode::offset_factor)
+        .def_ro("buffer_type", &BufferNode::buffer_type)
+        .def_ro("span", &BufferNode::span);
   }
 
   bool SEqualReduce(const BufferNode* other, SEqualReducer equal) const {
@@ -204,7 +207,7 @@ class Buffer : public ObjectRef {
    */
   TVM_DLL PrimExpr access_ptr(int access_mask, DataType ptr_type = DataType::Handle(),
                               int content_lanes = 1, PrimExpr offset = IntImm(DataType::Int(32), 0),
-                              Optional<PrimExpr> input_extent = NullOpt) const;
+                              Optional<PrimExpr> input_extent = std::nullopt) const;
   /*!
    * \brief Create an Expr that does a vector load at begin index.
    * \param begin The beginning index
@@ -213,7 +216,7 @@ class Buffer : public ObjectRef {
    * loaded. The number lanes of the mask must be equal to the number of lanes in being loaded.
    */
   TVM_DLL PrimExpr vload(Array<PrimExpr> begin, DataType dtype,
-                         Optional<PrimExpr> predicate = NullOpt) const;
+                         Optional<PrimExpr> predicate = std::nullopt) const;
   /*!
    * \brief Create a Stmt that does a vector store at begin index.
    * \param begin The beginning index
@@ -222,7 +225,7 @@ class Buffer : public ObjectRef {
    * stored. The number lanes of the mask must be equal to the number of lanes in value.
    */
   TVM_DLL Stmt vstore(Array<PrimExpr> begin, PrimExpr value,
-                      Optional<PrimExpr> predicate = NullOpt) const;
+                      Optional<PrimExpr> predicate = std::nullopt) const;
 
   /*!
    * \brief Get a flattened version of the buffer
@@ -259,7 +262,8 @@ class Buffer : public ObjectRef {
  */
 TVM_DLL Buffer decl_buffer(Array<PrimExpr> shape, DataType dtype = DataType::Float(32),
                            String name = "buffer", String storage_scope = "",
-                           Array<IntImm> axis_separators = {}, Span span = Span());
+                           Optional<Array<IntImm>> axis_separators = std::nullopt,
+                           Span span = Span());
 
 /*!
  * \brief Base node for data producers.
@@ -273,7 +277,7 @@ TVM_DLL Buffer decl_buffer(Array<PrimExpr> shape, DataType dtype = DataType::Flo
  *
  * \sa tvm::te::Tensor
  */
-class DataProducerNode : public Object {
+class DataProducerNode : public PrimExprConvertibleNode {
  public:
   /*! \brief destructor. */
   virtual ~DataProducerNode() {}
@@ -303,16 +307,16 @@ class DataProducerNode : public Object {
   static constexpr const char* _type_key = "tir.DataProducer";
   static constexpr const bool _type_has_method_sequal_reduce = true;
   static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_BASE_OBJECT_INFO(DataProducerNode, Object);
+  TVM_DECLARE_BASE_OBJECT_INFO(DataProducerNode, PrimExprConvertibleNode);
 };
 
 /*!
  * \brief Managed reference to DataProducerNode.
  * \sa DataProducerNode
  */
-class DataProducer : public ObjectRef {
+class DataProducer : public PrimExprConvertible {
  public:
-  TVM_DEFINE_OBJECT_REF_METHODS(DataProducer, ObjectRef, DataProducerNode);
+  TVM_DEFINE_OBJECT_REF_METHODS(DataProducer, PrimExprConvertible, DataProducerNode);
 };
 
 /*!
