@@ -263,6 +263,15 @@ TVM_DLL const Op& tvm_context_id();
  */
 TVM_DLL const Op& tvm_tuple();
 
+/*!
+ * \brief See pesudo code
+ *
+ *  void* handle_add_byte_offset(void* handle, int offset) {
+ *     return reinterpret_cast<v*>(reinterpret_cast<char*>(handle) + offset);
+ *  }
+ */
+TVM_DLL const Op& handle_add_byte_offset();
+
 /*!
  * \brief See pesudo code
  *
@@ -347,14 +356,13 @@ TVM_DLL const Op& tvm_stack_make_array();
 /*!
  * \brief See pesudo code
  *
- *  return_type tvm_call_packed(name, TVMValue* args) {
- *     TVMValue ret_value;
- *     int ret_code;
+ *  return_type tvm_call_packed(name, TVMFFIAny* args) {
+ *     TVMFFIAny result;
  *     ModuleNode* env = GetCurrentEnv();
- *     const PackedFunc* f = env->GetFuncFromEnv(name);
- *     (*f)(args, type_code_of(args), len(args), &ret_value, &ret_code);
+ *     const ffi::Function* f = env->GetFuncFromEnv(name);
+ *     (*f)(args, args, len(args), &result);
  *     // return type can be int, float, handle.
- *     return cast(return_type, ret_value.v_return_type);
+ *     return cast(return_type, result);
  *  }
  */
 TVM_DLL const Op& tvm_call_packed();
@@ -362,11 +370,10 @@ TVM_DLL const Op& tvm_call_packed();
 /*!
  * \brief See pesudo code
  *
- * return_type tvm_call_packed(fname, TVMValue* args) {
- * 	   int ret_code;
- *     TVMValue ret_value;
- *     (*fname)(args, type_code_of(args), len(args), &ret_value, &ret_code);
- *     return cast(return_type, ret_value.v_return_type);
+ * return_type tvm_call_packed(fname, TVMFFIAny* args) {
+ *     TVMFFIAny result;
+ *     (*fname)(args, args, len(args), &result);
+ *     return cast(return_type, result);
  *  }
  */
 TVM_DLL const Op& tvm_call_cpacked();
@@ -374,30 +381,16 @@ TVM_DLL const Op& tvm_call_cpacked();
 /*!
  * \brief See pesudo code
  *
- *  return_type tvm_call_trace_packed(name, TVMValue* args) {
+ *  return_type tvm_call_trace_packed(name, TVMFFIAny* args) {
  *     ModuleNode* env = GetCurrentEnv();
- *     const PackedFunc* f = env->GetFuncFromEnv(name);
- *     (*f)(args, type_code_of(args), len(args));
+ *     const ffi::Function* f = env->GetFuncFromEnv(name);
+ *     (*f)(args, args, len(args));
  *     // return type can be int, float, handle.
- *     return cast(return_type, ret_value.v_return_type);
+ *     return cast(return_type, result);
  *  }
  */
 TVM_DLL const Op& tvm_call_trace_packed();
 
-/*!
- * \brief Checks the return value of another call is correct or returns a given value.
- *
- * \note  This is meant to serve a specific case for AOT code generator whilst this
- *        cannot be fully represented in TIR.
- *
- *  Type tvm_check_return(expected, return_unexpected, nested_call) {
- *     if (nested_call() != expected) {
- *         return return_unexpected;
- *     }
- *  }
- */
-TVM_DLL const Op& tvm_check_return();
-
 /*!
  * \brief See pesudo code
  *  Mark the content as thread local context, can get optimized
@@ -422,17 +415,15 @@ TVM_DLL const Op& tvm_thread_invariant();
  *  type codes are explicitly allocated.
  *
  *  return_type tvm_call_packed_lowered(name,
- *                                      TVMValue* value_stack,
- *                                      int* tcode_stack,
+ *                                      TVMFFIAny* args_stack,
  *                                      int begin,
  *                                      int end) {
  *     ModuleNode* env = GetCurrentEnv();
- *     const PackedFunc* f = env->GetFuncFromEnv(name);
- *     f->CallPacked(TVMArgs(value_stack[begin:end],
- *                           tcode_stack[begin:end]),
- *                   TVMRetValue(value_stack + end, tcode_stack + end));
+ *     const ffi::Function* f = env->GetFuncFromEnv(name);
+ *     f->CallPacked(ffi::PackedArgs(args_stack[begin:end]),
+ *                   ffi::Any(args_stack + end));
  *     // return type can be int, float, handle.
- *     return cast(return_type, load_return_from(tcode_stack + end))
+ *     return cast(return_type, load_return_from(args_stack + end))
  *  }
  */
 TVM_DLL const Op& tvm_call_packed_lowered();
@@ -442,12 +433,12 @@ TVM_DLL const Op& tvm_call_packed_lowered();
  *  type codes are explicitly allocated.
  *
  *  int tvm_call_packed_lowered(fname,
- *                              TVMValue* value_stack,
- *                              int* tcode_stack,
+ *                              TVMFFIAny* args_stack,
  *                              int begin,
- *                              int end) {
- *     fname(TVMArgs(value_stack[begin:end], tcode_stack[begin:end]),
- *                   TVMRetValue(value_stack + end, tcode_stack + end));
+ *                              int end,
+ *                              void* self) {
+ *     fname(ffi::PackedArgs(value_stack[begin:end], tcode_stack[begin:end]),
+ *                   ffi::Any(value_stack + end, tcode_stack + end));
  *  }
  */
 TVM_DLL const Op& tvm_call_cpacked_lowered();
@@ -458,17 +449,15 @@ TVM_DLL const Op& tvm_call_cpacked_lowered();
  *  (end - 1) value on the stack.
  *
  *  return_type tvm_call_trace_packed_lowered(name,
- *                                            TVMValue* value_stack,
- *                                            int* tcode_stack,
+ *                                            TVMFFIAny* args_stack,
  *                                            int begin,
  *                                            int end) {
  *     ModuleNode* env = GetCurrentEnv();
- *     const PackedFunc* f = env->GetFuncFromEnv(name);
- *     f->CallPacked(TVMArgs(value_stack[begin:end],
- *                           tcode_stack[begin:end]),
- *                   TVMRetValue(value_stack + end, tcode_stack + end));
+ *     const ffi::Function* f = env->GetFuncFromEnv(name);
+ *     f->CallPacked(ffi::PackedArgs(args_stack[begin:end]),
+ *                   ffi::Any(args_stack + end));
  *     // return type can be int, float, handle.
- *     return cast(return_type, load_return_from(tcode_stack + end))
+ *     return cast(return_type, load_return_from(args_stack + end))
  *  }
  */
 TVM_DLL const Op& tvm_call_trace_packed_lowered();
@@ -990,6 +979,8 @@ enum TVMStructFieldKind : int {
   kArrKindBound_,
   // TVMValue field
   kTVMValueContent,
+  kTVMFFIAnyTypeIndex,
+  kTVMFFIAnyUnionValue,
   kTVMValueKindBound_
 };
 }  // namespace builtin
