@@ -24,6 +24,8 @@
 #ifndef TVM_TIR_FUNCTION_H_
 #define TVM_TIR_FUNCTION_H_
 
+#include <tvm/ffi/container/map.h>
+#include <tvm/ffi/container/variant.h>
 #include <tvm/ir/function.h>
 #include <tvm/runtime/ndarray.h>
 #include <tvm/tir/buffer.h>
@@ -98,14 +100,13 @@ class PrimFuncNode : public BaseFuncNode {
    */
   Map<tir::Var, Buffer> buffer_map;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("params", &params);
-    v->Visit("body", &body);
-    v->Visit("ret_type", &ret_type);
-    v->Visit("buffer_map", &buffer_map);
-    v->Visit("attrs", &attrs);
-    v->Visit("span", &span);
-    v->Visit("_checked_type_", &checked_type_);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<PrimFuncNode>()
+        .def_ro("params", &PrimFuncNode::params)
+        .def_ro("body", &PrimFuncNode::body)
+        .def_ro("ret_type", &PrimFuncNode::ret_type)
+        .def_ro("buffer_map", &PrimFuncNode::buffer_map);
   }
 
   bool SEqualReduce(const PrimFuncNode* other, SEqualReducer equal) const {
@@ -179,9 +180,11 @@ class TensorIntrinNode : public Object {
   /*! \brief The function of the implementation for the execution. */
   PrimFunc impl;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("desc", &desc);
-    v->Visit("impl", &impl);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<TensorIntrinNode>()
+        .def_ro("desc", &TensorIntrinNode::desc)
+        .def_ro("impl", &TensorIntrinNode::impl);
   }
 
   static constexpr const char* _type_key = "tir.TensorIntrin";
@@ -222,7 +225,7 @@ class TensorIntrin : public ObjectRef {
    */
   TVM_DLL static Optional<TensorIntrin> Get(String name, bool allow_missing = false);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(TensorIntrin, ObjectRef, TensorIntrinNode)
+  TVM_DEFINE_OBJECT_REF_METHODS(TensorIntrin, ObjectRef, TensorIntrinNode);
 };
 
 /*!
@@ -286,7 +289,7 @@ namespace attr {
  * Here n = len(arg), m = len(work_size) = len(launch_params)-1.
  *
  * The list of kernel launch params indicates which additional
- * parameters will be provided to the PackedFunc by the calling
+ * parameters will be provided to the ffi::Function by the calling
  * scope.
  *
  * - "threadIdx.x", "threadIdx.y", "threadIdx.z"
