@@ -24,12 +24,12 @@
 #ifndef TVM_TIR_STMT_H_
 #define TVM_TIR_STMT_H_
 
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/tir/expr.h>
 
 #include <string>
 #include <type_traits>
 #include <utility>
-#include <vector>
 
 namespace tvm {
 namespace tir {
@@ -46,6 +46,11 @@ class StmtNode : public Object {
   StmtNode() = default;
   explicit StmtNode(Span span) : span(span) {}
 
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<StmtNode>().def_ro("span", &StmtNode::span);
+  }
+
   TVM_OBJECT_ENABLE_SCRIPT_PRINTER();
 
   static constexpr const char* _type_key = "tir.Stmt";
@@ -73,11 +78,12 @@ class LetStmtNode : public StmtNode {
   /*! \brief The body block. */
   Stmt body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("var", &var);
-    v->Visit("value", &value);
-    v->Visit("body", &body);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<LetStmtNode>()
+        .def_ro("var", &LetStmtNode::var)
+        .def_ro("value", &LetStmtNode::value)
+        .def_ro("body", &LetStmtNode::body);
   }
 
   bool SEqualReduce(const LetStmtNode* other, SEqualReducer equal) const {
@@ -128,12 +134,13 @@ class AttrStmtNode : public StmtNode {
   /*! \brief The body statement to be executed */
   Stmt body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("node", &node);
-    v->Visit("attr_key", &attr_key);
-    v->Visit("value", &value);
-    v->Visit("body", &body);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<AttrStmtNode>()
+        .def_ro("node", &AttrStmtNode::node)
+        .def_ro("attr_key", &AttrStmtNode::attr_key)
+        .def_ro("value", &AttrStmtNode::value)
+        .def_ro("body", &AttrStmtNode::body);
   }
 
   bool SEqualReduce(const AttrStmtNode* other, SEqualReducer equal) const {
@@ -179,11 +186,12 @@ class AssertStmtNode : public StmtNode {
    */
   Stmt body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("condition", &condition);
-    v->Visit("message", &message);
-    v->Visit("body", &body);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<AssertStmtNode>()
+        .def_ro("condition", &AssertStmtNode::condition)
+        .def_ro("message", &AssertStmtNode::message)
+        .def_ro("body", &AssertStmtNode::body);
   }
 
   bool SEqualReduce(const AssertStmtNode* other, SEqualReducer equal) const {
@@ -234,12 +242,13 @@ class BufferStoreNode : public StmtNode {
   /*! \brief The predicate mask for storing values. */
   Optional<PrimExpr> predicate;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer", &buffer);
-    v->Visit("value", &value);
-    v->Visit("indices", &indices);
-    v->Visit("predicate", &predicate);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BufferStoreNode>()
+        .def_ro("buffer", &BufferStoreNode::buffer)
+        .def_ro("value", &BufferStoreNode::value)
+        .def_ro("indices", &BufferStoreNode::indices)
+        .def_ro("predicate", &BufferStoreNode::predicate);
   }
 
   bool SEqualReduce(const BufferStoreNode* other, SEqualReducer equal) const {
@@ -265,7 +274,7 @@ class BufferStoreNode : public StmtNode {
 class BufferStore : public Stmt {
  public:
   TVM_DLL explicit BufferStore(Buffer buffer, PrimExpr value, Array<PrimExpr> indices,
-                               Optional<PrimExpr> predicate = NullOpt, Span span = Span());
+                               Optional<PrimExpr> predicate = std::nullopt, Span span = Span());
 
   TVM_DEFINE_OBJECT_REF_METHODS(BufferStore, Stmt, BufferStoreNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferStoreNode);
@@ -293,12 +302,13 @@ class BufferRealizeNode : public StmtNode {
   /*! \brief The body of realization. */
   Stmt body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer", &buffer);
-    v->Visit("bounds", &bounds);
-    v->Visit("condition", &condition);
-    v->Visit("body", &body);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BufferRealizeNode>()
+        .def_ro("buffer", &BufferRealizeNode::buffer)
+        .def_ro("bounds", &BufferRealizeNode::bounds)
+        .def_ro("condition", &BufferRealizeNode::condition)
+        .def_ro("body", &BufferRealizeNode::body);
   }
 
   bool SEqualReduce(const BufferRealizeNode* other, SEqualReducer equal) const {
@@ -335,124 +345,6 @@ class BufferRealize : public Stmt {
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferRealizeNode);
 };
 
-/*!
- * \brief Store value into mult-dimensional array that will be read by the consumer
- *        of the producer.
- *
- * \note This node only appears in high-level DSLs that are built on top of the TIR.
- *       It should not appear in a valid TIR PrimFunc. A high-level DSL needs to lower
- *       this node before TIR transformations.
- *
- * \sa DataProducer
- */
-class ProducerStoreNode : public StmtNode {
- public:
-  /*! \brief The producer to store the results into. */
-  DataProducer producer;
-  /*! \brief The value to be stored. */
-  PrimExpr value;
-  /*! \brief The index arguments of the function. */
-  Array<PrimExpr> indices;
-
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("producer", &producer);
-    v->Visit("value", &value);
-    v->Visit("indices", &indices);
-    v->Visit("span", &span);
-  }
-
-  bool SEqualReduce(const ProducerStoreNode* other, SEqualReducer equal) const {
-    return equal(producer, other->producer) && equal(value, other->value) &&
-           equal(indices, other->indices);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(producer);
-    hash_reduce(value);
-    hash_reduce(indices);
-  }
-
-  static constexpr const char* _type_key = "tir.ProducerStore";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ProducerStoreNode, StmtNode);
-};
-
-/*!
- * \brief Managed reference to ProducerStoreNode.
- * \sa ProducerStoreNode
- */
-class ProducerStore : public Stmt {
- public:
-  TVM_DLL ProducerStore(DataProducer producer, PrimExpr value, Array<PrimExpr> indices,
-                        Span span = Span());
-
-  TVM_DEFINE_OBJECT_REF_METHODS(ProducerStore, Stmt, ProducerStoreNode);
-  TVM_DEFINE_OBJECT_REF_COW_METHOD(ProducerStoreNode);
-};
-
-/*!
- * \brief Annotate the bounds where the data produced by the producer
- *  need to be written and read in body.
- *  We will need to allocate space for the corresponding regions.
- *
- * \note This node only appears in high-level DSLs that are built on top of the TIR.
- *       It should not appear in a valid TIR PrimFunc. A high-level DSL needs to lower
- *       this node before TIR transformations.
- *
- * \sa DataProducer
- */
-class ProducerRealizeNode : public StmtNode {
- public:
-  /*! \brief The producer that produces the data. */
-  DataProducer producer;
-  /*! \brief Bounds to be realized. */
-  Region bounds;
-  /*! \brief Only realize if condition holds. */
-  PrimExpr condition;
-  /*! \brief The body of realization. */
-  Stmt body;
-  /*! \brief The storage scope associated with this realization. */
-  String storage_scope;
-
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("producer", &producer);
-    v->Visit("bounds", &bounds);
-    v->Visit("condition", &condition);
-    v->Visit("body", &body);
-    v->Visit("storage_scope", &storage_scope);
-    v->Visit("span", &span);
-  }
-
-  bool SEqualReduce(const ProducerRealizeNode* other, SEqualReducer equal) const {
-    return equal(producer, other->producer) && equal(bounds, other->bounds) &&
-           equal(condition, other->condition) && equal(body, other->body) &&
-           equal(storage_scope, other->storage_scope);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(producer);
-    hash_reduce(bounds);
-    hash_reduce(condition);
-    hash_reduce(body);
-    hash_reduce(storage_scope);
-  }
-
-  static constexpr const char* _type_key = "tir.ProducerRealize";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ProducerRealizeNode, StmtNode);
-};
-
-/*!
- * \brief Managed reference to ProducerRealizeNode.
- * \sa ProducerRealizeNode
- */
-class ProducerRealize : public Stmt {
- public:
-  TVM_DLL ProducerRealize(DataProducer producer, Region bounds, PrimExpr condition, Stmt body,
-                          String storage_scope = "", Span span = Span());
-
-  TVM_DEFINE_OBJECT_REF_METHODS(ProducerRealize, Stmt, ProducerRealizeNode);
-  TVM_DEFINE_OBJECT_REF_COW_METHOD(ProducerRealizeNode);
-};
-
 /*!
  * \brief Allocate a buffer that can be used in body.
  */
@@ -474,16 +366,17 @@ class AllocateNode : public StmtNode {
    *  These annotations can be used as auxiliary hint
    *  to future transformations.
    */
-  Map<String, ObjectRef> annotations;
+  Map<String, ffi::Any> annotations;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer_var", &buffer_var);
-    v->Visit("dtype", &dtype);
-    v->Visit("extents", &extents);
-    v->Visit("condition", &condition);
-    v->Visit("body", &body);
-    v->Visit("annotations", &annotations);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<AllocateNode>()
+        .def_ro("buffer_var", &AllocateNode::buffer_var)
+        .def_ro("dtype", &AllocateNode::dtype)
+        .def_ro("extents", &AllocateNode::extents)
+        .def_ro("condition", &AllocateNode::condition)
+        .def_ro("body", &AllocateNode::body)
+        .def_ro("annotations", &AllocateNode::annotations);
   }
 
   bool SEqualReduce(const AllocateNode* other, SEqualReducer equal) const {
@@ -528,7 +421,7 @@ class AllocateNode : public StmtNode {
 class Allocate : public Stmt {
  public:
   TVM_DLL Allocate(Var buffer_var, DataType dtype, Array<PrimExpr> extents, PrimExpr condition,
-                   Stmt body, Map<String, ObjectRef> annotations = Map<String, ObjectRef>(),
+                   Stmt body, Map<String, ffi::Any> annotations = Map<String, ffi::Any>(),
                    Span span = Span());
 
   TVM_DEFINE_OBJECT_REF_METHODS(Allocate, Stmt, AllocateNode);
@@ -562,17 +455,18 @@ class AllocateConstNode : public StmtNode {
    *  These annotations can be used as auxiliary hint
    *  to future transformations.
    */
-  Map<String, ObjectRef> annotations;
+  Map<String, ffi::Any> annotations;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer_var", &buffer_var);
-    v->Visit("data", &data);
-    v->Visit("irmod_storage_idx", &irmod_storage_idx);
-    v->Visit("dtype", &dtype);
-    v->Visit("extents", &extents);
-    v->Visit("body", &body);
-    v->Visit("annotations", &annotations);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<AllocateConstNode>()
+        .def_ro("buffer_var", &AllocateConstNode::buffer_var)
+        .def_ro("data", &AllocateConstNode::data)
+        .def_ro("irmod_storage_idx", &AllocateConstNode::irmod_storage_idx)
+        .def_ro("dtype", &AllocateConstNode::dtype)
+        .def_ro("extents", &AllocateConstNode::extents)
+        .def_ro("body", &AllocateConstNode::body)
+        .def_ro("annotations", &AllocateConstNode::annotations);
   }
 
   bool SEqualReduce(const AllocateConstNode* other, SEqualReducer equal) const {
@@ -622,7 +516,7 @@ class AllocateConst : public Stmt {
    */
   TVM_DLL AllocateConst(Var buffer_var, DataType dtype, Array<PrimExpr> extents,
                         ObjectRef data_or_idx, Stmt body,
-                        Map<String, ObjectRef> annotations = Map<String, ObjectRef>(),
+                        Map<String, ffi::Any> annotations = Map<String, ffi::Any>(),
                         Span span = Span());
   TVM_DEFINE_OBJECT_REF_METHODS(AllocateConst, Stmt, AllocateConstNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AllocateConstNode);
@@ -636,10 +530,11 @@ class DeclBufferNode : public StmtNode {
   /*! \brief The body to be executed */
   Stmt body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer", &buffer);
-    v->Visit("body", &body);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<DeclBufferNode>()
+        .def_ro("buffer", &DeclBufferNode::buffer)
+        .def_ro("body", &DeclBufferNode::body);
   }
 
   bool SEqualReduce(const DeclBufferNode* other, SEqualReducer equal) const {
@@ -679,9 +574,9 @@ class SeqStmtNode : public StmtNode {
    */
   Stmt operator[](size_t index) const { return seq[index]; }
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("seq", &seq);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<SeqStmtNode>().def_ro("seq", &SeqStmtNode::seq);
   }
 
   bool SEqualReduce(const SeqStmtNode* other, SEqualReducer equal) const {
@@ -705,9 +600,9 @@ class EvaluateNode : public StmtNode {
   /*! \brief The expression to be evaluated. */
   PrimExpr value;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("value", &value);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<EvaluateNode>().def_ro("value", &EvaluateNode::value);
   }
 
   bool SEqualReduce(const EvaluateNode* other, SEqualReducer equal) const {
@@ -773,7 +668,8 @@ class SeqStmt : public Stmt {
   template <typename... Args>
   static Stmt Flatten(Args&&... seq_args) {
     Array<Stmt> seq;
-    runtime::detail::for_each(Flattener(&seq), std::forward<Args>(seq_args)...);
+
+    ffi::details::for_each(Flattener(&seq), std::forward<Args>(seq_args)...);
 
     if (seq.empty()) {
       return Evaluate(0);
@@ -815,13 +711,18 @@ class SeqStmt : public Stmt {
     static Optional<SeqStmt> AsSeqStmt(const T& t) {
       if constexpr (std::is_same_v<T, SeqStmt>) {
         return t;
-      } else if constexpr (!std::is_base_of_v<T, SeqStmt>) {
-        return NullOpt;
-      } else if (auto* ptr = t.template as<SeqStmtNode>()) {
-        return GetRef<SeqStmt>(ptr);
-      } else {
-        return NullOpt;
       }
+      if constexpr (!std::is_base_of_v<T, SeqStmt>) {
+        return std::nullopt;
+      }
+      if constexpr (std::is_base_of_v<Stmt, T>) {
+        if (const SeqStmtNode* ptr = t.template as<SeqStmtNode>()) {
+          return GetRef<SeqStmt>(ptr);
+        } else {
+          return std::nullopt;
+        }
+      }
+      return std::nullopt;
     }
 
     template <typename T>
@@ -891,11 +792,12 @@ class IfThenElseNode : public StmtNode {
   /*! \brief The branch to be executed when condition is false, can be null. */
   Optional<Stmt> else_case;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("condition", &condition);
-    v->Visit("then_case", &then_case);
-    v->Visit("else_case", &else_case);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<IfThenElseNode>()
+        .def_ro("condition", &IfThenElseNode::condition)
+        .def_ro("then_case", &IfThenElseNode::then_case)
+        .def_ro("else_case", &IfThenElseNode::else_case);
   }
 
   bool SEqualReduce(const IfThenElseNode* other, SEqualReducer equal) const {
@@ -919,7 +821,7 @@ class IfThenElseNode : public StmtNode {
  */
 class IfThenElse : public Stmt {
  public:
-  TVM_DLL IfThenElse(PrimExpr condition, Stmt then_case, Optional<Stmt> else_case = NullOpt,
+  TVM_DLL IfThenElse(PrimExpr condition, Stmt then_case, Optional<Stmt> else_case = std::nullopt,
                      Span span = Span());
 
   TVM_DEFINE_OBJECT_REF_METHODS(IfThenElse, Stmt, IfThenElseNode);
@@ -989,17 +891,18 @@ class ForNode : public StmtNode {
    *  not change the control flow semantics of the loop
    *  and can be ignored in most passes.
    */
-  Map<String, ObjectRef> annotations;
+  Map<String, ffi::Any> annotations;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("loop_var", &loop_var);
-    v->Visit("min", &min);
-    v->Visit("extent", &extent);
-    v->Visit("kind", &kind);
-    v->Visit("body", &body);
-    v->Visit("thread_binding", &thread_binding);
-    v->Visit("annotations", &annotations);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ForNode>()
+        .def_ro("loop_var", &ForNode::loop_var)
+        .def_ro("min", &ForNode::min)
+        .def_ro("extent", &ForNode::extent)
+        .def_ro("kind", &ForNode::kind)
+        .def_ro("body", &ForNode::body)
+        .def_ro("thread_binding", &ForNode::thread_binding)
+        .def_ro("annotations", &ForNode::annotations);
   }
 
   bool SEqualReduce(const ForNode* other, SEqualReducer equal) const {
@@ -1029,8 +932,8 @@ class ForNode : public StmtNode {
 class For : public Stmt {
  public:
   TVM_DLL For(Var loop_var, PrimExpr min, PrimExpr extent, ForKind kind, Stmt body,
-              Optional<IterVar> thread_binding = NullOpt,
-              Map<String, ObjectRef> annotations = Map<String, ObjectRef>(), Span span = Span());
+              Optional<IterVar> thread_binding = std::nullopt,
+              Map<String, ffi::Any> annotations = Map<String, ffi::Any>(), Span span = Span());
 
   TVM_DEFINE_OBJECT_REF_METHODS(For, Stmt, ForNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ForNode);
@@ -1053,10 +956,11 @@ class WhileNode : public StmtNode {
   /*! \brief The body of the while loop. */
   Stmt body;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("condition", &condition);
-    v->Visit("body", &body);
-    v->Visit("span", &span);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<WhileNode>()
+        .def_ro("condition", &WhileNode::condition)
+        .def_ro("body", &WhileNode::body);
   }
 
   bool SEqualReduce(const WhileNode* other, SEqualReducer equal) const {
@@ -1084,64 +988,21 @@ class While : public Stmt {
   TVM_DEFINE_OBJECT_REF_COW_METHOD(WhileNode);
 };
 
-/*!
- * \brief A prefetch hint for a buffer
- */
-class PrefetchNode : public StmtNode {
- public:
-  /*! \brief The function to be prefetched. */
-  Buffer buffer;
-  /*! \brief Bounds to be prefetched. */
-  Array<Range> bounds;
-
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer", &buffer);
-    v->Visit("bounds", &bounds);
-    v->Visit("span", &span);
-  }
-
-  bool SEqualReduce(const PrefetchNode* other, SEqualReducer equal) const {
-    return equal(buffer, other->buffer) && equal(bounds, other->bounds);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(buffer);
-    hash_reduce(bounds);
-  }
-
-  PrefetchNode() = default;
-  PrefetchNode(Buffer buffer, Array<Range> bounds, Span span = Span())
-      : StmtNode(span), buffer(buffer), bounds(bounds) {}
-
-  static constexpr const char* _type_key = "tir.Prefetch";
-  TVM_DECLARE_FINAL_OBJECT_INFO(PrefetchNode, StmtNode);
-};
-
-/*!
- * \brief Managed reference to PrefetchNode.
- * \sa PrefetchNode
- */
-class Prefetch : public Stmt {
- public:
-  TVM_DLL explicit Prefetch(Buffer buffer, Array<Range> bounds, Span span = Span());
-
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(Prefetch, Stmt, PrefetchNode);
-  TVM_DEFINE_OBJECT_REF_COW_METHOD(PrefetchNode);
-};
-
 /*!
  * \brief Representing the region of multi-dimensional buffer access.
  */
-class BufferRegionNode : public Object {
+class BufferRegionNode : public PrimExprConvertibleNode {
  public:
   /*! \brief The buffer of the buffer region. */
   Buffer buffer;
   /*! \brief The region array of the buffer region. */
   Array<Range> region;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer", &buffer);
-    v->Visit("region", &region);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BufferRegionNode>()
+        .def_ro("buffer", &BufferRegionNode::buffer)
+        .def_ro("region", &BufferRegionNode::region);
   }
 
   bool SEqualReduce(const BufferRegionNode* other, SEqualReducer equal) const {
@@ -1153,17 +1014,19 @@ class BufferRegionNode : public Object {
     hash_reduce(region);
   }
 
+  TVM_DLL PrimExpr ToPrimExpr() const final;
+
   static constexpr const char* _type_key = "tir.BufferRegion";
   static constexpr const bool _type_has_method_sequal_reduce = true;
   static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_FINAL_OBJECT_INFO(BufferRegionNode, Object);
+  TVM_DECLARE_FINAL_OBJECT_INFO(BufferRegionNode, PrimExprConvertibleNode);
 };
 
 /*!
  * \brief Managed reference to BufferRegionNode.
  * \sa BufferRegionNode
  */
-class BufferRegion : public ObjectRef {
+class BufferRegion : public PrimExprConvertible {
  public:
   TVM_DLL explicit BufferRegion(Buffer buffer, Array<Range> region);
 
@@ -1182,7 +1045,7 @@ class BufferRegion : public ObjectRef {
    */
   TVM_DLL static BufferRegion FromPoint(Buffer buffer, Array<PrimExpr> indices);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(BufferRegion, ObjectRef, BufferRegionNode);
+  TVM_DEFINE_OBJECT_REF_METHODS(BufferRegion, PrimExprConvertible, BufferRegionNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferRegionNode);
 };
 
@@ -1202,9 +1065,11 @@ class MatchBufferRegionNode : public Object {
   /*! \brief The source buffer region. */
   BufferRegion source;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("buffer", &buffer);
-    v->Visit("source", &source);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<MatchBufferRegionNode>()
+        .def_ro("buffer", &MatchBufferRegionNode::buffer)
+        .def_ro("source", &MatchBufferRegionNode::source);
   }
 
   bool SEqualReduce(const MatchBufferRegionNode* other, SEqualReducer equal) const {
@@ -1272,7 +1137,7 @@ class BlockNode : public StmtNode {
    *  reduction block. The optional init field allows us to represent initialization and
    *  reduction update in a single block and transform them collectively.
    *  We also provide primitives to decompose the init into a separate block during scheduling.
-   *  Init field is `NullOpt` if there is no reduction iter_vars
+   *  Init field is `std::nullopt` if there is no reduction iter_vars
    */
   Optional<Stmt> init;
   /*! \brief The buffer allocated in the block. */
@@ -1280,18 +1145,20 @@ class BlockNode : public StmtNode {
   /*! \brief The match buffer regions. */
   Array<MatchBufferRegion> match_buffers;
   /*! \brief The annotation of the block. */
-  Map<String, ObjectRef> annotations;
+  Map<String, ffi::Any> annotations;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("iter_vars", &iter_vars);
-    v->Visit("reads", &reads);
-    v->Visit("writes", &writes);
-    v->Visit("name_hint", &name_hint);
-    v->Visit("body", &body);
-    v->Visit("init", &init);
-    v->Visit("alloc_buffers", &alloc_buffers);
-    v->Visit("match_buffers", &match_buffers);
-    v->Visit("annotations", &annotations);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BlockNode>()
+        .def_ro("iter_vars", &BlockNode::iter_vars)
+        .def_ro("reads", &BlockNode::reads)
+        .def_ro("writes", &BlockNode::writes)
+        .def_ro("name_hint", &BlockNode::name_hint)
+        .def_ro("body", &BlockNode::body)
+        .def_ro("init", &BlockNode::init)
+        .def_ro("alloc_buffers", &BlockNode::alloc_buffers)
+        .def_ro("match_buffers", &BlockNode::match_buffers)
+        .def_ro("annotations", &BlockNode::annotations);
   }
 
   bool SEqualReduce(const BlockNode* other, SEqualReducer equal) const {
@@ -1326,10 +1193,10 @@ class Block : public Stmt {
  public:
   TVM_DLL explicit Block(Array<IterVar> iter_vars, Array<BufferRegion> reads,
                          Array<BufferRegion> writes, String name_hint, Stmt body,
-                         Optional<Stmt> init = NullOpt,
+                         Optional<Stmt> init = std::nullopt,
                          Array<Buffer> alloc_buffers = Array<Buffer>(),
                          Array<MatchBufferRegion> match_buffers = Array<MatchBufferRegion>(),
-                         Map<String, ObjectRef> annotations = Map<String, ObjectRef>(),
+                         Map<String, ffi::Any> annotations = Map<String, ffi::Any>(),
                          Span span = Span());
 
   TVM_DEFINE_OBJECT_REF_METHODS(Block, Stmt, BlockNode);
@@ -1351,10 +1218,12 @@ class BlockRealizeNode : public StmtNode {
   /*! \brief The block to be realized. */
   Block block;
 
-  void VisitAttrs(AttrVisitor* v) {
-    v->Visit("iter_values", &iter_values);
-    v->Visit("predicate", &predicate);
-    v->Visit("block", &block);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<BlockRealizeNode>()
+        .def_ro("iter_values", &BlockRealizeNode::iter_values)
+        .def_ro("predicate", &BlockRealizeNode::predicate)
+        .def_ro("block", &BlockRealizeNode::block);
   }
 
   bool SEqualReduce(const BlockRealizeNode* other, SEqualReducer equal) const {
@@ -1664,6 +1533,16 @@ constexpr const char* warp_execution = "warp_execution";
 /*! \brief Mark that a block is disallowed in auto inline. */
 constexpr const char* meta_schedule_inline_rule = "meta_schedule.inline_rule";
 
+/*! \brief Mark that a block has an explicitly specified read region.
+ * This is used to override the default read region inference in TIR.
+ */
+constexpr const char* explicit_read_region = "explicit_read_region";
+
+/*! \brief Mark that a block has an explicitly specified write region.
+ * This is used to override the default write region inference in TIR.
+ */
+constexpr const char* explicit_write_region = "explicit_write_region";
+
 /*!
  * \brief Check if attr_key is a pragma key extension
  * \param attr_key The attr key to be compared
