@@ -34,7 +34,7 @@ except ImportError:
     ml_dtypes = None
 
 import tvm
-from tvm._ffi.libinfo import find_lib_path
+from tvm.libinfo import find_lib_path
 from tvm.runtime import DataType
 
 from .emcc import create_tvmjs_wasm
@@ -279,8 +279,8 @@ def dump_ndarray_cache(
         # prefer to preserve original dtype, especially if the format was bfloat16
         dtype = origin_v.dtype if isinstance(origin_v, tvm.nd.NDArray) else v.dtype
 
-        if dtype in DataType.NUMPY2STR:
-            dtype = DataType.NUMPY2STR[dtype]
+        if dtype in DataType.NUMPY_DTYPE_TO_STR:
+            dtype = DataType.NUMPY_DTYPE_TO_STR[dtype]
         else:
             dtype = str(dtype)
 
