@@ -22,7 +22,7 @@ import sys
 from typing import Callable
 
 import tvm
-from tvm._ffi import get_global_func, register_func
+from tvm.ffi import get_global_func, register_func
 from tvm.runtime import NDArray, ShapeTuple, String
 from tvm.runtime.ndarray import array
 
