@@ -22,7 +22,7 @@ three key abstractions: a Diagnostic, the DiagnosticContext,
 and the DiagnosticRenderer.
 """
 import enum
-import tvm._ffi
+import tvm.ffi
 from . import _ffi_api
 from ... import get_global_func, register_func, Object
 
@@ -69,7 +69,7 @@ class DiagnosticLevel(enum.IntEnum):
     HELP = 50
 
 
-@tvm._ffi.register_object("Diagnostic")
+@tvm.ffi.register_object("Diagnostic")
 class Diagnostic(Object):
     """A single diagnostic object from TVM."""
 
@@ -77,7 +77,7 @@ class Diagnostic(Object):
         self.__init_handle_by_constructor__(_ffi_api.Diagnostic, level, span, message)
 
 
-@tvm._ffi.register_object("DiagnosticRenderer")
+@tvm.ffi.register_object("DiagnosticRenderer")
 class DiagnosticRenderer(Object):
     """
     A diagnostic renderer, which given a diagnostic context produces a "rendered"
@@ -100,7 +100,7 @@ class DiagnosticRenderer(Object):
 
 
 # Register the diagnostic context.
-@tvm._ffi.register_object("DiagnosticContext")
+@tvm.ffi.register_object("DiagnosticContext")
 class DiagnosticContext(Object):
     """
     A diagnostic context which records active errors
