@@ -22,7 +22,7 @@ from typing_extensions import Literal
 
 # isort: on
 
-from tvm._ffi import register_object
+from tvm.ffi import register_object
 from tvm.ir.module import IRModule
 from tvm.runtime import Object
 from tvm.target import Target
