@@ -16,7 +16,7 @@
 # under the License.
 """A postprocessor that verifies if the GPU code is correct"""
 
-from tvm._ffi.registry import register_object
+from tvm.ffi.registry import register_object
 from .. import _ffi_api
 from .postproc import Postproc
 
