@@ -24,7 +24,7 @@ from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Tuple, Union
 from typing_extensions import Literal
 
 # isort: on
-from tvm._ffi import register_object
+from tvm.ffi import register_object
 from tvm.ir import IRModule
 from tvm.runtime import Object
 from tvm.tir.schedule import Schedule
