@@ -24,6 +24,7 @@ from .measure_callback import MeasureCallback
 from .runner import Runner
 from .task_scheduler import TaskScheduler
 from .tune_context import TuneContext
+from .post_optimization import PostOpt
 
 
 def tune_tasks(
@@ -41,6 +42,7 @@ def tune_tasks(
     measure_callbacks: MeasureCallback.CallbackListType = "default",
     task_scheduler: TaskScheduler.TaskSchedulerType = "gradient",
     module_equality: str = "structural",
+    post_optimization: Optional[bool] = False,
 ) -> Database:
     """Tune a list of tasks. Using a task scheduler.
 
@@ -81,6 +83,8 @@ def tune_tasks(
                 a given module. The "ignore-ndarray" varint is used for the extracted blocks or in
                 case no anchor block is found. For the definition of the anchor block, see
                 tir/analysis/analysis.py.
+    post_optimization : Optional[Bool]
+        Generate post-optimization using Droplet Search as exploitation space.
 
     Returns
     -------
@@ -108,7 +112,7 @@ def tune_tasks(
     elif not isinstance(database, Database):
         database = Database.create(database, module_equality=module_equality)
     if not isinstance(cost_model, CostModel):
-        cost_model = CostModel.create(cost_model, num_tuning_cores=num_cores)
+        cost_model = CostModel.create(cost_model, num_tuning_cores=num_cores, tree_method="auto")
     if isinstance(measure_callbacks, MeasureCallback):
         measure_callbacks = [measure_callbacks]
     elif measure_callbacks == "default":
@@ -127,4 +131,7 @@ def tune_tasks(
         database=database,
         cost_model=cost_model,
     )
+    if post_optimization:
+        post_opt = PostOpt(work_dir, tasks[0].target)
+        post_opt.run()
     return database
