@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI API for Relax."""
-import tvm._ffi
+import tvm.ffi
 
-tvm._ffi._init_api("relax", __name__)
+tvm.ffi._init_api("relax", __name__)
