@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """DataFlow Pattern Language FFI bindings."""
-import tvm._ffi
+import tvm.ffi
 
-tvm._ffi._init_api("relax.dpl", __name__)
+tvm.ffi._init_api("relax.dpl", __name__)
