@@ -56,7 +56,7 @@ class VMFuncScope(object):
         self.exit_callback()
 
 
-@tvm._ffi.register_object("relax.ExecBuilder")
+@tvm.ffi.register_object("relax.ExecBuilder")
 class ExecBuilder(Object):
     """A builder to emit instructions and build executable for the virtual machine."""
 
@@ -87,7 +87,7 @@ class ExecBuilder(Object):
 
     def declare_function(self, func_name: str, kind: VMFuncKind = VMFuncKind.PACKED_FUNC) -> None:
         """Declare a function"""
-        _ffi_api.ExecBuilderDecalreFunction(self, func_name, kind)  # type: ignore
+        _ffi_api.ExecBuilderDeclareFunction(self, func_name, kind)  # type: ignore
 
     def function(
         self, func_name: str, num_inputs: Optional[int] = 0, param_names: List[str] = None
