@@ -327,7 +327,7 @@ def llama_rope(  # pylint: disable=too-many-arguments
         T.func_attr(
             {
                 "op_pattern": 8,  # 2 means injective, 8 means opaque
-                "tir.noalias": T.bool(True),
+                "tir.noalias": True,
             }
         )
         batch_size = T.int64()
@@ -456,7 +456,7 @@ def llama_rope_with_position_map(  # pylint: disable=too-many-arguments
         T.func_attr(
             {
                 "op_pattern": 8,  # 2 means injective, 8 means opaque
-                "tir.noalias": T.bool(True),
+                "tir.noalias": True,
             }
         )
         seq_len = T.int32()
@@ -493,12 +493,12 @@ def llama_rope_with_position_map(  # pylint: disable=too-many-arguments
         var_q: T.handle,
         var_k: T.handle,
         var_v: T.handle,
-        ext_factors: T.Buffer((head_dim // 2,), "float32"),  # type: ignore
+        ext_factors: T.Buffer((rotary_dim // 2,), "float32"),  # type: ignore
     ):
         T.func_attr(
             {
                 "op_pattern": 8,  # 2 means injective, 8 means opaque
-                "tir.noalias": T.bool(True),
+                "tir.noalias": True,
             }
         )
         seq_len = T.int64()
