@@ -50,6 +50,8 @@ from .binary import (
     divide,
     equal,
     floor_divide,
+    log_add_exp,
+    floor_mod,
     greater,
     greater_equal,
     left_shift,
@@ -60,6 +62,7 @@ from .binary import (
     logical_xor,
     maximum,
     minimum,
+    mod,
     multiply,
     not_equal,
     power,
@@ -70,8 +73,11 @@ from .create import (
     arange,
     full,
     full_like,
+    hamming_window,
     ones,
     ones_like,
+    eye,
+    eye_like,
     tril,
     triu,
     zeros,
@@ -79,7 +85,7 @@ from .create import (
 )
 from .datatype import astype, wrap_param
 from .index import dynamic_strided_slice, strided_slice, take
-from .linear_algebra import einsum, linear, matmul
+from .linear_algebra import einsum, linear, matmul, outer
 from .manipulate import (
     broadcast_to,
     collapse_sum_like,
@@ -88,20 +94,28 @@ from .manipulate import (
     expand_dims,
     flatten,
     flip,
+    gather_elements,
+    gather_nd,
+    index_put,
+    index_tensor,
+    meshgrid,
     layout_transform,
+    one_hot,
     permute_dims,
     repeat,
     reshape,
     scatter_elements,
     scatter_nd,
+    slice_scatter,
     split,
     squeeze,
+    stack,
     tile,
 )
 from .mask import masked_fill
 from .qdq import dequantize, quantize
 from .sampling import multinomial_from_uniform
-from .search import argmax, argmin, where
+from .search import argmax, argmin, where, bucketize
 from .set import nonzero, unique
 from .sorting import argsort, sort, topk
 from .statistical import cumprod, cumsum, max, mean, min, prod, std, sum, variance
@@ -138,6 +152,7 @@ from .unary import (
     square,
     tan,
     tanh,
+    trunc,
 )
 
 
