@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI APIs for tvm.relax.op.grad"""
-import tvm._ffi
+import tvm.ffi
 
-tvm._ffi._init_api("relax.op.grad", __name__)
+tvm.ffi._init_api("relax.op.grad", __name__)
