@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-# pylint: disable=used-before-assignment
+# pylint: disable=used-before-assignment,broad-exception-caught
 """RPC client tools"""
 import os
 import socket
@@ -22,11 +22,11 @@ import stat
 import struct
 import time
 
-import tvm._ffi
-from tvm._ffi.base import TVMError
+import tvm.ffi
+from tvm.base import TVMError
 from tvm.contrib import utils
 from tvm.runtime import ndarray as nd
-from tvm._ffi.runtime_ctypes import Device
+from tvm.runtime import Device
 
 from . import _ffi_api, base, server
 
@@ -88,7 +88,7 @@ class RPCSession(object):
         """
         dev = nd.device(dev_type, dev_id)
         encode = (self._tbl_index + 1) * base.RPC_SESS_MASK
-        dev.device_type += encode
+        dev = nd.device(dev.device_type + encode, dev.device_id)
         dev._rpc_sess = self
         return dev
 
@@ -263,7 +263,7 @@ class LocalSession(RPCSession):
         RPCSession.__init__(self, _ffi_api.LocalSession())
 
 
-@tvm._ffi.register_func("rpc.PopenSession")
+@tvm.ffi.register_func("rpc.PopenSession")
 def _popen_session(binary):
     temp = utils.tempdir()
 
