@@ -21,18 +21,19 @@ from .packed_func import PackedFunc
 from .object import Object
 from .object_path import ObjectPath, ObjectPathPair
 from .script_printer import Scriptable
-from .object_generic import ObjectGeneric, ObjectTypes
-from .ndarray import NDArray, DataType, DataTypeCode, Device
+from .object_generic import ObjectGeneric
+from .device import Device
+from .ndarray import NDArray
 from .module import Module
 from .profiling import Report
 from .executable import Executable
 
 # function exposures
-from .ndarray import device, cpu, cuda, gpu, opencl, cl, vulkan, metal, mtl
+from .ndarray import device, cpu, cuda, opencl, vulkan, metal
 from .ndarray import vpi, rocm, ext_dev
 from .module import load_module, enabled, system_lib, load_static_library, num_threads
-from .container import String, ShapeTuple  # , BoxBool
-from .object_generic import convert_to_object, convert, const
+from .container import String, ShapeTuple
+from .object_generic import const
 from .params import (
     save_param_dict,
     load_param_dict,
@@ -43,3 +44,4 @@ from .params import (
 from . import disco
 
 from .support import _regex_match
+from ..ffi import convert, dtype as DataType, DataTypeCode
