@@ -17,7 +17,7 @@
 """Registration of profiling objects in python."""
 
 from typing import Dict, Sequence, Optional
-from ... import _ffi
+from ... import ffi as _ffi
 from . import _ffi_api
 from .. import Object, Device
 
