@@ -14,7 +14,7 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI APIs for tvm.script"""
-import tvm._ffi
+import tvm.ffi
 
 
-tvm._ffi._init_api("script", __name__)
+tvm.ffi._init_api("script", __name__)
