@@ -26,7 +26,7 @@ from tvm.tir.expr import (
     BinaryOpExpr as _BinaryOpExpr,
 )
 from tvm.tir.op import call_pure_extern
-from tvm._ffi import register_func as _register_func
+from tvm.ffi import register_func as _register_func
 from tvm.tir import call_intrin
 
 
@@ -214,7 +214,7 @@ def register_op(
         )
     else:
         lower_func_name = "tvm.datatype.lower." + target + "." + op_name + "." + src_type_name
-    tvm._ffi.register_func(lower_func_name, lower_func)
+    tvm.ffi.register_func(lower_func_name, lower_func)
 
 
 def register_min_func(func, type_name):
