@@ -36,19 +36,19 @@ def after_initializer():
     return TEST_GLOBAL_STATE_1, TEST_GLOBAL_STATE_2, TEST_GLOBAL_STATE_3
 
 
-@tvm._ffi.register_func("testing.identity_py")
+@tvm.ffi.register_func("testing.identity_py")
 def identity_py(arg):
     return arg
 
 
 def register_ffi():
-    @tvm._ffi.register_func("testing.nested_identity_py")
+    @tvm.ffi.register_func("testing.nested_identity_py")
     def _identity_py(arg):  # pylint: disable=unused-variable
         return arg
 
 
 def call_py_ffi(arg):
-    _identity_py = tvm._ffi.get_global_func("testing.nested_identity_py")
+    _identity_py = tvm.ffi.get_global_func("testing.nested_identity_py")
     return _identity_py(arg)
 
 
