@@ -29,12 +29,12 @@ Each statement node have subfields that can be visited from python side.
 from enum import IntEnum
 from typing import List, Mapping, Optional, Union
 
-import tvm._ffi
+import tvm.ffi
 from tvm.ir import PrimExpr, Range, Span
 from tvm.runtime import Object, Scriptable, const, NDArray
 
 from . import _ffi_api
-from .buffer import Buffer, DataProducer
+from .buffer import Buffer
 from .expr import Var, IterVar
 
 
@@ -42,7 +42,7 @@ class Stmt(Object, Scriptable):
     """Base class of all the statements."""
 
 
-@tvm._ffi.register_object("tir.LetStmt")
+@tvm.ffi.register_object("tir.LetStmt")
 class LetStmt(Stmt):
     """LetStmt node.
 
@@ -72,7 +72,7 @@ class LetStmt(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.AssertStmt")
+@tvm.ffi.register_object("tir.AssertStmt")
 class AssertStmt(Stmt):
     """AssertStmt node.
 
@@ -120,7 +120,7 @@ class ForKind(IntEnum):
     THREAD_BINDING = 4  # pylint: disable=invalid-name
 
 
-@tvm._ffi.register_object("tir.For")
+@tvm.ffi.register_object("tir.For")
 class For(Stmt):
     """For node.
 
@@ -185,7 +185,7 @@ class For(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.While")
+@tvm.ffi.register_object("tir.While")
 class While(Stmt):
     """While node.
 
@@ -209,7 +209,7 @@ class While(Stmt):
         self.__init_handle_by_constructor__(_ffi_api.While, condition, body, span)  # type: ignore
 
 
-@tvm._ffi.register_object("tir.BufferStore")
+@tvm.ffi.register_object("tir.BufferStore")
 class BufferStore(Stmt):
     """Buffer store node.
 
@@ -252,7 +252,7 @@ class BufferStore(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.BufferRealize")
+@tvm.ffi.register_object("tir.BufferRealize")
 class BufferRealize(Stmt):
     """Buffer realize node.
 
@@ -293,43 +293,7 @@ class BufferRealize(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.ProducerStore")
-class ProducerStore(Stmt):
-    """ProducerStore node.
-
-    Parameters
-    ----------
-    producer : DataProducer
-        The data producer.
-
-    value : PrimExpr
-        The value to be stored.
-
-    indices : list of Expr
-        The index arguments of the store.
-
-    span : Optional[Span]
-        The location of the stmt in the source code.
-    """
-
-    producer: DataProducer
-    value: PrimExpr
-    indices: List[PrimExpr]
-    span: Optional[Span]
-
-    def __init__(
-        self,
-        producer: DataProducer,
-        value: PrimExpr,
-        indices: List[PrimExpr],
-        span: Optional[Span] = None,
-    ) -> None:
-        self.__init_handle_by_constructor__(
-            _ffi_api.ProducerStore, producer, value, indices, span  # type: ignore
-        )
-
-
-@tvm._ffi.register_object("tir.Allocate")
+@tvm.ffi.register_object("tir.Allocate")
 class Allocate(Stmt):
     """Allocate node.
 
@@ -389,7 +353,7 @@ class Allocate(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.AllocateConst")
+@tvm.ffi.register_object("tir.AllocateConst")
 class AllocateConst(Stmt):
     """Allocate constant node.
 
@@ -451,7 +415,7 @@ class AllocateConst(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.DeclBuffer")
+@tvm.ffi.register_object("tir.DeclBuffer")
 class DeclBuffer(Stmt):
     """DeclBuffer node.
 
@@ -475,7 +439,7 @@ class DeclBuffer(Stmt):
         self.__init_handle_by_constructor__(_ffi_api.DeclBuffer, buffer, body, span)
 
 
-@tvm._ffi.register_object("tir.AttrStmt")
+@tvm.ffi.register_object("tir.AttrStmt")
 class AttrStmt(Stmt):
     """AttrStmt node.
 
@@ -511,59 +475,7 @@ class AttrStmt(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.ProducerRealize")
-class ProducerRealize(Stmt):
-    """ProducerRealize node.
-
-    Parameters
-    ----------
-    producer : DataProducer
-        The data producer.
-
-    bounds : List[Range]
-        The bound of realize
-
-    condition : PrimExpr
-        The realize condition.
-
-    body : Stmt
-        The realize body
-
-    storage_scope : str
-        The storage scope associated with this realization
-
-    span : Optional[Span]
-        The location of the stmt in the source code.
-    """
-
-    producer: DataProducer
-    bounds: List[Range]
-    condition: PrimExpr
-    body: Stmt
-    storage_scope: str
-    span: Optional[Span]
-
-    def __init__(
-        self,
-        producer: DataProducer,
-        bounds: List[Range],
-        condition: PrimExpr,
-        body: Stmt,
-        storage_scope: str = "",
-        span: Optional[Span] = None,
-    ) -> None:
-        self.__init_handle_by_constructor__(
-            _ffi_api.ProducerRealize,  # type: ignore
-            producer,
-            bounds,
-            condition,
-            body,
-            storage_scope,
-            span,
-        )
-
-
-@tvm._ffi.register_object("tir.SeqStmt")
+@tvm.ffi.register_object("tir.SeqStmt")
 class SeqStmt(Stmt):
     """Sequence of statements.
 
@@ -589,7 +501,7 @@ class SeqStmt(Stmt):
         return len(self.seq)
 
 
-@tvm._ffi.register_object("tir.IfThenElse")
+@tvm.ffi.register_object("tir.IfThenElse")
 class IfThenElse(Stmt):
     """IfThenElse node.
 
@@ -624,7 +536,7 @@ class IfThenElse(Stmt):
         )
 
 
-@tvm._ffi.register_object("tir.Evaluate")
+@tvm.ffi.register_object("tir.Evaluate")
 class Evaluate(Stmt):
     """Evaluate node.
 
@@ -644,31 +556,7 @@ class Evaluate(Stmt):
         self.__init_handle_by_constructor__(_ffi_api.Evaluate, value, span)  # type: ignore
 
 
-@tvm._ffi.register_object("tir.Prefetch")
-class Prefetch(Stmt):
-    """Prefetch node.
-
-    Parameters
-    ----------
-    buffer : Buffer
-        The buffer to be prefetched.
-
-    bounds : List[Range]
-        The bounds to be prefetched.
-
-    span : Optional[Span]
-        The location of the stmt in the source code.
-    """
-
-    buffer: Buffer
-    bounds: List[Range]
-    span: Optional[Span]
-
-    def __init__(self, buffer: Buffer, bounds: List[Range], span: Optional[Span] = None) -> None:
-        self.__init_handle_by_constructor__(_ffi_api.Prefetch, buffer, bounds, span)  # type: ignore
-
-
-@tvm._ffi.register_object("tir.BufferRegion")
+@tvm.ffi.register_object("tir.BufferRegion")
 class BufferRegion(Object, Scriptable):
     """BufferRegion node.
 
@@ -688,7 +576,7 @@ class BufferRegion(Object, Scriptable):
         self.__init_handle_by_constructor__(_ffi_api.BufferRegion, buffer, region)  # type: ignore
 
 
-@tvm._ffi.register_object("tir.MatchBufferRegion")
+@tvm.ffi.register_object("tir.MatchBufferRegion")
 class MatchBufferRegion(Object, Scriptable):
     """MatchBufferRegion node.
 
@@ -710,7 +598,7 @@ class MatchBufferRegion(Object, Scriptable):
         )
 
 
-@tvm._ffi.register_object("tir.Block")
+@tvm.ffi.register_object("tir.Block")
 class Block(Stmt):
     """Block node.
 
@@ -792,7 +680,7 @@ class Block(Stmt):
         )  # type: ignore
 
 
-@tvm._ffi.register_object("tir.BlockRealize")
+@tvm.ffi.register_object("tir.BlockRealize")
 class BlockRealize(Stmt):
     """BlockRealize node.
 
