@@ -41,12 +41,12 @@ from .upsampling import *
 from .instance_norm import instance_norm
 from .layer_norm import layer_norm
 from .group_norm import group_norm
+from .rms_norm import rms_norm
 from .local_response_norm import *
 from .bitserial_conv2d import *
 from .bitserial_dense import *
 from .batch_matmul import *
 from .batch_norm import *
-from .sparse import *
 from .pad import *
 from .fifo_buffer import *
 from .depth_to_space import *
