@@ -19,7 +19,7 @@
 import logging
 
 import tvm
-from tvm import auto_scheduler, te
+from tvm import te
 
 from ..utils import get_const_tuple
 
@@ -79,23 +79,15 @@ def batch_matmul(
     else:
         XB, XI, XK = get_const_tuple(tensor_a.shape)
     if auto_scheduler_rewritten_layout:
-        # Infer shape for the rewritten layout
-        YB, YK, YJ = auto_scheduler.get_shape_from_rewritten_layout(
-            auto_scheduler_rewritten_layout, ["b", "k", "j"]
-        )
-        auto_scheduler.remove_index_check(tensor_b)
-    elif meta_schedule_original_shape:
-        auto_scheduler.rewrite_tensor_shape(tensor_b, meta_schedule_original_shape)
-        if transpose_b:
-            YB, YJ, YK = get_const_tuple(tensor_b.shape)
-        else:
-            YB, YK, YJ = get_const_tuple(tensor_b.shape)
+        raise RuntimeError("LEGACY-FLOW triggered, to be removed")
+    if meta_schedule_original_shape:
+        raise RuntimeError("LEGACY-FLOW triggered, to be removed")
+
+    assert len(tensor_b.shape) == 3, "tensor_b only support 3-dim"
+    if transpose_b:
+        YB, YJ, YK = get_const_tuple(tensor_b.shape)
     else:
-        assert len(tensor_b.shape) == 3, "tensor_b only support 3-dim"
-        if transpose_b:
-            YB, YJ, YK = get_const_tuple(tensor_b.shape)
-        else:
-            YB, YK, YJ = get_const_tuple(tensor_b.shape)
+        YB, YK, YJ = get_const_tuple(tensor_b.shape)
 
     assert XK == YK or isinstance(YK, tvm.tir.expr.Var), "shapes of x and y are inconsistent"
     k = te.reduce_axis((0, XK), name="k")
@@ -153,29 +145,6 @@ def batch_matmul(
         attrs={"layout_free_placeholders": [tensor_b]},
     )
     if auto_scheduler_rewritten_layout:
-        output = auto_scheduler.rewrite_compute_body(output, auto_scheduler_rewritten_layout)
+        raise RuntimeError("LEGACY-FLOW triggered, to be removed")
 
     return output
-
-
-@tvm.target.generic_func
-def batch_matmul_legalize(attrs, inputs, types):
-    """Legalizes batch_matmul op.
-
-    Parameters
-    ----------
-    attrs : tvm.ir.Attrs
-        Attributes of current batch_matmul
-    inputs : list of tvm.relay.Expr
-        The args of the Relay expr to be legalized
-    types : list of types
-        List of input and output types
-
-    Returns
-    -------
-    result : tvm.relay.Expr
-        The legalized expr
-    """
-    # not to change by default
-    # pylint: disable=unused-argument
-    return None
