@@ -52,9 +52,6 @@ class CanonicalExprNode : public PrimExprNode {
    */
   virtual PrimExpr Normalize() const = 0;
 
-  // overrides
-  void VisitAttrs(tvm::AttrVisitor* v) {}
-
   static constexpr const char* _type_key = "arith.CanonicalExpr";
   static constexpr const uint32_t _type_child_slots = 2;
   TVM_DECLARE_BASE_OBJECT_INFO(CanonicalExprNode, PrimExprNode);
@@ -755,7 +752,7 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const AddNode* op) {
   } else {
     ret.CopyOnWrite()->AddToSelf(ToSplitExpr(b), 1);
   }
-  return std::move(ret);
+  return ret;
 }
 
 PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const SubNode* op) {
@@ -779,7 +776,7 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const SubNode* op) {
   } else {
     ret.CopyOnWrite()->AddToSelf(ToSplitExpr(b), -1);
   }
-  return std::move(ret);
+  return ret;
 }
 
 PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const MulNode* op) {
@@ -801,11 +798,12 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const MulNode* op) {
     if (a.as<SumExprNode>()) {
       SumExpr ret = Downcast<SumExpr>(std::move(a));
       ret.CopyOnWrite()->MulToSelf(bconst->value);
-      return std::move(ret);
+      return ret;
+
     } else {
       SplitExpr ret = ToSplitExpr(std::move(a));
       ret.CopyOnWrite()->MulToSelf(bconst->value);
-      return std::move(ret);
+      return ret;
     }
   }
 
@@ -921,7 +919,7 @@ bool CanonicalSimplifier::Impl::ProdDivSimplify(PrimExpr* plhs, PrimExpr* prhs,
       // try eliminate from lhs
       for (size_t i = 0; i < lhs_prods.size(); ++i) {
         if (lhs_prods[i].defined() && deep_equal(value, lhs_prods[i].value())) {
-          lhs_prods.Set(i, NullOpt);
+          lhs_prods.Set(i, std::nullopt);
           ++num_elimination;
           new_common_scale = new_common_scale * value;
           return;
@@ -972,7 +970,7 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const DivNode* op) {
       // can be divided by cval
       if (extra->IsZero()) {
         lhs.CopyOnWrite()->DivideBy(cval);
-        return std::move(lhs);
+        return lhs;
       }
       // both lhs and extra are non-negative
       if (analyzer_->CanProveGreaterEqual(lhs->Normalize(), 0) &&
@@ -987,7 +985,7 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const DivNode* op) {
             lhs.CopyOnWrite()->AddToSelf(SplitDivConst(ToSplitExpr(temp), cval, kTruncDiv), 1);
           }
         }
-        return std::move(lhs);
+        return lhs;
       }
     } else {
       // if a >= 0 && a < cval, then result == 0
@@ -1034,7 +1032,7 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const FloorDivNode* op) {
       SeparateDivisibleParts(psum, cval, &lhs, &extra);
       if (extra->IsZero()) {
         lhs.CopyOnWrite()->DivideBy(cval);
-        return std::move(lhs);
+        return lhs;
       }
       // continue simplification.
       lhs.CopyOnWrite()->DivideBy(cval);
@@ -1048,7 +1046,8 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const FloorDivNode* op) {
           lhs.CopyOnWrite()->AddToSelf(SplitDivConst(ToSplitExpr(temp), cval, kFloorDiv), 1);
         }
       }
-      return std::move(lhs);
+      return lhs;
+
     } else {
       // if a >= 0 && a < cval, then result == 0
       auto cbound = analyzer_->const_int_bound(Normalize(a));
@@ -1374,14 +1373,14 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const CastNode* op) {
     SumExpr se = Downcast<SumExpr>(value);
     if (se->CanPushCastToChildren(op->dtype, analyzer_)) {
       se.CopyOnWrite()->PushCastToChildren(op->dtype);
-      return std::move(se);
+      return se;
     }
   }
   if (value.as<SplitExprNode>()) {
     SplitExpr se = Downcast<SplitExpr>(value);
     if (se->CanPushCastToChildren(op->dtype, analyzer_)) {
       se.CopyOnWrite()->PushCastToChildren(op->dtype);
-      return std::move(se);
+      return se;
     }
   }
   return Rewriter::VisitExpr_(op);
@@ -1391,7 +1390,7 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const LTNode* op) {
   // First convert a < b into a - b < 0
   PrimExpr expr = this->CanonicalMutate(op->a - op->b);
   // Case: x0 * s0 + x1 * s1 + ... + xn + c < 0, let d = gcd(s0, s1, ..., s{n-1}, c)
-  // 1. if can prove -d < xn < d, then we can simplify
+  // 1. if can prove 0 <= xn < d, then we can simplify
   //    the expression to x0 * (s0/d) + x1 * (s1/d) + ... + x{n-1} * (s{n-1}/d) < c/d,
   //    e.g. `x * 8 + y < 16` where `y` \in [0, 8), we can simplify it to `x < 2`
   // 2. if xn is in pattern of yn % m, where m % d == 0, convert it to yn // d % (m/d)
@@ -1417,8 +1416,8 @@ PrimExpr CanonicalSimplifier::Impl::VisitExpr_(const LTNode* op) {
     ICHECK(extra->dtype == dtype);
     PrimExpr normal_extra = extra->Normalize();
     if (this->analyzer_->CanProve(normal_extra < make_const(dtype, gcd)) &&
-        this->analyzer_->CanProve(normal_extra > make_const(dtype, -gcd))) {
-      // Case 1. -d < xn < d
+        this->analyzer_->CanProve(normal_extra >= make_const(dtype, 0))) {
+      // Case 1. 0 <= xn < d
       divisible.CopyOnWrite()->DivideBy(gcd);
       return Rewriter::VisitExpr(divisible->Normalize() < make_zero(dtype));
     } else if (extra->args.size() == 1 &&
