@@ -31,23 +31,19 @@ namespace contrib {
 namespace msc {
 
 LiteralDoc PrototxtPrinter::ToLiteralDoc(const ObjectRef& obj) {
-  if (obj.as<StringObj>()) {
-    return LiteralDoc::Str(Downcast<String>(obj), NullOpt);
-  } else if (auto ptr = obj.as<runtime::Int::ContainerType>()) {
-    return LiteralDoc::Int(ptr->value, NullOpt);
-  } else if (auto ptr = obj.as<runtime::Float::ContainerType>()) {
-    return LiteralDoc::Float(ptr->value, NullOpt);
+  if (obj.as<ffi::StringObj>()) {
+    return LiteralDoc::Str(Downcast<String>(obj), std::nullopt);
   } else if (obj.as<IntImmNode>()) {
-    return LiteralDoc::Int(Downcast<IntImm>(obj)->value, NullOpt);
+    return LiteralDoc::Int(Downcast<IntImm>(obj)->value, std::nullopt);
   } else if (obj.as<FloatImmNode>()) {
-    return LiteralDoc::Float(Downcast<FloatImm>(obj)->value, NullOpt);
+    return LiteralDoc::Float(Downcast<FloatImm>(obj)->value, std::nullopt);
   }
   std::ostringstream obj_des;
   obj_des << obj;
-  return LiteralDoc::Str(obj_des.str(), NullOpt);
+  return LiteralDoc::Str(obj_des.str(), std::nullopt);
 }
 
-DictDoc PrototxtPrinter::ToDictDoc(const Map<String, ObjectRef>& dict) {
+DictDoc PrototxtPrinter::ToDictDoc(const Map<String, ffi::Any>& dict) {
   Array<ExprDoc> keys;
   Array<ExprDoc> values;
   for (const auto& pair : dict) {
@@ -55,13 +51,13 @@ DictDoc PrototxtPrinter::ToDictDoc(const Map<String, ObjectRef>& dict) {
     if (pair.second.as<DictDocNode>()) {
       values.push_back(Downcast<DictDoc>(pair.second));
     } else {
-      values.push_back(ToLiteralDoc(pair.second));
+      values.push_back(ToLiteralDoc(pair.second.cast<ObjectRef>()));
     }
   }
   return DictDoc(keys, values);
 }
 
-DictDoc PrototxtPrinter::ToDictDoc(const std::vector<std::pair<String, ObjectRef>>& dict) {
+DictDoc PrototxtPrinter::ToDictDoc(const std::vector<std::pair<String, Any>>& dict) {
   Array<ExprDoc> keys;
   Array<ExprDoc> values;
   for (const auto& pair : dict) {
@@ -69,24 +65,24 @@ DictDoc PrototxtPrinter::ToDictDoc(const std::vector<std::pair<String, ObjectRef
     if (pair.second.as<DictDocNode>()) {
       values.push_back(Downcast<DictDoc>(pair.second));
     } else {
-      values.push_back(ToLiteralDoc(pair.second));
+      values.push_back(ToLiteralDoc(pair.second.cast<ObjectRef>()));
     }
   }
   return DictDoc(keys, values);
 }
 
-void PrototxtPrinter::Append(const Map<String, ObjectRef>& dict) {
+void PrototxtPrinter::Append(const Map<String, ffi::Any>& dict) {
   DictDoc doc = ToDictDoc(dict);
   PrintDoc(doc, false);
 }
 
-void PrototxtPrinter::Append(const std::vector<std::pair<String, ObjectRef>>& dict) {
+void PrototxtPrinter::Append(const std::vector<std::pair<String, Any>>& dict) {
   DictDoc doc = ToDictDoc(dict);
   PrintDoc(doc, false);
 }
 
-void PrototxtPrinter::AppendPair(const String& key, const ObjectRef& value) {
-  Map<String, ObjectRef> dict;
+void PrototxtPrinter::AppendPair(const String& key, const ffi::Any& value) {
+  Map<String, ffi::Any> dict;
   dict.Set(key, value);
   return Append(dict);
 }
