@@ -266,7 +266,7 @@ InferLayoutOutput ForwardInferLayoutArgMaxMin(const Call& call,
   if (attrs->keepdims) {
     return InferLayoutOutput({input_layout}, {input_layout}, Attrs());
   }
-  if (!attrs->axis.defined()) {
+  if (!attrs->axis.has_value()) {
     return InferLayoutOutput({input_layout}, {LayoutDecision("")}, Attrs());
   }
   const auto& input_shape = ExprUtils::GetShape(call->args[0]);
@@ -274,7 +274,7 @@ InferLayoutOutput ForwardInferLayoutArgMaxMin(const Call& call,
     return InferLayoutOutput();
   }
   std::vector<size_t> axes;
-  axes.push_back(CommonUtils::GetIndex(Downcast<Integer>(attrs->axis)->value, input_shape.size()));
+  axes.push_back(CommonUtils::GetIndex(attrs->axis.value(), input_shape.size()));
   LayoutDecision output_layout = LayoutUtils::ReduceLayout(input_layout, axes);
   return InferLayoutOutput({input_layout}, {output_layout}, Attrs());
 }
@@ -514,12 +514,12 @@ InferLayoutOutput ForwardInferLayoutPlugin(const Call& call,
     return InferLayoutOutput();
   }
   const auto& name = Downcast<ExternFunc>(call->args[0])->global_symbol;
-  const auto* pf = runtime::Registry::Get("msc.plugin.op.InferLayout" + name);
-  if (pf == nullptr) {
+  const auto pf = tvm::ffi::Function::GetGlobal("msc.plugin.op.InferLayout" + name);
+  if (!pf.has_value()) {
     return InferLayoutOutput();
   }
   const auto& args = Downcast<Tuple>(call->args[1]);
-  return (*pf)(args->fields, var_layout_map);
+  return (*pf)(args->fields, var_layout_map).cast<InferLayoutOutput>();
 }
 
 // nn ops
@@ -1102,7 +1102,7 @@ class LayoutInfer : public ExprVisitor {
               BackwardInferLayoutCommon(call, Map<String, Array<String>>(), var_layout_map_);
         }
       } catch (runtime::InternalError& err) {
-        LOG(WARNING) << "Failed to backward infer layout " << expr << " : " << err.message();
+        LOG(WARNING) << "Failed to backward infer layout " << expr << " : " << err.what();
         infered_layout = InferLayoutOutput();
       }
       try {
@@ -1110,8 +1110,7 @@ class LayoutInfer : public ExprVisitor {
           SetInputLayouts(call, infered_layout->input_layouts);
         }
       } catch (runtime::InternalError& err) {
-        LOG(WARNING) << "Failed to backward set inputs layout for " << call << " : "
-                     << err.message();
+        LOG(WARNING) << "Failed to backward set inputs layout for " << call << " : " << err.what();
       }
     }
   }
@@ -1163,7 +1162,7 @@ class LayoutInfer : public ExprVisitor {
           }
         } catch (runtime::InternalError& err) {
           LOG(WARNING) << "Failed to forward infer layout for " << binding->var << " : "
-                       << binding->value << ", reason: " << err.message();
+                       << binding->value << ", reason: " << err.what();
           infered_layout = InferLayoutOutput();
         }
         if (infered_layout.defined() && infered_layout->output_layouts.size() == 1) {
@@ -1171,7 +1170,7 @@ class LayoutInfer : public ExprVisitor {
             SetExprLayout(binding->var, infered_layout->output_layouts[0]);
           } catch (runtime::InternalError& err) {
             LOG(WARNING) << "Failed to forward set output layout for " << binding->var << " : "
-                         << binding->value << ", reason: " << err.message();
+                         << binding->value << ", reason: " << err.what();
           }
         }
         if (set_inputs && infered_layout.defined()) {
@@ -1179,7 +1178,7 @@ class LayoutInfer : public ExprVisitor {
             SetInputLayouts(call, infered_layout->input_layouts);
           } catch (runtime::InternalError& err) {
             LOG(WARNING) << "Failed to forward set inputs layout for " << call << " : "
-                         << err.message();
+                         << err.what();
           }
         }
       }
@@ -1353,15 +1352,14 @@ void SetExprLayout(const IRModule& ref_module, const Expr& func, bool allow_miss
 namespace transform {
 
 Pass SetExprLayout(bool allow_missing, const String& entry_name) {
-  runtime::TypedPackedFunc<IRModule(IRModule, PassContext)> pass_func = [=](IRModule m,
-                                                                            PassContext pc) {
+  auto pass_func = [=](IRModule m, PassContext pc) {
     relax::SetExprLayout(m, m->Lookup(entry_name), allow_missing);
     return m;
   };
   return CreateModulePass(pass_func, 0, "SetExprLayout", {});
 }
 
-TVM_REGISTER_GLOBAL("relax.transform.SetExprLayout").set_body_typed(SetExprLayout);
+TVM_FFI_REGISTER_GLOBAL("relax.transform.SetExprLayout").set_body_typed(SetExprLayout);
 
 }  // namespace transform
 }  // namespace relax
