@@ -158,7 +158,7 @@ class RelaxExprNameSetter : public ExprVisitor {
 
   void VisitBinding_(const VarBindingNode* binding, const FunctionNode* val) {
     ExprVisitor::VisitBinding_(binding, val);
-    const auto& name_opt = val->GetAttr<runtime::String>(attr::kComposite);
+    const auto& name_opt = val->GetAttr<String>(attr::kComposite);
     if (name_opt.defined()) {
       local_funcs_.Set(binding->var, GetRef<Function>(val));
     }
@@ -209,8 +209,7 @@ class RelaxExprNameSetter : public ExprVisitor {
       try {
         input_types = ExprUtils::GetInputTypes(optype, val->args.size(), true);
       } catch (runtime::InternalError& err) {
-        LOG(WARNING) << "Failed to GetInputTypes for " << GetRef<Call>(val) << " : "
-                     << err.message();
+        LOG(WARNING) << "Failed to GetInputTypes for " << GetRef<Call>(val) << " : " << err.what();
         throw err;
       }
       for (size_t i = 0; i < input_types.size(); i++) {
@@ -258,8 +257,8 @@ class RelaxExprNameSetter : public ExprVisitor {
 
   const String GetFuncType(const Function& func) {
     String optype;
-    const auto& comp_opt = func->GetAttr<runtime::String>(attr::kComposite);
-    const auto& code_opt = func->GetAttr<runtime::String>(attr::kCodegen);
+    const auto& comp_opt = func->GetAttr<String>(attr::kComposite);
+    const auto& code_opt = func->GetAttr<String>(attr::kCodegen);
     if (comp_opt.defined()) {
       optype = comp_opt.value();
     } else if (code_opt.defined()) {
@@ -276,7 +275,7 @@ class RelaxExprNameSetter : public ExprVisitor {
   const String GetFuncName(const Call& call, const Function& func) {
     String name;
     // get from unique
-    const auto& name_opt = func->GetAttr<runtime::String>(msc_attr::kUnique);
+    const auto& name_opt = func->GetAttr<String>(msc_attr::kUnique);
     if (name_opt.defined()) {
       return name_opt.value();
     }
@@ -318,15 +317,14 @@ namespace transform {
 
 Pass SetRelaxExprName(const String& entry_name, const String& target,
                       const Map<String, String>& var_names) {
-  runtime::TypedPackedFunc<IRModule(IRModule, PassContext)> pass_func = [=](IRModule m,
-                                                                            PassContext pc) {
+  auto pass_func = [=](IRModule m, PassContext pc) {
     relax::SetRelaxExprName(m, m->Lookup(entry_name), target, var_names);
     return m;
   };
   return CreateModulePass(pass_func, 0, "SetRelaxExprName", {});
 }
 
-TVM_REGISTER_GLOBAL("relax.transform.SetRelaxExprName").set_body_typed(SetRelaxExprName);
+TVM_FFI_REGISTER_GLOBAL("relax.transform.SetRelaxExprName").set_body_typed(SetRelaxExprName);
 
 }  // namespace transform
 }  // namespace relax
