@@ -769,8 +769,8 @@ void TensorRTPluginCodeGen::CodegenCreator(const Plugin& plugin, bool dynamic, b
       stack_.call_arg(DocUtils::ToAttrAccess("meta_attr", a->name));
     }
     stack_.call_arg("layouts")
-        .func_call("setPluginNamespace", NullOpt, DocUtils::ToPtr("plugin"))
-        .inplace_start("c_str", NullOpt, DocUtils::ToDoc("name_space_"))
+        .func_call("setPluginNamespace", std::nullopt, DocUtils::ToPtr("plugin"))
+        .inplace_start("c_str", std::nullopt, DocUtils::ToDoc("name_space_"))
         .inplace_end()
         .func_end("plugin");
     // deserializePlugin
@@ -784,8 +784,8 @@ void TensorRTPluginCodeGen::CodegenCreator(const Plugin& plugin, bool dynamic, b
         .call_arg("name")
         .call_arg("data")
         .call_arg("length")
-        .func_call("setPluginNamespace", NullOpt, DocUtils::ToPtr("plugin"))
-        .inplace_start("c_str", NullOpt, DocUtils::ToDoc("name_space_"))
+        .func_call("setPluginNamespace", std::nullopt, DocUtils::ToPtr("plugin"))
+        .inplace_start("c_str", std::nullopt, DocUtils::ToDoc("name_space_"))
         .inplace_end()
         .func_end("plugin");
   }
@@ -883,7 +883,7 @@ void TensorRTPluginCodeGen::CodegenEnqueue(const Plugin& plugin, bool dynamic) {
   }
 }
 
-TVM_REGISTER_GLOBAL("msc.plugin.GetTensorRTPluginSources")
+TVM_FFI_REGISTER_GLOBAL("msc.plugin.GetTensorRTPluginSources")
     .set_body_typed([](const String& codegen_config, const String& print_config,
                        const String& codegen_type) -> Map<String, String> {
       TensorRTPluginCodeGen codegen = TensorRTPluginCodeGen(codegen_config);
