@@ -430,7 +430,7 @@ void TorchPluginCodeGen::CodeGenMalloc(const Plugin& plugin, const Array<PluginT
     int device_idx = plugin->FindDeviceRefIdx(tensors[i]);
     if (device_idx >= 0) {
       const auto& input_doc = DocUtils::ToIndex("input_tensors", device_idx);
-      stack_.inplace_start("device", NullOpt, input_doc).inplace_end();
+      stack_.inplace_start("device", std::nullopt, input_doc).inplace_end();
     } else {
       stack_.inplace_start("TorchUtils::ToTorchDevice")
           .call_arg(DocUtils::ToStr(tensors[i]->device))
@@ -492,7 +492,7 @@ void TorchPluginCodeGen::CodeGenCompute(const Plugin& plugin, const String& devi
   }
 }
 
-TVM_REGISTER_GLOBAL("msc.plugin.GetTorchPluginSources")
+TVM_FFI_REGISTER_GLOBAL("msc.plugin.GetTorchPluginSources")
     .set_body_typed([](const String& codegen_config, const String& print_config,
                        const String& codegen_type) -> Map<String, String> {
       TorchPluginCodeGen codegen = TorchPluginCodeGen(codegen_config);
