@@ -21,19 +21,23 @@
  * \file src/ir/function.cc
  * \brief The function data structure.
  */
+#include <tvm/ffi/function.h>
+#include <tvm/ffi/rvalue_ref.h>
 #include <tvm/ir/function.h>
 #include <tvm/relax/expr.h>
-#include <tvm/runtime/registry.h>
 #include <tvm/tir/function.h>
 
 namespace tvm {
 
-TVM_REGISTER_GLOBAL("ir.BaseFunc_Attrs").set_body_typed([](BaseFunc func) { return func->attrs; });
+TVM_FFI_REGISTER_GLOBAL("ir.BaseFunc_Attrs").set_body_typed([](BaseFunc func) {
+  return func->attrs;
+});
 
-TVM_REGISTER_GLOBAL("ir.BaseFuncCopy").set_body_typed([](BaseFunc func) { return func; });
+TVM_FFI_REGISTER_GLOBAL("ir.BaseFuncCopy").set_body_typed([](BaseFunc func) { return func; });
 
-TVM_REGISTER_GLOBAL("ir.BaseFuncWithAttr")
-    .set_body_typed([](BaseFunc func, String key, ObjectRef value) -> BaseFunc {
+TVM_FFI_REGISTER_GLOBAL("ir.BaseFuncWithAttr")
+    .set_body_typed([](ffi::RValueRef<BaseFunc> func_ref, String key, Any value) -> BaseFunc {
+      BaseFunc func = *std::move(func_ref);
       if (func->IsInstance<tir::PrimFuncNode>()) {
         return WithAttr(Downcast<tir::PrimFunc>(std::move(func)), key, value);
       } else if (func->IsInstance<relax::FunctionNode>()) {
@@ -43,28 +47,32 @@ TVM_REGISTER_GLOBAL("ir.BaseFuncWithAttr")
       }
     });
 
-TVM_REGISTER_GLOBAL("ir.BaseFuncWithAttrs")
-    .set_body_typed([](BaseFunc func, Map<String, ObjectRef> attr_map) -> BaseFunc {
+TVM_FFI_REGISTER_GLOBAL("ir.BaseFuncWithAttrs")
+    .set_body_typed([](ffi::RValueRef<BaseFunc> func_ref,
+                       Map<String, ffi::Any> attr_map) -> BaseFunc {
+      BaseFunc func = *std::move(func_ref);
       if (func->IsInstance<tir::PrimFuncNode>()) {
         return WithAttrs(Downcast<tir::PrimFunc>(std::move(func)), attr_map);
       }
-      if (const auto* f = runtime::Registry::Get("relax.FuncWithAttrs")) {
-        if (Optional<BaseFunc> ret = (*f)(func, attr_map)) {
+      if (const auto f = tvm::ffi::Function::GetGlobal("relax.FuncWithAttrs")) {
+        if (auto ret = (*f)(func, attr_map).cast<Optional<BaseFunc>>()) {
           return ret.value();
         }
       }
       LOG(FATAL) << "Do not support function type " << func->GetTypeKey();
+      TVM_FFI_UNREACHABLE();
     });
 
-TVM_REGISTER_GLOBAL("ir.BaseFuncWithoutAttr")
-    .set_body_typed([](BaseFunc func, String key) -> BaseFunc {
+TVM_FFI_REGISTER_GLOBAL("ir.BaseFuncWithoutAttr")
+    .set_body_typed([](ffi::RValueRef<BaseFunc> func_ref, String key) -> BaseFunc {
+      BaseFunc func = *std::move(func_ref);
       if (func->IsInstance<tir::PrimFuncNode>()) {
         return WithoutAttr(Downcast<tir::PrimFunc>(std::move(func)), key);
       } else if (func->IsInstance<relax::FunctionNode>()) {
         return WithoutAttr(Downcast<relax::Function>(std::move(func)), key);
       } else {
         LOG(FATAL) << "Do not support function type " << func->GetTypeKey();
-        return func;
+        TVM_FFI_UNREACHABLE();
       }
     });
 
