@@ -25,14 +25,14 @@ namespace meta_schedule {
 /******** Workload ********/
 
 Workload::Workload(IRModule mod) {
-  ObjectPtr<WorkloadNode> n = runtime::make_object<WorkloadNode>();
+  ObjectPtr<WorkloadNode> n = ffi::make_object<WorkloadNode>();
   n->mod = mod;
   n->shash = ModuleEquality::Create("structural")->Hash(mod);
   data_ = std::move(n);
 }
 
 Workload::Workload(IRModule mod, Workload::THashCode shash) {
-  ObjectPtr<WorkloadNode> n = runtime::make_object<WorkloadNode>();
+  ObjectPtr<WorkloadNode> n = ffi::make_object<WorkloadNode>();
   n->mod = mod;
   n->shash = shash;
   data_ = std::move(n);
@@ -51,15 +51,15 @@ Workload Workload::FromJSON(const ObjectRef& json_obj) {
   IRModule mod{nullptr};
   THashCode shash = 0;
   try {
-    const ArrayNode* json_array = json_obj.as<ArrayNode>();
+    const ffi::ArrayObj* json_array = json_obj.as<ffi::ArrayObj>();
     CHECK(json_array && json_array->size() == 2);
     // Load json[0] => shash
-    String str_shash = Downcast<String>(json_array->at(0));
+    String str_shash = json_array->at(0).cast<String>();
     // Load json[1] => mod
     {
-      String b64_mod = Downcast<String>(json_array->at(1));
+      String b64_mod = json_array->at(1).cast<String>();
       std::string json_mod = Base64Decode(b64_mod);
-      mod = Downcast<IRModule>(LoadJSON(json_mod));
+      mod = LoadJSON(json_mod).cast<IRModule>();
       std::stringstream(str_shash) >> shash;
     }
   } catch (const std::runtime_error& e) {  // includes tvm::Error and dmlc::Error
@@ -94,8 +94,8 @@ MeasureCandidate TuningRecordNode::AsMeasureCandidate() const {
 }
 
 ObjectRef TuningRecordNode::AsJSON() const {
-  Optional<Array<ObjectRef>> json_args_info{nullptr};
-  Optional<ObjectRef> json_target{nullptr};
+  Optional<Array<ObjectRef>> json_args_info;
+  Optional<ObjectRef> json_target;
   if (args_info.defined()) {
     Array<ObjectRef> info;
     info.reserve(args_info.value().size());
@@ -130,33 +130,33 @@ bool TuningRecordNode::IsValid() const {
 
 TuningRecord TuningRecord::FromJSON(const ObjectRef& json_obj, const Workload& workload) {
   tir::Trace trace{nullptr};
-  Optional<Array<FloatImm>> run_secs{nullptr};
-  Optional<Target> target{nullptr};
-  Optional<Array<ArgInfo>> args_info{nullptr};
+  Optional<Array<FloatImm>> run_secs;
+  Optional<Target> target;
+  Optional<Array<ArgInfo>> args_info;
   try {
-    const ArrayNode* json_array = json_obj.as<ArrayNode>();
+    const ffi::ArrayObj* json_array = json_obj.as<ffi::ArrayObj>();
     CHECK(json_array && json_array->size() == 4);
     // Load json[1] => run_secs
-    if (json_array->at(1).defined()) {
-      run_secs = AsFloatArray(json_array->at(1));
+    if (json_array->at(1) != nullptr) {
+      run_secs = AsFloatArray(json_array->at(1).cast<ObjectRef>());
     }
     // Load json[2] => target
-    if (json_array->at(2).defined()) {
-      target = Target(Downcast<Map<String, ObjectRef>>(json_array->at(2)));
+    if (json_array->at(2) != nullptr) {
+      target = Target(json_array->at(2).cast<Map<String, ffi::Any>>());
     }
     // Load json[3] => args_info
-    if (json_array->at(3).defined()) {
-      const ArrayNode* json_args_info = json_array->at(3).as<ArrayNode>();
+    if (json_array->at(3) != nullptr) {
+      const ffi::ArrayObj* json_args_info = json_array->at(3).cast<const ffi::ArrayObj*>();
       Array<ArgInfo> info;
       info.reserve(json_args_info->size());
-      for (const ObjectRef& json_arg_info : *json_args_info) {
-        info.push_back(ArgInfo::FromJSON(json_arg_info));
+      for (Any json_arg_info : *json_args_info) {
+        info.push_back(ArgInfo::FromJSON(json_arg_info.cast<ObjectRef>()));
       }
       args_info = info;
     }
     // Load json[0] => trace
     {
-      const ObjectRef& json_trace = json_array->at(0);
+      auto json_trace = json_array->at(0).cast<ObjectRef>();
       tir::Schedule sch =
           tir::Schedule::Traced(workload->mod, /*seed=*/-1, /*debug_mask=*/0,
                                 /*error_render_level=*/tir::ScheduleErrorRenderLevel::kNone);
@@ -177,11 +177,11 @@ DatabaseNode::~DatabaseNode() = default;
 Optional<TuningRecord> DatabaseNode::QueryTuningRecord(const IRModule& mod, const Target& target,
                                                        const String& workload_name) {
   if (!this->HasWorkload(mod)) {
-    return NullOpt;
+    return std::nullopt;
   }
   Array<TuningRecord> records = this->GetTopK(this->CommitWorkload(mod), 1);
   if (records.empty()) {
-    return NullOpt;
+    return std::nullopt;
   }
   ICHECK_EQ(records.size(), 1);
   return records[0];
@@ -197,7 +197,7 @@ Optional<tir::Schedule> DatabaseNode::QuerySchedule(const IRModule& mod, const T
     record->trace->ApplyToSchedule(sch, false);
     return sch;
   } else {
-    return NullOpt;
+    return std::nullopt;
   }
 }
 
@@ -206,7 +206,7 @@ Optional<IRModule> DatabaseNode::QueryIRModule(const IRModule& mod, const Target
   if (Optional<tir::Schedule> opt_sch = this->QuerySchedule(mod, target, workload_name)) {
     return opt_sch.value()->mod();
   } else {
-    return NullOpt;
+    return std::nullopt;
   }
 }
 
@@ -245,7 +245,7 @@ void Database::ExitWithScope() { ThreadLocalDatabases()->pop_back(); }
 Optional<Database> Database::Current() {
   std::vector<Database>* tls = ThreadLocalDatabases();
   if (tls->empty()) {
-    return NullOpt;
+    return std::nullopt;
   } else {
     return tls->back();
   }
@@ -278,51 +278,56 @@ Database Database::PyDatabase(PyDatabaseNode::FHasWorkload f_has_workload,
 
 /******** FFI ********/
 
+TVM_FFI_STATIC_INIT_BLOCK({
+  WorkloadNode::RegisterReflection();
+  TuningRecordNode::RegisterReflection();
+  PyDatabaseNode::RegisterReflection();
+});
+
 TVM_REGISTER_NODE_TYPE(WorkloadNode);
 TVM_REGISTER_NODE_TYPE(TuningRecordNode);
 TVM_REGISTER_OBJECT_TYPE(DatabaseNode);
 TVM_REGISTER_NODE_TYPE(PyDatabaseNode);
-TVM_REGISTER_GLOBAL("meta_schedule.Workload").set_body_typed([](IRModule mod) {
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.Workload").set_body_typed([](IRModule mod) {
   return Workload(mod);
 });
-TVM_REGISTER_GLOBAL("meta_schedule.WorkloadAsJSON")
-    .set_body_method<Workload>(&WorkloadNode::AsJSON);
-TVM_REGISTER_GLOBAL("meta_schedule.WorkloadFromJSON").set_body_typed(&Workload::FromJSON);
-TVM_REGISTER_GLOBAL("meta_schedule.TuningRecord")
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.WorkloadAsJSON").set_body_method(&WorkloadNode::AsJSON);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.WorkloadFromJSON").set_body_typed(&Workload::FromJSON);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecord")
     .set_body_typed([](tir::Trace trace, Workload workload, Optional<Array<FloatImm>> run_secs,
                        Optional<Target> target, Optional<Array<ArgInfo>> args_info) {
       return TuningRecord(trace, workload, run_secs, target, args_info);
     });
-TVM_REGISTER_GLOBAL("meta_schedule.TuningRecordAsMeasureCandidate")
-    .set_body_method<TuningRecord>(&TuningRecordNode::AsMeasureCandidate);
-TVM_REGISTER_GLOBAL("meta_schedule.TuningRecordAsJSON")
-    .set_body_method<TuningRecord>(&TuningRecordNode::AsJSON);
-TVM_REGISTER_GLOBAL("meta_schedule.TuningRecordFromJSON").set_body_typed(TuningRecord::FromJSON);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseEnterWithScope")
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecordAsMeasureCandidate")
+    .set_body_method(&TuningRecordNode::AsMeasureCandidate);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecordAsJSON")
+    .set_body_method(&TuningRecordNode::AsJSON);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecordFromJSON")
+    .set_body_typed(TuningRecord::FromJSON);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseEnterWithScope")
     .set_body_method(&Database::EnterWithScope);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseExitWithScope")
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseExitWithScope")
     .set_body_method(&Database::ExitWithScope);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseCurrent").set_body_typed(Database::Current);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseHasWorkload")
-    .set_body_method<Database>(&DatabaseNode::HasWorkload);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseCommitWorkload")
-    .set_body_method<Database>(&DatabaseNode::CommitWorkload);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseCommitTuningRecord")
-    .set_body_method<Database>(&DatabaseNode::CommitTuningRecord);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseGetTopK")
-    .set_body_method<Database>(&DatabaseNode::GetTopK);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseGetAllTuningRecords")
-    .set_body_method<Database>(&DatabaseNode::GetAllTuningRecords);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseSize").set_body_method<Database>(&DatabaseNode::Size);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseQueryTuningRecord")
-    .set_body_method<Database>(&DatabaseNode::QueryTuningRecord);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseQuerySchedule")
-    .set_body_method<Database>(&DatabaseNode::QuerySchedule);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseQueryIRModule")
-    .set_body_method<Database>(&DatabaseNode::QueryIRModule);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseDumpPruned")
-    .set_body_method<Database>(&DatabaseNode::DumpPruned);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabasePyDatabase").set_body_typed(Database::PyDatabase);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseCurrent").set_body_typed(Database::Current);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseHasWorkload")
+    .set_body_method(&DatabaseNode::HasWorkload);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseCommitWorkload")
+    .set_body_method(&DatabaseNode::CommitWorkload);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseCommitTuningRecord")
+    .set_body_method(&DatabaseNode::CommitTuningRecord);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseGetTopK").set_body_method(&DatabaseNode::GetTopK);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseGetAllTuningRecords")
+    .set_body_method(&DatabaseNode::GetAllTuningRecords);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseSize").set_body_method(&DatabaseNode::Size);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseQueryTuningRecord")
+    .set_body_method(&DatabaseNode::QueryTuningRecord);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseQuerySchedule")
+    .set_body_method(&DatabaseNode::QuerySchedule);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseQueryIRModule")
+    .set_body_method(&DatabaseNode::QueryIRModule);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseDumpPruned")
+    .set_body_method(&DatabaseNode::DumpPruned);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabasePyDatabase").set_body_typed(Database::PyDatabase);
 
 }  // namespace meta_schedule
 }  // namespace tvm
