@@ -16,6 +16,8 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+#include <tvm/ffi/reflection/reflection.h>
+
 #include <set>
 #include <thread>
 #include <unordered_map>
@@ -33,7 +35,7 @@ namespace meta_schedule {
  * \param allow_missing Whether to create new file when the given path is not found.
  * \return An array containing lines read from the json file.
  */
-std::vector<ObjectRef> JSONFileReadLines(const String& path, int num_threads, bool allow_missing) {
+std::vector<Any> JSONFileReadLines(const String& path, int num_threads, bool allow_missing) {
   std::ifstream is(path);
   if (is.good()) {
     std::vector<String> json_strs;
@@ -41,7 +43,7 @@ std::vector<ObjectRef> JSONFileReadLines(const String& path, int num_threads, bo
       json_strs.push_back(str);
     }
     int n = json_strs.size();
-    std::vector<ObjectRef> json_objs;
+    std::vector<Any> json_objs;
     json_objs.resize(n);
     support::parallel_for_dynamic(0, n, num_threads, [&](int thread_id, int task_id) {
       json_objs[task_id] = JSONLoads(json_strs[task_id]);
@@ -81,11 +83,11 @@ class JSONDatabaseNode : public DatabaseNode {
   /*! \brief All the tuning records in the database */
   std::multiset<TuningRecord, SortTuningRecordByMeanRunSecs> tuning_records_;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("path_workload", &path_workload);
-    v->Visit("path_tuning_record", &path_tuning_record);
-    // `workloads2idx_` is not visited
-    // `tuning_records_` is not visited
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<JSONDatabaseNode>()
+        .def_ro("path_workload", &JSONDatabaseNode::path_workload)
+        .def_ro("path_tuning_record", &JSONDatabaseNode::path_tuning_record);
   }
 
   static constexpr const char* _type_key = "meta_schedule.JSONDatabase";
@@ -113,7 +115,7 @@ class JSONDatabaseNode : public DatabaseNode {
   void CommitTuningRecord(const TuningRecord& record) {
     this->tuning_records_.insert(record);
     JSONFileAppendLine(this->path_tuning_record,
-                       JSONDumps(Array<ObjectRef>{
+                       JSONDumps(Array<Any>{
                            /*workload_index=*/Integer(this->workloads2idx_.at(record->workload)),
                            /*tuning_record=*/record->AsJSON()  //
                        }));
@@ -161,12 +163,12 @@ Database Database::JSONDatabase(String path_workload, String path_tuning_record,
   // Load `n->workloads2idx_` from `path_workload`
   std::vector<Workload> workloads;
   {
-    std::vector<ObjectRef> json_objs = JSONFileReadLines(path_workload, num_threads, allow_missing);
+    std::vector<Any> json_objs = JSONFileReadLines(path_workload, num_threads, allow_missing);
     int n_objs = json_objs.size();
     n->workloads2idx_.reserve(n_objs);
     workloads.reserve(n_objs);
     for (int i = 0; i < n_objs; ++i) {
-      Workload workload = Workload::FromJSON(json_objs[i]);
+      Workload workload = Workload::FromJSON(json_objs[i].cast<ObjectRef>());
       auto recalc_hash = n->GetModuleEquality().Hash(workload->mod);
       // Todo(tvm-team): re-enable the shash check when we get environment
       // independent structural hash values.
@@ -181,21 +183,20 @@ Database Database::JSONDatabase(String path_workload, String path_tuning_record,
   }
   // Load `n->tuning_records_` from `path_tuning_record`
   {
-    std::vector<ObjectRef> json_objs =
-        JSONFileReadLines(path_tuning_record, num_threads, allow_missing);
+    std::vector<Any> json_objs = JSONFileReadLines(path_tuning_record, num_threads, allow_missing);
     std::vector<TuningRecord> records;
     records.resize(json_objs.size(), TuningRecord{nullptr});
     support::parallel_for_dynamic(
         0, json_objs.size(), num_threads, [&](int thread_id, int task_id) {
-          const ObjectRef& json_obj = json_objs[task_id];
+          auto json_obj = json_objs[task_id].cast<ObjectRef>();
           Workload workload{nullptr};
           try {
-            const ArrayNode* arr = json_obj.as<ArrayNode>();
+            const ffi::ArrayObj* arr = json_obj.as<ffi::ArrayObj>();
             ICHECK_EQ(arr->size(), 2);
-            int64_t workload_index = Downcast<runtime::Int>(arr->at(0));
+            int64_t workload_index = arr->at(0).cast<IntImm>()->value;
             ICHECK(workload_index >= 0 && static_cast<size_t>(workload_index) < workloads.size());
             workload = workloads[workload_index];
-            records[task_id] = TuningRecord::FromJSON(arr->at(1), workload);
+            records[task_id] = TuningRecord::FromJSON(arr->at(1).cast<ObjectRef>(), workload);
           } catch (std::runtime_error& e) {
             LOG(FATAL) << "ValueError: Unable to parse TuningRecord, on line " << (task_id + 1)
                        << " of file " << path_tuning_record << ". The workload is:\n"
@@ -214,8 +215,10 @@ Database Database::JSONDatabase(String path_workload, String path_tuning_record,
   return Database(n);
 }
 
+TVM_FFI_STATIC_INIT_BLOCK({ JSONDatabaseNode::RegisterReflection(); });
 TVM_REGISTER_NODE_TYPE(JSONDatabaseNode);
-TVM_REGISTER_GLOBAL("meta_schedule.DatabaseJSONDatabase").set_body_typed(Database::JSONDatabase);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseJSONDatabase")
+    .set_body_typed(Database::JSONDatabase);
 
 }  // namespace meta_schedule
 }  // namespace tvm
