@@ -16,6 +16,7 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+#include <tvm/ffi/reflection/reflection.h>
 #include <tvm/tir/transform.h>
 
 #include <cmath>
@@ -279,6 +280,9 @@ Pass SimplifyForFeatureExtraction() {
     }
 
     Stmt VisitStmt_(const ForNode* loop) final {
+      if (is_zero(loop->extent)) {
+        return Evaluate(0);
+      }
       if (is_zero(loop->min) && is_one(loop->extent) && loop->kind == ForKind::kSerial &&
           loop->annotations.empty()) {
         unit_vars_.insert(loop->loop_var);
@@ -1364,11 +1368,15 @@ class PerStoreFeatureNode : public FeatureExtractorNode {
   bool extract_workload;
   int feature_vector_length;
 
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("buffers_per_store", &buffers_per_store);
-    v->Visit("arith_intensity_curve_num_samples", &arith_intensity_curve_num_samples);
-    v->Visit("cache_line_bytes", &cache_line_bytes);
-    v->Visit("feature_vector_length", &feature_vector_length);
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<PerStoreFeatureNode>()
+        .def_ro("buffers_per_store", &PerStoreFeatureNode::buffers_per_store)
+        .def_ro("arith_intensity_curve_num_samples",
+                &PerStoreFeatureNode::arith_intensity_curve_num_samples)
+        .def_ro("cache_line_bytes", &PerStoreFeatureNode::cache_line_bytes)
+        .def_ro("extract_workload", &PerStoreFeatureNode::extract_workload)
+        .def_ro("feature_vector_length", &PerStoreFeatureNode::feature_vector_length);
   }
 
   void ExtractSingle(IRModule mod, bool is_gpu, std::vector<std::vector<double>>* results) {
@@ -1438,8 +1446,10 @@ FeatureExtractor FeatureExtractor::PerStoreFeature(int buffers_per_store,
   return FeatureExtractor(n);
 }
 
+TVM_FFI_STATIC_INIT_BLOCK({ PerStoreFeatureNode::RegisterReflection(); });
+
 TVM_REGISTER_NODE_TYPE(PerStoreFeatureNode);
-TVM_REGISTER_GLOBAL("meta_schedule.FeatureExtractorPerStoreFeature")
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.FeatureExtractorPerStoreFeature")
     .set_body_typed(FeatureExtractor::PerStoreFeature);
 
 }  // namespace meta_schedule
