@@ -27,12 +27,11 @@ class RemoveBuildArtifactNode : public MeasureCallbackNode {
              const Array<MeasureCandidate>& measure_candidates,
              const Array<BuilderResult>& builder_results,
              const Array<RunnerResult>& runner_results) final {
-    static const PackedFunc* f_rm = runtime::Registry::Get("meta_schedule.remove_build_dir");
-    ICHECK(f_rm != nullptr) << "The `remove_build_dir` func is not in tvm registry.";
+    static auto f_rm = tvm::ffi::Function::GetGlobalRequired("meta_schedule.remove_build_dir");
     auto _ = Profiler::TimedScope("MeasureCallback/RemoveBuildArtifact");
     for (const BuilderResult& build_result : builder_results) {
       if (Optional<String> path = build_result->artifact_path) {
-        (*f_rm)(path.value());
+        f_rm(path.value());
       }
     }
   }
@@ -47,7 +46,7 @@ MeasureCallback MeasureCallback::RemoveBuildArtifact() {
 }
 
 TVM_REGISTER_NODE_TYPE(RemoveBuildArtifactNode);
-TVM_REGISTER_GLOBAL("meta_schedule.MeasureCallbackRemoveBuildArtifact")
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.MeasureCallbackRemoveBuildArtifact")
     .set_body_typed(MeasureCallback::RemoveBuildArtifact);
 
 }  // namespace meta_schedule
