@@ -105,8 +105,8 @@ class LayoutFreeBufferCollector : public StmtVisitor {
  public:
   void VisitStmt_(const BlockNode* block) final {
     StmtVisitor::VisitStmt_(block);
-    if (Optional<ObjectRef> ann = block->annotations.Get("layout_free_placeholders")) {
-      for (Buffer buffer : Downcast<Array<Buffer>>(ann)) {
+    if (auto ann = block->annotations.Get("layout_free_placeholders")) {
+      for (Buffer buffer : Downcast<Array<Buffer>>(ann.value())) {
         buffers.insert(buffer);
       }
     }
@@ -183,7 +183,7 @@ bool RewriteLayout(const Schedule& sch) {
   std::vector<std::pair<StmtSRef, String>> results;
   auto add_layout_rewrite_block = [&sch](BlockRV consumer_block_rv, int buffer_index) {
     BlockRV rewrite_block_rv = sch->CacheRead(consumer_block_rv, buffer_index, "global");
-    sch->Annotate(rewrite_block_rv, attr::meta_schedule_layout_rewrite_preproc, const_true());
+    sch->Annotate(rewrite_block_rv, attr::meta_schedule_layout_rewrite_preproc, true);
   };
 
   for (const auto& [g_var, base_func] : sch->mod()->functions) {
@@ -206,7 +206,7 @@ bool RewriteLayout(const Schedule& sch) {
         auto anchor_block_rv = sch->GetBlock(anchor_block->name_hint, func_name);
         add_layout_rewrite_block(anchor_block_rv, buffer_index);
         sch->TransformLayout(anchor_block_rv, buffer_index, BufferIndexType::kRead, index_map,
-                             NullOpt);
+                             std::nullopt);
       } else {
         // When the layout-free buffer is consumed by cache_read, we need to find the index map
         // for a cache-read buffer that is directly consumed by an anchor op. The last buffer
@@ -219,7 +219,7 @@ bool RewriteLayout(const Schedule& sch) {
         auto [anchor_block, buffer_index, index_map] = *tup_opt;
         // Transform the layout of the last cache-read buffer.
         sch->TransformLayout(sch->GetBlock(anchor_block->name_hint, func_name), buffer_index,
-                             BufferIndexType::kRead, index_map, NullOpt);
+                             BufferIndexType::kRead, index_map, std::nullopt);
 
         // Propagate the layout transformation over cache_read_chain, starting from
         // the next-to-last cache-read buffer.
@@ -231,7 +231,8 @@ bool RewriteLayout(const Schedule& sch) {
             // transformed by TransformLayout below.
             add_layout_rewrite_block(cache_read_block_rv, 0);
           }
-          sch->TransformLayout(cache_read_block_rv, 0, BufferIndexType::kRead, index_map, NullOpt);
+          sch->TransformLayout(cache_read_block_rv, 0, BufferIndexType::kRead, index_map,
+                               std::nullopt);
         }
       }
     }
@@ -272,7 +273,8 @@ Postproc Postproc::RewriteLayout() {
 }
 
 TVM_REGISTER_NODE_TYPE(RewriteLayoutNode);
-TVM_REGISTER_GLOBAL("meta_schedule.PostprocRewriteLayout").set_body_typed(Postproc::RewriteLayout);
+TVM_FFI_REGISTER_GLOBAL("meta_schedule.PostprocRewriteLayout")
+    .set_body_typed(Postproc::RewriteLayout);
 
 }  // namespace meta_schedule
 }  // namespace tvm
